/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <dirent.h>
#include <ctype.h>
#include <stdio.h>
#include <gefax/gefax.h>
#include <gal/e-paned/e-hpaned.h>

#include "control.h"
#include "gefax.h"
#include "address_b.h"
#include "support.h"
#include "appbar.h"

typedef struct _AddressBook_Config		AddressBook_Config;
typedef struct _View_Text       		View_Text;
typedef struct _View_Toolbar			View_Toolbar;
typedef struct _Search_Toolbar			Search_Toolbar;
typedef struct _Main_Toolbar			Main_Toolbar;
typedef struct _Address_Book			Address_Book;
typedef struct _Select_Toolbar			Select_Toolbar;

struct _AddressBook_Config {
	gint		paned_pos;
	gint		clist_pos[4];
	gint		option_filter_pos;
	gchar 		*file_path;		/*path to current vcard file*/
	gchar		filter;
	gboolean	search_toolbar;
	gboolean	filter_toolbar;
};

struct _View_Text {
	GdkFont		*fixed_font;
	GdkFont		*header_font;
	GdkFont		*section_font;
	GdkColormap	*cmap;
	GdkColor	bgcolor;
	GdkColor	scolor;
	GdkColor	tcolor;
	GtkWidget	*sw_text;
	GtkWidget	*text;
};

struct _View_Toolbar {
	GtkWidget *toolbar_view;
	GtkWidget *option_filter;
	GtkWidget *option_filter_menu;
};

struct _Search_Toolbar {
	GtkWidget		*toolbar_search;
	GtkWidget		*tog_case_sens;
	GtkWidget		*entry_search;
	LibGefaxSearch		search;
	LibGefaxSearchType	search_type;
};

struct _Main_Toolbar { 
	GtkWidget	*toolbar;
	GtkWidget	*tmp_toolbar_icon;
	GtkWidget	*btn_open;
	GtkWidget	*btn_help;
	GtkWidget	*btn_close;
	GtkWidget	*btn_save;
	GtkWidget	*btn_save_as;
	GtkWidget	*btn_addcard;
	GtkWidget	*btn_editcard;
	GtkWidget	*btn_delete;
};

struct _Select_Toolbar {
	GtkWidget	*sw_select;
	GtkWidget	*v_port_select;
	GtkWidget	*toolbar;
	GtkWidget	*last_togglebutton;
};

struct _Address_Book {
	GtkWidget	*vbox_contacts;
	GtkWidget	*hp_address;
	GtkWidget	*sw_address;
	GtkWidget	*clist_address;
	GtkWidget	*sort_h1;
	GtkWidget	*sort_h2;
	GtkWidget	*sort_h3;
	GtkWidget	*sort_h4;
	GtkWidget	*sw_text;
	GtkWidget	*ab_pop_menu;
	View_Toolbar	ViewToolbar;
	Search_Toolbar	SearchToolbar;
	Main_Toolbar	MainToolbar;
	Select_Toolbar	SelectToolbar;
	View_Text	ViewText;
	gint		selected_contact_key;
	gint		selected_row;
	gchar		filter_char[2];
	gint		clist_length;
};

static AddressBook_Config AddressBookConfig;
static Address_Book AddressBook;
static GList *AddressList = NULL;
static GList *ActiveAddressList = NULL;

static GnomeHelpMenuEntry help_ref = {
	"gefax",
	"contacts.html#top"
};

static void
address_book_filter_address_book		(void);
static void
address_book_filter_address_book_call		(GtkWidget			*widget,
						gpointer			data);
static void
address_book_search_callback			(GtkWidget			*widget,
						Search_Toolbar			*SearchToolbar);
static void
address_book_save_AddressList_as_callback	(GtkWidget			*w,
						GtkFileSelection		*fs);
static void
address_book_free_data				(void);
static void
address_book_save_AddressList_as		(GtkWidget			*widget,
						gpointer			data);
static void
address_book_add_contact			(GtkWidget			*widget,
						gpointer			data);
static void
address_book_edit_contact			(GtkWidget			*widget,
						gpointer			data);
static void
address_book_remove_contact			(GtkWidget			*widget,
						gpointer			data);

static void
addressbook_changed				(LibGefaxContactDialog		*contactdialog,
						GList				*ABook,
						gint				key,
						gpointer			user_data);

static void
address_book_load_file				(gchar				*file_path);
static void
address_book_save_file				(GtkWidget			*widget,
						gchar				*file_path);
static void
address_book_change_AddressList			(GtkButton			*b,
						gpointer			data);

static void
file_change_AddressList_callback		(GtkWidget			*w,
						GtkFileSelection		*fs);
static void
address_book_clist_unselect_row			(GtkCList			*clist,
						gint				row,
						gint				column,
						GdkEventButton			*event,
						gpointer			data);
static void
address_book_clist_select_row			(GtkCList			*clist,
						gint				row,
						gint				column,
						GdkEventButton			*event,
						gpointer			data);
static gint
on_clist_address_event				(GtkWidget			*w,
						GdkEvent			*event,
						gpointer			data);
static void
on_clist_address_sorted				(GtkWidget			*widget,
						gpointer			data);
static void
on_clist_address_resize_column			(GtkCList			*clist,
						gint				col,
						gint				width,
						gpointer			data);

static void
address_book_show_help				(GtkButton			*b,
						gpointer			data);

static GtkWidget
*my_gtk_toolbar_append_togglebutton		(GtkWidget			*toolbar,
						gchar				*label,
						gint				width,
						gint				height,
						GtkSignalFunc			callback,
						gpointer			user_data);

static void
create_ViewText					(View_Text			*ViewText);

static void
create_MainToolbar				(Main_Toolbar			*MainToolbar);
static void
address_book_view_search_toolbar		(GtkMenuItem			*tb,
						gpointer			data);
static void
address_book_view_filter_toolbar		(GtkMenuItem			*tb,
						gpointer			data);
static void
address_book_search_field_altered_call		(GtkWidget			*widget,
						gpointer			index);
static void
address_book_search_place_altered_call		(GtkWidget			*widget,
						gpointer			index);
static void
create_SearchToolbar				(Search_Toolbar			*SearchToolbar);
static void
create_SelectToolbar				(Select_Toolbar			*SelectToolbar);

static void
on_SelectToolbar_btn_toggled			(GtkToggleButton		*B,
						gpointer			letter);
static void
address_book_view_all_contacts			(void);
static void
address_book_view_select_contacts		(const gchar		filter);
static void
address_book_view_contacts			(void);
static void
view_ViewText_subitem				(const gchar		*data,
						const gchar		*header);
static void
view_ViewText_address				(LibGefaxAddress	*A);
static void
view_ViewText_telephone				(LibGefaxTelephone	*T);
static void
view_ViewText_email				(LibGefaxEmail		*E);
static void
diplay_contact_in_view_text			(gint			key);
static GnomeUIInfo
ab_edits_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Add Contact"), N_("Add a new contact"),
		address_book_add_contact, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Edit Contact"), N_("Edit selected contact"),
		address_book_edit_contact, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXEC, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Delete Contact"), N_("Delete selected contact"),
		address_book_remove_contact, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH, 0, 0, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo
ab_views_menu[] =
{
	{
		GNOME_APP_UI_TOGGLEITEM, N_("_Search toolbar"), N_("View address book search toolbar"),
		address_book_view_search_toolbar, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN, 0, 0, NULL
	},
	{
		GNOME_APP_UI_TOGGLEITEM, N_("_Filter toolbar"), N_("View address book filter toolbar"),
		address_book_view_filter_toolbar, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 0, 0, NULL
	},
	GNOMEUIINFO_END
};
static GnomeUIInfo
ab_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Open Address Book"), N_("Open alternate address book"),
		address_book_change_AddressList, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Save"), N_("Save address book"),
		address_book_save_file, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Save As"), N_("Save address book as a different file"),
		address_book_save_AddressList_as, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS, 0, 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	{
		GNOME_APP_UI_ITEM, N_("_Close Contacts"), N_("Close contacts and return to fax"),
		change_to_browser, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 0, 0, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo
ab_edit_menu[] =
{
	GNOMEUIINFO_SUBTREE (N_("_Edit"), ab_edits_menu),
	GNOMEUIINFO_END
};

static GnomeUIInfo
ab_view_menu[] =
{
	GNOMEUIINFO_SUBTREE (N_("_View"), ab_views_menu),
	GNOMEUIINFO_END
};

static GnomeUIInfo
ab_popup_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Add Contact"), N_("Add a new contact"),
		address_book_add_contact, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Edit Contact"), N_("Edit selected contact"),
		address_book_edit_contact, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXEC, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Delete Contact"), N_("Delete selected contact"),
		address_book_remove_contact, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH, 0, 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	{
		GNOME_APP_UI_ITEM, N_("_Close Contacts"), N_("Close contacts and return to fax"),
		change_to_browser, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 0, 0, NULL
	},
	GNOMEUIINFO_END
};

static void
address_book_add_contact			(GtkWidget			*widget,
						gpointer			data)
{
	GtkWidget *pb = NULL;
	pb = lib_gefax_contact_dialog_new (AddressList, LIBGEFAX_CONTACT_DIALOG_NEW_CONTACT);
	if (pb) {
		gtk_widget_show (pb);
		gtk_signal_connect (GTK_OBJECT (pb), "addressbook_changed",
			GTK_SIGNAL_FUNC (addressbook_changed), NULL);
	}
}

static void
address_book_edit_contact			(GtkWidget			*widget,
						gpointer			data)
{
	if (AddressBook.selected_contact_key > 0) {
		GtkWidget *pb = NULL;
		pb = lib_gefax_contact_dialog_new (AddressList, AddressBook.selected_contact_key);
		if (pb) {
			gtk_widget_show (pb);
			gtk_signal_connect (GTK_OBJECT (pb), "addressbook_changed",
				GTK_SIGNAL_FUNC (addressbook_changed), NULL);
		}
	}
}

static void
address_book_remove_contact			(GtkWidget			*widget,
						gpointer			data)
{
	if (AddressBook.selected_contact_key > 0) {
		LibGefaxContact *C = lib_gefax_contact_get_by_key (AddressList, AddressBook.selected_contact_key);
		AddressList = lib_gefax_remove_contact (AddressList, C);
	}
	address_book_filter_address_book ();
	address_book_view_contacts ();
}

static void
address_book_filter_address_book		(void)
{
	
	if (ActiveAddressList) {
		g_list_free (ActiveAddressList);
		ActiveAddressList = NULL;
	}
	switch (AddressBookConfig.option_filter_pos) {
		case 1:
			ActiveAddressList = lib_gefax_filter_address_book (AddressList, LIBGEFAX_FILTER_FAX, FALSE);
			break;
		case 2:
			ActiveAddressList = lib_gefax_filter_address_book (AddressList, LIBGEFAX_FILTER_PHONE, FALSE);
			break;
		case 3:
			ActiveAddressList = lib_gefax_filter_address_book (AddressList, LIBGEFAX_FILTER_ADDRESS, FALSE);
			break;
		case 4:
			ActiveAddressList = lib_gefax_filter_address_book (AddressList, LIBGEFAX_FILTER_EMAIL, FALSE);
			break;
		case 5:
			ActiveAddressList = lib_gefax_filter_address_book (AddressList, LIBGEFAX_FILTER_URL, FALSE);
			break;
		default:
			ActiveAddressList = g_list_copy (AddressList);
			break;
	}
}

static void
address_book_search_callback			(GtkWidget			*widget,
						Search_Toolbar			*SearchToolbar)
{
	gint i;
	gchar *text;
	gboolean found = FALSE;
	
	text = gtk_entry_get_text (GTK_ENTRY (SearchToolbar->entry_search));
	if (text[0] == '\0') {
		return;
	}
	if (AddressBook.selected_row != -1) {
		i = AddressBook.selected_row;
		++i;
	} else {
		i = 0;
	}
	while (i < AddressBook.clist_length) {
		gint key;
		gchar *key_char;
		LibGefaxContact *C;

		gtk_clist_get_text (GTK_CLIST (AddressBook.clist_address), i, 0, &key_char);
		key = atoi (key_char);
		C = lib_gefax_contact_get_by_key (ActiveAddressList, key);
		found = lib_gefax_search_contact (C, text, SearchToolbar->search, SearchToolbar->search_type,
			GTK_TOGGLE_BUTTON (SearchToolbar->tog_case_sens)->active);
		if (found) {
			gtk_clist_select_row (GTK_CLIST (AddressBook.clist_address), i, 0);
			break;
		}
		++i;		
	}
	if (!found) {
		gnome_ok_dialog (_("Search text not found"));
	}
}

static void
addressbook_changed				(LibGefaxContactDialog		*contactdialog,
						GList				*ABook,
						gint				key,
						gpointer			user_data)
{
		AddressList = ABook;
		address_book_filter_address_book ();
		address_book_view_contacts ();
}

static void
address_book_load_file				(gchar				*file_path)
{
	if (AddressList) {
		address_book_free_data ();
	}
	AddressList = lib_gefax_read_file (file_path);
}

void
address_book_save_file				(GtkWidget			*widget,
						gchar				*file_path)
{
	gchar *path;
	gboolean new_file;
	
	if (file_path) {
		path = file_path;
		new_file = TRUE;
	} else {
		path = AddressBookConfig.file_path;
		new_file = FALSE;
	}
	if (lib_gefax_save_file (AddressList, path, new_file, LIBGEFAX_FILETYPE_VCARD)) {
		gnome_warning_dialog (_("Unable to save Address Book.\nCheck that you have the necessary file permissions"));
	}else {
		static_status_msg ("Saved Address Book");
	}
}

void
address_book_save_AddressList_as_callback	(GtkWidget			*w,
						GtkFileSelection		*fs)
{
	gchar *fn;
	GtkWidget *check;
	
	fn = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
	
	/*check if the file exists*/
	if (g_file_test (fn, G_FILE_TEST_ISFILE)) {	
		check = gnome_message_box_new (_("File exists\nOverwrite?"),
						GNOME_MESSAGE_BOX_QUESTION,
						GNOME_STOCK_BUTTON_OK,
						GNOME_STOCK_BUTTON_CANCEL,
						NULL);
		gtk_window_set_position (GTK_WINDOW (check), GTK_WIN_POS_MOUSE);
		if ((gnome_dialog_run_and_close (GNOME_DIALOG (check))) == 0) {
			address_book_save_file (NULL, fn);
			g_free (AddressBookConfig.file_path);
			AddressBookConfig.file_path = fn;
			gtk_object_destroy(GTK_OBJECT (fs));
			return;
		}
	}else{
		address_book_save_file (NULL, fn);
		g_free (AddressBookConfig.file_path);
		AddressBookConfig.file_path = fn;
		gtk_object_destroy (GTK_OBJECT (fs));
		return;
	}
	g_free (fn);
	return;
}

void
address_book_save_AddressList_as		(GtkWidget			*widget,
						gpointer			data)
{
	GtkWidget *fs;
	gchar *fixed_home;
	
	/*open file selector*/
	fs = gtk_file_selection_new (_("gefax - Save Address Book as..."));
	gtk_window_set_position (GTK_WINDOW (fs), GTK_WIN_POS_CENTER);
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button), "clicked",
					(GtkSignalFunc) address_book_save_AddressList_as_callback, fs );
    
	/* Connect the cancel_button to destroy the widget */
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button), "clicked",
    						(GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (fs));
    
	fixed_home = g_strdup_printf ("%s/",home);
	gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), fixed_home);
	g_free (fixed_home);
    
	gtk_widget_show(fs);
	
}

void
file_change_AddressList_callback		(GtkWidget			*w,
						GtkFileSelection		*fs)
{
	gchar *path;
	
	path = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
	address_book_load_file (path);
	gtk_object_destroy (GTK_OBJECT (fs));
	
	/*resort to old if error*/
	if (!AddressList){
		address_book_load_file (AddressBookConfig.file_path);
		gnome_error_dialog (_("Unable to read the selected file\nThe file may be corrupt, not a valid VCard file or,\na bug report may be in order"));
	} else {
		g_free (AddressBookConfig.file_path);
		AddressBookConfig.file_path = path;
	}
	/*Refresh the list*/
	address_book_filter_address_book ();
	address_book_view_contacts ();
	/*altered = FALSE;
	gtk_widget_set_sensitive (btn_save, altered);
	gtk_widget_set_sensitive (ab_menu[1].widget, altered);*/
	
	
}

void
address_book_change_AddressList			(GtkButton			*b,
						gpointer			data)
{
	GtkWidget *fs;
	
	/*open file selector*/
	fs = gtk_file_selection_new(_("gefax - Select vcard file"));
	gtk_window_set_position (GTK_WINDOW (fs), GTK_WIN_POS_CENTER);
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button), "clicked",
					(GtkSignalFunc) file_change_AddressList_callback, fs );
    
	/* Connect the cancel_button to destroy the widget */
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button), "clicked",
    						(GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (fs));
    
	if (AddressBookConfig.file_path) gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), AddressBookConfig.file_path);
    
	gtk_widget_show (fs);
	
}

static void
address_book_free_data				(void)
{
	lib_gefax_free_address_book (AddressList);
	AddressList = NULL;
	if (ActiveAddressList) {
		g_list_free (ActiveAddressList);
		ActiveAddressList = NULL;
	}
}



GList
*address_book_get_AddressList			(void)
{
	return g_list_copy (AddressList);
}

static void
address_book_show_help				(GtkButton		*b,
						gpointer		data)
{
	gnome_help_display ("ignored", &help_ref);
}

static void
address_book_view_all_contacts			(void)
{
	GList *AddrL = ActiveAddressList;
	gint i;

	if (!AddrL) {
		gtk_clist_clear (GTK_CLIST (AddressBook.clist_address));
		return;
	}
	if (AddrL) {
		gtk_clist_freeze (GTK_CLIST (AddressBook.clist_address));
		gtk_clist_clear (GTK_CLIST (AddressBook.clist_address));
		AddressBook.clist_length = g_list_length (AddrL);
		for (i = 0; i < AddressBook.clist_length; i++) {
			gchar *EMPTY = " ";
			gchar *list[5];
			LibGefaxContact *C = LIB_GEFAX_CONTACT (AddrL);
			LibGefaxIdentity *I = C->Identity;
			LibGefaxBusiness *B = C->Business;
			
			list[0] = g_strdup_printf("%d",C->Key);
			if (I->prefix) {
				list[1] = I->prefix;
			} else {
				list[1] = EMPTY;
			}
			if (I->given) {
				list[2] = I->given;
			} else {
				list[2] = EMPTY;
			}
			if (I->family) {
				list[3] = I->family;
			} else {
				list[3] = EMPTY;
			}
			if (B->org_name) {
				list[4] = B->org_name;
			} else {
				list[4] = EMPTY;
			}

			gtk_clist_append (GTK_CLIST (AddressBook.clist_address), list);
			g_free(list[0]);
			AddrL = g_list_next (AddrL);
		}
		gtk_clist_thaw (GTK_CLIST (AddressBook.clist_address));
	} else {
									/*NO address book open - do a dialog FIX*/
	}
}

static void
address_book_view_contacts			(void)
{
	gtk_text_set_point (GTK_TEXT (AddressBook.ViewText.text), 0);
	gtk_text_forward_delete (GTK_TEXT (AddressBook.ViewText.text),
		gtk_text_get_length (GTK_TEXT (AddressBook.ViewText.text)));
	
	AddressBook.selected_contact_key = -1;
	AddressBook.selected_row = -1;
	if (AddressBookConfig.filter == -1) {
		address_book_view_all_contacts ();
	} else {
		gtk_clist_freeze (GTK_CLIST (AddressBook.clist_address));
		gtk_clist_clear (GTK_CLIST (AddressBook.clist_address));
		switch (AddressBookConfig.filter) {
			case '1':
				address_book_view_select_contacts ('0');
				address_book_view_select_contacts ('1');
				address_book_view_select_contacts ('2');
				address_book_view_select_contacts ('3');
				address_book_view_select_contacts ('4');
				address_book_view_select_contacts ('5');
				address_book_view_select_contacts ('6');
				address_book_view_select_contacts ('7');
				address_book_view_select_contacts ('8');
				address_book_view_select_contacts ('9');
				break;
			default:
				address_book_view_select_contacts (AddressBookConfig.filter);
				address_book_view_select_contacts (tolower (AddressBookConfig.filter));
				break;
		}
		gtk_clist_thaw (GTK_CLIST (AddressBook.clist_address));
	}

}

static void
address_book_view_select_contacts			(const gchar		filter)
{
	GList *AddrL = ActiveAddressList;
	gint list_len, i;
	
	AddressBook.clist_length = 0;
	if (!AddrL) {
		gtk_clist_clear (GTK_CLIST (AddressBook.clist_address));
		return;
	}
	if (AddrL) {
		list_len = g_list_length (AddrL);
		for (i = 0; i < list_len; i++) {
			gchar *EMPTY = " ";
			gchar *list[5];
			LibGefaxContact *C = LIB_GEFAX_CONTACT (AddrL);
			LibGefaxIdentity *I = C->Identity;
			LibGefaxBusiness *B = C->Business;
			
			if (!I->family) {
				AddrL = g_list_next (AddrL);
				continue;
			}
			if (I->family[0] == filter) {
				list[0] = g_strdup_printf ("%d",C->Key);
				if (I->prefix) {
					list[1] = I->prefix;
				} else {
					list[1] = EMPTY;
				}
				if (I->given) {
					list[2] = I->given;
				} else {
					list[2] = EMPTY;
				}
				if (I->family) {
					list[3] = I->family;
				} else {
					list[2] = EMPTY;
				}
				if (B->org_name) {
					list[4] = B->org_name;
				} else {
					list[4] = EMPTY;
				}
				++AddressBook.clist_length;
				gtk_clist_append (GTK_CLIST (AddressBook.clist_address), list);
				g_free(list[0]);
			}
			AddrL = g_list_next (AddrL);
		}
	} else {
									/*NO address book open - do a dialog FIX*/
	}
	++AddressBook.clist_length;
}

static void
address_book_filter_address_book_call		(GtkWidget			*widget,
						gpointer			data)
{
	AddressBookConfig.option_filter_pos = GPOINTER_TO_INT (data);
	address_book_filter_address_book ();
	address_book_view_contacts ();
	
	
}
static void
on_SelectToolbar_btn_toggled			(GtkToggleButton		*B,
						gpointer			letter)
{
	if (B->active) {
		if (AddressBook.SelectToolbar.last_togglebutton) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (AddressBook.SelectToolbar.last_togglebutton), FALSE);
		}
		AddressBook.SelectToolbar.last_togglebutton = (GtkWidget *) B;
		AddressBookConfig.filter = GPOINTER_TO_INT (letter);;
	} else {
		AddressBookConfig.filter = -1;
		AddressBook.SelectToolbar.last_togglebutton = NULL;
	}
	address_book_view_contacts ();

	return;
}

static gint
on_clist_address_event				(GtkWidget			*w,
						GdkEvent			*event,
						gpointer			data)
{
	if (event->type == GDK_BUTTON_PRESS) {
		if (event->button.button == 3) {
			GdkEventButton *bevent = (GdkEventButton *) event;
			gtk_menu_popup (GTK_MENU (AddressBook.ab_pop_menu), NULL,
				NULL, NULL, NULL, bevent->button, bevent->time);
			return TRUE;
		}
	}
	return FALSE;
}

static void
on_clist_address_sorted				(GtkWidget			*widget,
						gpointer			data)
{
	gint i;
	
	if (AddressBook.selected_contact_key == -1) {
		return;
	}
	for (i = 0; i < AddressBook.clist_length; i++) {
		gint key;
		gchar *key_char;

		gtk_clist_get_text (GTK_CLIST (AddressBook.clist_address), i, 0, &key_char);
		key = atoi (key_char);
		if (key == AddressBook.selected_contact_key) {
			gtk_clist_select_row (GTK_CLIST (AddressBook.clist_address), i, 0);
			break;
		}
	}
}

static void
address_book_clist_unselect_row			(GtkCList			*clist,
						gint				row,
						gint				column,
						GdkEventButton			*event,
						gpointer			data)
{
	AddressBook.selected_contact_key = -1;
	AddressBook.selected_row = -1;
}
static void
address_book_clist_select_row			(GtkCList			*clist,
						gint				row,
						gint				column,
						GdkEventButton			*event,
						gpointer			data)
{
	GtkWidget *pb = NULL;
	gchar *key_char;

	gtk_clist_get_text (GTK_CLIST (clist), row, 0, &key_char);
	AddressBook.selected_contact_key = atoi (key_char);
	AddressBook.selected_row = row;
	diplay_contact_in_view_text (AddressBook.selected_contact_key);
	if (!event) {
		return;
	}
	if ((event->type == GDK_2BUTTON_PRESS) && (event->button == 1 )){
		pb = lib_gefax_contact_dialog_new (AddressList, AddressBook.selected_contact_key);
		if (pb) {
			gtk_widget_show (pb);
			gtk_signal_connect (GTK_OBJECT (pb), "addressbook_changed",
				GTK_SIGNAL_FUNC (addressbook_changed), NULL);
		}
	}
}

void
address_book_load_config			(void)
{
	gchar *tmp;
	gnome_config_push_prefix ("Gefax-AddressBook/");
	
	/*Try default ~/.cardfile*/
	tmp = g_strdup_printf ("UserConfig/CardFile=%s/.gnome/GnomeCard.gcrd",home);
	AddressBookConfig.file_path = gnome_config_get_string (tmp);
	g_free (tmp);
	if (access (AddressBookConfig.file_path, F_OK) == -1) {
		g_free (AddressBookConfig.file_path);
		AddressBookConfig.file_path = NULL;
	} else {
		address_book_load_file (AddressBookConfig.file_path);
		address_book_filter_address_book ();
	}
	
	AddressBookConfig.paned_pos = gnome_config_get_int ("UserConfig/PanedPos=350");
	AddressBookConfig.clist_pos[0] = gnome_config_get_int ("UserConfig/ClistPos[0]=45");
	AddressBookConfig.clist_pos[1] = gnome_config_get_int ("UserConfig/ClistPos[1]=90");
	AddressBookConfig.clist_pos[2] = gnome_config_get_int ("UserConfig/ClistPos[2]=90");
	AddressBookConfig.clist_pos[3] = gnome_config_get_int ("UserConfig/ClistPos[3]=90");
	AddressBookConfig.option_filter_pos = gnome_config_get_int ("UserConfig/FilterPos=0");
	AddressBookConfig.filter = gnome_config_get_int ("UserConfig/Filter=-1");
	AddressBookConfig.search_toolbar = gnome_config_get_bool ("UserConfig/SearchToolbar=TRUE");
	AddressBookConfig.filter_toolbar = gnome_config_get_bool ("UserConfig/FilterToolbar=TRUE");
	
	gnome_config_sync ();
	gnome_config_pop_prefix ();

}

void
address_book_save_yourself			(gboolean			save_address_book)
{
	if (save_address_book) {
		address_book_save_file (NULL, NULL);
	}
	address_book_free_data ();
	gnome_config_push_prefix ("Gefax-AddressBook/");
	gnome_config_set_string ("UserConfig/CardFile", AddressBookConfig.file_path);
	gnome_config_set_int ("UserConfig/PanedPos", AddressBookConfig.paned_pos);
	gnome_config_set_int ("UserConfig/ClistPos[0]", AddressBookConfig.clist_pos[0]);
	gnome_config_set_int ("UserConfig/ClistPos[1]", AddressBookConfig.clist_pos[1]);
	gnome_config_set_int ("UserConfig/ClistPos[2]", AddressBookConfig.clist_pos[2]);
	gnome_config_set_int ("UserConfig/ClistPos[3]", AddressBookConfig.clist_pos[3]);
	gnome_config_set_int ("UserConfig/FilterPos", AddressBookConfig.option_filter_pos);
	gnome_config_set_int ("UserConfig/Filter", AddressBookConfig.filter);
	gnome_config_set_bool ("UserConfig/SearchToolbar", AddressBookConfig.search_toolbar);
	gnome_config_set_bool ("UserConfig/FilterToolbar", AddressBookConfig.filter_toolbar);
	
	
	gnome_config_sync ();
	gnome_config_pop_prefix ();
}

static void
on_clist_address_resize_column			(GtkCList			*clist,
						gint				col,
						gint				width,
						gpointer			data)
{
	if (col > 0) {
		AddressBookConfig.clist_pos[col - 1] = width;
	}
}

void
address_book_adjust_your_dimensions		(gboolean			shortcuts_state,
						gint				width)
{
	if (shortcuts_state == TRUE) {
		e_paned_set_position (E_PANED (AddressBook.hp_address),
			(E_PANED (AddressBook.hp_address)->handle_xpos) - width);
	}else{
		e_paned_set_position (E_PANED (AddressBook.hp_address),
			(E_PANED (AddressBook.hp_address)->handle_xpos) + width);
	}
}

static GtkWidget
*my_gtk_toolbar_append_togglebutton		(GtkWidget			*toolbar,
						gchar				*label,
						gint				width,
						gint				height,
						GtkSignalFunc			callback,
						gpointer			user_data)
{
	GtkWidget *btn;
	
	btn = gtk_toggle_button_new_with_label (label);
	gtk_widget_show (btn);
	gtk_button_set_relief (GTK_BUTTON (btn), GTK_RELIEF_NONE);
	gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar), btn, NULL, NULL);
	gtk_widget_set_usize (btn, width, height);
	gtk_signal_connect (GTK_OBJECT (btn), "toggled", GTK_SIGNAL_FUNC (callback), user_data);
	if (AddressBookConfig.filter == label[0]) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (btn), TRUE);
	}
	return btn;
}

static void
address_book_view_search_toolbar		(GtkMenuItem			*tb,
						gpointer			data)
{
	if (GTK_CHECK_MENU_ITEM (tb)->active) {
		gtk_widget_show (AddressBook.SearchToolbar.toolbar_search);
		AddressBookConfig.search_toolbar = TRUE;
	} else {
		gtk_widget_hide (AddressBook.SearchToolbar.toolbar_search);
		AddressBookConfig.search_toolbar = FALSE;
	}
}

static void
address_book_view_filter_toolbar		(GtkMenuItem			*tb,
						gpointer			data)
{
	if (GTK_CHECK_MENU_ITEM (tb)->active) {
		gtk_widget_show (AddressBook.ViewToolbar.toolbar_view);
		AddressBookConfig.filter_toolbar = TRUE;
	} else {
		gtk_widget_hide (AddressBook.ViewToolbar.toolbar_view);
		AddressBookConfig.filter_toolbar = FALSE;
	}
}

static void
create_ViewToolbar				(View_Toolbar			*ViewToolbar)
{
	GtkWidget *lbl_view;
	GtkWidget *menuitem;
	gint i;
	const gchar *MENU_LABELS[] = {
		N_("All Contacts"), N_("Fax Number"), N_("Phone Number"),
		N_("Address"), N_("Email Address"), N_("Web Page")
	};
	const gint MENU_LABELS_LENGTH = 6;

	ViewToolbar->toolbar_view = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	gtk_container_set_border_width (GTK_CONTAINER (ViewToolbar->toolbar_view), 3);
	gtk_toolbar_set_space_size (GTK_TOOLBAR (ViewToolbar->toolbar_view), 15);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (ViewToolbar->toolbar_view), GTK_RELIEF_NONE);
	
	lbl_view = gtk_label_new (_("Contacts contain:"));
	gtk_widget_show (lbl_view);
	gtk_toolbar_append_widget (GTK_TOOLBAR (ViewToolbar->toolbar_view), lbl_view, NULL, NULL);
	gtk_misc_set_padding (GTK_MISC (lbl_view), 7, 0);
	
	ViewToolbar->option_filter = gtk_option_menu_new ();
	gtk_widget_show (ViewToolbar->option_filter);
	gtk_toolbar_append_widget (GTK_TOOLBAR (ViewToolbar->toolbar_view), ViewToolbar->option_filter, _("Filter address book by contact contents"), NULL);
	ViewToolbar->option_filter_menu = gtk_menu_new ();
	
	for (i = 0; i < MENU_LABELS_LENGTH; i++) {
		menuitem = gtk_menu_item_new_with_label (_(MENU_LABELS[i]));
		gtk_widget_show (menuitem);
		gtk_menu_append (GTK_MENU (ViewToolbar->option_filter_menu), menuitem);
		gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
				GTK_SIGNAL_FUNC (address_book_filter_address_book_call), GINT_TO_POINTER (i));
	}
	gtk_option_menu_set_menu (GTK_OPTION_MENU (ViewToolbar->option_filter), ViewToolbar->option_filter_menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (ViewToolbar->option_filter), AddressBookConfig.option_filter_pos);
}

static void
address_book_search_field_altered_call		(GtkWidget			*widget,
						gpointer			index)
{
	gint s_index = GPOINTER_TO_INT (index);

	if (s_index == 0) {
		AddressBook.SearchToolbar.search = LIBGEFAX_SEARCH_IDENTITY;
	} else if (s_index == 1) {
		AddressBook.SearchToolbar.search = LIBGEFAX_SEARCH_BUSINESS;
	} else if (s_index == 2) {
		AddressBook.SearchToolbar.search = LIBGEFAX_SEARCH_ADDRESS;
	} else if (s_index == 3) {
		AddressBook.SearchToolbar.search = LIBGEFAX_SEARCH_TELEPHONE;
	} else if (s_index == 4) {
		AddressBook.SearchToolbar.search = LIBGEFAX_SEARCH_EMAIL;
	} else if (s_index == 5) {
		AddressBook.SearchToolbar.search = LIBGEFAX_SEARCH_CATEGORY;
	} else {
		AddressBook.SearchToolbar.search = LIBGEFAX_SEARCH_ALL;
	}
}

static void
address_book_search_place_altered_call		(GtkWidget			*widget,
						gpointer			index)
{
	gint s_index = GPOINTER_TO_INT (index);
	
	if (s_index == 0) {
		AddressBook.SearchToolbar.search_type = LIBGEFAX_SEARCHTYPE_CONTAINS;
	} else if (s_index == 1) {
		AddressBook.SearchToolbar.search_type = LIBGEFAX_SEARCHTYPE_IS;
	} else if (s_index == 2) {
		AddressBook.SearchToolbar.search_type = LIBGEFAX_SEARCHTYPE_BEGIN;
	} else if (s_index == 3) {
		AddressBook.SearchToolbar.search_type = LIBGEFAX_SEARCHTYPE_END;
	}
}

static void
create_SearchToolbar				(Search_Toolbar			*SearchToolbar)
{
	GtkWidget *opt_field;
	GtkWidget *opt_field_menu;
	GtkWidget *menuitem;
	GtkWidget *opt_place;
	GtkWidget *opt_place_menu;
	GtkWidget *tmp_toolbar_icon;
	GtkWidget *btn_search;
	gint i;
	const gchar *PLACE_MENU_LABELS[] = {
		N_("Contains"), N_("Is"), N_("Begins with"),
		N_("Ends with")
	};
	const gint PLACE_MENU_LABELS_LENGTH = 4;
	const gchar *FIELD_MENU_LABELS[] = {
		N_("Name or Filename"), N_("Organization or Title"), N_("Address"),
		N_("Telephone"), N_("Email"), N_("Category"), N_("Any Field")
	};
	const gint FIELD_MENU_LABELS_LENGTH = 7;

	SearchToolbar->search = LIBGEFAX_SEARCH_IDENTITY;
	SearchToolbar->search_type = LIBGEFAX_SEARCHTYPE_CONTAINS;
	
	SearchToolbar->toolbar_search = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (SearchToolbar->toolbar_search), GTK_RELIEF_NONE);
	gtk_container_set_border_width (GTK_CONTAINER (SearchToolbar->toolbar_search), 1);
	
	opt_field = gtk_option_menu_new ();
	gtk_widget_show (opt_field);
	gtk_toolbar_append_widget (GTK_TOOLBAR (SearchToolbar->toolbar_search), opt_field, NULL, NULL);
	opt_field_menu = gtk_menu_new ();
	for (i = 0; i < FIELD_MENU_LABELS_LENGTH; i++) {
		menuitem = gtk_menu_item_new_with_label (_(FIELD_MENU_LABELS[i]));
		gtk_widget_show (menuitem);
		gtk_menu_append (GTK_MENU (opt_field_menu), menuitem);
		gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
				GTK_SIGNAL_FUNC (address_book_search_field_altered_call), GINT_TO_POINTER (i));
	}
	gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_field), opt_field_menu);
	
	opt_place = gtk_option_menu_new ();
	gtk_widget_show (opt_place);
	gtk_toolbar_append_space (GTK_TOOLBAR (SearchToolbar->toolbar_search));
	
	gtk_toolbar_append_widget (GTK_TOOLBAR (SearchToolbar->toolbar_search), opt_place, NULL, NULL);
	opt_place_menu = gtk_menu_new ();
	for (i = 0; i < PLACE_MENU_LABELS_LENGTH; i++) {
		menuitem = gtk_menu_item_new_with_label (_(PLACE_MENU_LABELS[i]));
		gtk_widget_show (menuitem);
		gtk_menu_append (GTK_MENU (opt_place_menu), menuitem);
		gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
				GTK_SIGNAL_FUNC (address_book_search_place_altered_call), GINT_TO_POINTER (i));
	}
	gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_place), opt_place_menu);
	
	SearchToolbar->tog_case_sens = gtk_toggle_button_new_with_label (_("Case Sensitive"));
	gtk_widget_show (SearchToolbar->tog_case_sens);
	gtk_toolbar_append_space (GTK_TOOLBAR (SearchToolbar->toolbar_search));
	
	gtk_toolbar_append_widget (GTK_TOOLBAR (SearchToolbar->toolbar_search), SearchToolbar->tog_case_sens, NULL, NULL);
	gtk_button_set_relief (GTK_BUTTON (SearchToolbar->tog_case_sens), GTK_RELIEF_NONE);
	
	SearchToolbar->entry_search = gtk_entry_new ();
	gtk_widget_show (SearchToolbar->entry_search);
	gtk_toolbar_append_space (GTK_TOOLBAR (SearchToolbar->toolbar_search));
	
	gtk_toolbar_append_widget (GTK_TOOLBAR (SearchToolbar->toolbar_search), SearchToolbar->entry_search, NULL, NULL);
	gtk_widget_set_usize (SearchToolbar->entry_search, 158, -2);
	
	gtk_toolbar_append_space (GTK_TOOLBAR (SearchToolbar->toolbar_search));
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (SearchToolbar->toolbar_search, GNOME_STOCK_PIXMAP_SRCHRPL);
	btn_search = gtk_toolbar_append_element (GTK_TOOLBAR (SearchToolbar->toolbar_search), GTK_TOOLBAR_CHILD_BUTTON, NULL,
						_("Search"), NULL, NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (btn_search);
	gtk_signal_connect (GTK_OBJECT (btn_search), "clicked",
				GTK_SIGNAL_FUNC (address_book_search_callback), SearchToolbar);
}

static void
create_ViewText					(View_Text			*ViewText)
{
	ViewText->sw_text = gtk_scrolled_window_new (NULL,NULL);
	gtk_widget_show (ViewText->sw_text);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ViewText->sw_text),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	ViewText->text = gtk_text_new (NULL, NULL);
	e_shortcut_bar_set_canvas_style (ViewText->text);
	gtk_widget_ref (ViewText->text);
	gtk_widget_show (ViewText->text);
	gtk_container_add (GTK_CONTAINER (ViewText->sw_text), ViewText->text);
	gtk_text_set_line_wrap (GTK_TEXT(ViewText->text),FALSE);
	gtk_widget_set_sensitive (ViewText->text, FALSE);
	
	/* Get the system color map and allocate the color red */
	ViewText->cmap = gdk_colormap_get_system();
/*	ViewText->bgcolor.red = 0xffff;
	ViewText->bgcolor.green = 0xffff;
	ViewText->bgcolor.blue = 0;
	if (!gdk_color_alloc (ViewText->cmap, &ViewText->bgcolor)) {
		g_error ("couldn't allocate color");
	}*/
	ViewText->bgcolor.red = 250 * 255;
	ViewText->bgcolor.green = 250 * 255;
	ViewText->bgcolor.blue = 210 * 255;
	if (!gdk_color_alloc (ViewText->cmap, &ViewText->bgcolor)) {
		g_error ("couldn't allocate color");
	}
	ViewText->scolor.red = 0 * 255;
	ViewText->scolor.green = 0 * 255;
	ViewText->scolor.blue = 139 * 255;
	if (!gdk_color_alloc (ViewText->cmap, &ViewText->scolor)) {
		g_error ("couldn't allocate color");
	}

	ViewText->section_font = gdk_fontset_load (GEFAX_ADDRESS_VIEW_SECTION_FONT);
	ViewText->fixed_font = gdk_fontset_load (GEFAX_ADDRESS_VIEW_FIXED_FONT);
	ViewText->header_font = gdk_fontset_load (GEFAX_ADDRESS_VIEW_HEADER_FONT);

}

static void
create_MainToolbar				(Main_Toolbar			*MainToolbar)
{
	GtkWidget *tmp_toolbar_icon;
	GtkAccelGroup *accel_group;

	accel_group = gtk_accel_group_new ();

	MainToolbar->toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	
	gtk_container_set_border_width (GTK_CONTAINER (MainToolbar->toolbar), 2);
	gtk_toolbar_set_space_style (GTK_TOOLBAR (MainToolbar->toolbar), GTK_TOOLBAR_SPACE_LINE);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (MainToolbar->toolbar), GTK_RELIEF_NONE);
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_BOOK_OPEN);
	MainToolbar->btn_open = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL,
		_("Open"), _("Open address book"), NULL, tmp_toolbar_icon, NULL, NULL);
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_SAVE);
	MainToolbar->btn_save = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL,
		_("Save"), _("Save address book"), NULL, tmp_toolbar_icon, NULL, NULL);
/*gtk_widget_set_sensitive (MainToolbar->btn_save, altered);*/
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_SAVE_AS);	
	MainToolbar->btn_save_as = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL,
		_("Save As"), _("Save address book as a different file"), NULL, tmp_toolbar_icon, NULL, NULL);
	
	gtk_toolbar_append_space (GTK_TOOLBAR (MainToolbar->toolbar));
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_ADD);
	MainToolbar->btn_addcard = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL,
		_("Add"), _("Add new contact"), NULL, tmp_toolbar_icon, NULL, NULL);
		
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_REMOVE);
	MainToolbar->btn_editcard = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL,
		_("Edit"), _("Edit Contact"), NULL, tmp_toolbar_icon, NULL, NULL);
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_TRASH);
	MainToolbar->btn_delete = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL,
		_("Delete"), _("Delete Contact"), NULL, tmp_toolbar_icon, NULL, NULL);

	gtk_toolbar_append_space (GTK_TOOLBAR (MainToolbar->toolbar));
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_HELP);
	MainToolbar->btn_help = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
		GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Help"), _("Help"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_add_accelerator (MainToolbar->btn_help, "clicked", accel_group,
                              GDK_F1, 0,
                              GTK_ACCEL_VISIBLE);
        
	gtk_toolbar_append_space (GTK_TOOLBAR (MainToolbar->toolbar));
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_CLOSE);
	MainToolbar->btn_close = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
		GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Close"), _("Close"), NULL, tmp_toolbar_icon, NULL, NULL);
	
	gtk_widget_show (MainToolbar->toolbar);
	gtk_window_add_accel_group (GTK_WINDOW (gefax), accel_group);

	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_open), "clicked",
					GTK_SIGNAL_FUNC (address_book_change_AddressList), NULL);
	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_save_as), "clicked",
					GTK_SIGNAL_FUNC (address_book_save_AddressList_as), NULL);
	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_addcard), "clicked",
					GTK_SIGNAL_FUNC (address_book_add_contact), NULL);
	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_editcard), "clicked",
					GTK_SIGNAL_FUNC (address_book_edit_contact), NULL);
	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_delete), "clicked",
					GTK_SIGNAL_FUNC (address_book_remove_contact), NULL);
	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_close), "clicked",
					GTK_SIGNAL_FUNC (change_to_browser), NULL);
	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_save), "clicked",
					GTK_SIGNAL_FUNC (address_book_save_file), NULL);
	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_help), "clicked",
					GTK_SIGNAL_FUNC (address_book_show_help), NULL);

}

static void
create_SelectToolbar				(Select_Toolbar			*SelectToolbar)
{
	const gchar *BUTTON_LABELS[] = {
		"123..", "A", "B", "C", "D", "E",
		"F", "G", "H", "I", "J", "K", "L",
		"M", "N", "O", "P", "Q", "R", "S",
		"T", "U", "V", "W", "X", "Y", "Z"
	};
	const gint TOOLBAR_LENGTH = 27;
	const gint BTN_SIZE = 24;
	gint i;
	
	SelectToolbar->last_togglebutton = NULL;
	SelectToolbar->sw_select = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (SelectToolbar->sw_select);
	gtk_widget_set_usize (SelectToolbar->sw_select, -2, 52);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (SelectToolbar->sw_select), GTK_POLICY_ALWAYS, GTK_POLICY_NEVER);
	SelectToolbar->v_port_select = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (SelectToolbar->v_port_select);
	gtk_container_add (GTK_CONTAINER (SelectToolbar->sw_select), SelectToolbar->v_port_select);
	gtk_viewport_set_shadow_type (GTK_VIEWPORT (SelectToolbar->v_port_select), GTK_SHADOW_NONE);

	SelectToolbar->toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	gtk_widget_set_name(SelectToolbar->toolbar, "select_toolbar");
	gtk_container_add (GTK_CONTAINER (SelectToolbar->v_port_select), SelectToolbar->toolbar);
	gtk_container_set_border_width (GTK_CONTAINER (SelectToolbar->toolbar), 5);
	gtk_toolbar_set_space_size (GTK_TOOLBAR (SelectToolbar->toolbar), 2);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (SelectToolbar->toolbar), GTK_RELIEF_NONE);

	for (i = 0; i < TOOLBAR_LENGTH; i++) {
		GtkWidget *tb;
		if (i == 0) {
			tb = my_gtk_toolbar_append_togglebutton (SelectToolbar->toolbar, (gchar *) BUTTON_LABELS[i],
				-2, BTN_SIZE, on_SelectToolbar_btn_toggled, GINT_TO_POINTER ((gint)BUTTON_LABELS[i][0]));
		} else {
			tb = my_gtk_toolbar_append_togglebutton (SelectToolbar->toolbar, (gchar *) BUTTON_LABELS[i],
				BTN_SIZE, BTN_SIZE, on_SelectToolbar_btn_toggled, GINT_TO_POINTER ((gint)BUTTON_LABELS[i][0]));
		}
		gtk_toolbar_append_space (GTK_TOOLBAR (SelectToolbar->toolbar));
	}
	gtk_widget_show (SelectToolbar->toolbar);
}

void
create_address_book				(void)
{
	GSList *ab_group = NULL;
	gint i;

	AddressBook.selected_contact_key = -1;
	AddressBook.clist_length = 0;
	AddressBook.vbox_contacts = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (AddressBook.vbox_contacts);
	gefax_app_set_contents (AddressBook.vbox_contacts);

	ActiveComponent = CONTACTS_ON;
	
	create_SearchToolbar (&AddressBook.SearchToolbar);
	gtk_box_pack_start (GTK_BOX (AddressBook.vbox_contacts), AddressBook.SearchToolbar.toolbar_search, FALSE, FALSE, 0);
	create_ViewToolbar (&AddressBook.ViewToolbar);
	gtk_box_pack_start (GTK_BOX (AddressBook.vbox_contacts), AddressBook.ViewToolbar.toolbar_view, FALSE, FALSE, 0);
	
	AddressBook.hp_address = e_hpaned_new ();
	gtk_widget_show (AddressBook.hp_address);
	gtk_box_pack_start (GTK_BOX (AddressBook.vbox_contacts), AddressBook.hp_address, TRUE, TRUE, 0);
	e_paned_set_position (E_PANED (AddressBook.hp_address), AddressBookConfig.paned_pos);
		
	AddressBook.sw_address = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (AddressBook.sw_address);
	e_paned_pack1 (E_PANED (AddressBook.hp_address), AddressBook.sw_address, FALSE, TRUE);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (AddressBook.sw_address), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	AddressBook.clist_address = gtk_clist_new (5);
	gtk_clist_set_shadow_type(GTK_CLIST (AddressBook.clist_address), GTK_SHADOW_IN);
	gtk_widget_show (AddressBook.clist_address);
	gtk_container_add (GTK_CONTAINER (AddressBook.sw_address), AddressBook.clist_address);
  
  	AddressBook.sort_h1 = gtk_sort_heading_new (ab_group, GTK_CLIST (AddressBook.clist_address), 1, _("Prefix"));
  	gtk_sort_heading_set_label_justify (GTK_SORT_HEADING (AddressBook.sort_h1), GTK_JUSTIFY_LEFT);
  	ab_group = gtk_sort_heading_group (GTK_SORT_HEADING (AddressBook.sort_h1));
  	gtk_widget_show (AddressBook.sort_h1);
  	AddressBook.sort_h2 = gtk_sort_heading_new (ab_group, GTK_CLIST (AddressBook.clist_address), 2, _("Given Name"));
  	gtk_sort_heading_set_label_justify (GTK_SORT_HEADING (AddressBook.sort_h2), GTK_JUSTIFY_LEFT);
	ab_group = gtk_sort_heading_group (GTK_SORT_HEADING (AddressBook.sort_h2));
  	gtk_widget_show (AddressBook.sort_h2);
  	AddressBook.sort_h3 = gtk_sort_heading_new (ab_group, GTK_CLIST (AddressBook.clist_address), 3, _("Family Name"));
  	gtk_sort_heading_set_label_justify (GTK_SORT_HEADING (AddressBook.sort_h3), GTK_JUSTIFY_LEFT);
  	ab_group = gtk_sort_heading_group (GTK_SORT_HEADING (AddressBook.sort_h3));
  	gtk_widget_show (AddressBook.sort_h3);
  	AddressBook.sort_h4 = gtk_sort_heading_new (ab_group, GTK_CLIST (AddressBook.clist_address), 4, _("Organization"));
  	gtk_sort_heading_set_label_justify (GTK_SORT_HEADING (AddressBook.sort_h4), GTK_JUSTIFY_LEFT);
  	ab_group = gtk_sort_heading_group (GTK_SORT_HEADING (AddressBook.sort_h4));
  	gtk_widget_show (AddressBook.sort_h4);
  	
  	gtk_sort_heading_set_mode (GTK_SORT_HEADING (AddressBook.sort_h3), GTK_SORT_HEADING_ASCENDING);
  	
	gtk_clist_set_column_widget (GTK_CLIST (AddressBook.clist_address), 1, AddressBook.sort_h1);
	gtk_clist_set_column_widget (GTK_CLIST (AddressBook.clist_address), 2, AddressBook.sort_h2);
	gtk_clist_set_column_widget (GTK_CLIST (AddressBook.clist_address), 3, AddressBook.sort_h3);
	gtk_clist_set_column_widget (GTK_CLIST (AddressBook.clist_address), 4, AddressBook.sort_h4);
	
	for (i = 0; i < 4; i++) {
		gtk_clist_set_column_width (GTK_CLIST (AddressBook.clist_address), i + 1, AddressBookConfig.clist_pos[i]);
	}

	gtk_clist_column_titles_show (GTK_CLIST (AddressBook.clist_address));

	gtk_clist_column_titles_passive (GTK_CLIST (AddressBook.clist_address));
	gtk_clist_set_column_visibility (GTK_CLIST (AddressBook.clist_address), 0, FALSE);

	create_ViewText (&AddressBook.ViewText);
	e_paned_pack2 (E_PANED (AddressBook.hp_address), AddressBook.ViewText.sw_text, TRUE, TRUE);
	
	
	
	create_SelectToolbar (&AddressBook.SelectToolbar);
	gtk_box_pack_start (GTK_BOX (AddressBook.vbox_contacts), AddressBook.SelectToolbar.sw_select, FALSE, TRUE, 0);
	
	create_MainToolbar (&AddressBook.MainToolbar);
	gtk_container_add (GTK_CONTAINER (toolbar_dock), AddressBook.MainToolbar.toolbar);
	
	
	
	
	gtk_window_set_title (GTK_WINDOW (gefax), "gefax - Contacts");
	gnome_app_insert_menus (GNOME_APP (gefax),"File/",ab_menu);
	gnome_app_insert_menus (GNOME_APP (gefax),"File",ab_edit_menu);
	gnome_app_insert_menus (GNOME_APP (gefax),"Edit",ab_view_menu);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ab_views_menu[0].widget), AddressBookConfig.search_toolbar);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ab_views_menu[1].widget), AddressBookConfig.filter_toolbar);
/*gtk_widget_set_sensitive(ab_menu[1].widget, altered);*/
	gnome_app_install_menu_hints (GNOME_APP (gefax), ab_menu);
	gnome_app_install_menu_hints (GNOME_APP (gefax), ab_edit_menu);
	gnome_app_install_menu_hints (GNOME_APP (gefax), ab_view_menu);
	
	/*popup menu*/
	AddressBook.ab_pop_menu = gtk_menu_new ();
	gnome_app_fill_menu (GTK_MENU_SHELL (AddressBook.ab_pop_menu), ab_popup_menu, NULL, FALSE, 0);
	gnome_app_install_menu_hints (GNOME_APP (gefax), ab_popup_menu);
	
	gtk_signal_connect (GTK_OBJECT (AddressBook.clist_address), "select_row",
					GTK_SIGNAL_FUNC (address_book_clist_select_row), NULL);
	gtk_signal_connect (GTK_OBJECT (AddressBook.clist_address), "unselect_row",
					GTK_SIGNAL_FUNC (address_book_clist_unselect_row), NULL);
	/*right click menu callback*/
	gtk_signal_connect (GTK_OBJECT (AddressBook.clist_address), "event",
					GTK_SIGNAL_FUNC (on_clist_address_event), NULL);
	gtk_signal_connect (GTK_OBJECT (AddressBook.clist_address), "resize_column",
					GTK_SIGNAL_FUNC (on_clist_address_resize_column), NULL);
	gtk_signal_connect (GTK_OBJECT (AddressBook.sort_h1), "mode_changed",
					GTK_SIGNAL_FUNC (on_clist_address_sorted), NULL);
	gtk_signal_connect (GTK_OBJECT (AddressBook.sort_h2), "mode_changed",
					GTK_SIGNAL_FUNC (on_clist_address_sorted), NULL);
	gtk_signal_connect (GTK_OBJECT (AddressBook.sort_h3), "mode_changed",
					GTK_SIGNAL_FUNC (on_clist_address_sorted), NULL);
	gtk_signal_connect (GTK_OBJECT (AddressBook.sort_h4), "mode_changed",
					GTK_SIGNAL_FUNC (on_clist_address_sorted), NULL);
	
	
	
	/*View all contacts*/
	address_book_filter_address_book ();
	address_book_view_contacts ();
	return;
}

void
remove_address_book					(void)
{
	AddressBookConfig.paned_pos = e_paned_get_position (E_PANED (AddressBook.hp_address));
	gnome_app_remove_menus(GNOME_APP (gefax),"File/Open Address Book", 5);
	gnome_app_remove_menus(GNOME_APP (gefax),"Edit", 1);
	gnome_app_remove_menus(GNOME_APP (gefax),"View", 1);
	gtk_object_destroy (GTK_OBJECT (AddressBook.ab_pop_menu));
	gtk_object_destroy (GTK_OBJECT (AddressBook.vbox_contacts));
	gtk_object_destroy (GTK_OBJECT (AddressBook.MainToolbar.toolbar));
}

static void
view_ViewText_subitem				(const gchar		*data,
						const gchar		*header)
{
	if (!data) {
		return;
	}
	gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font, &AddressBook.ViewText.text->style->black, NULL, header, -1);
	gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.fixed_font, &AddressBook.ViewText.text->style->black, NULL, data, -1);
	gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), NULL, NULL, NULL, "\n", 1);
	return;
}

static void
view_ViewText_address					(LibGefaxAddress		*A)
{
	const gchar *ADDR_TYPES[] = {
		N_(" (Home)"), N_(" (Work)"), N_(" (Postal)"),
		N_(" (Parcel)"), N_(" (Domestic)"), N_(" (International)")
	};
	const gint ADDR_TYPES_LENGTH = 6;
	gint i;
	
	gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font,
						&AddressBook.ViewText.scolor, NULL, "Address:", -1);
	for (i = 0; i < ADDR_TYPES_LENGTH; i++) {
		if (A->type & 1 << i) {
			gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font,
						&AddressBook.ViewText.scolor, NULL, ADDR_TYPES[i], -1);
		}
	}
	gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font, &AddressBook.ViewText.text->style->black, NULL, "\n", -1);
	view_ViewText_subitem (A->post_box, "  Post Box:\t");
	view_ViewText_subitem (A->extended, "  Extended:\t");
	view_ViewText_subitem (A->street, "  Street:\t");
	view_ViewText_subitem (A->city, "  City:\t\t");
	view_ViewText_subitem (A->region, "  Region:\t");
	view_ViewText_subitem (A->post_code, "  Post Code:\t");
	view_ViewText_subitem (A->country, "  Country:\t");
}

static void
view_ViewText_telephone					(LibGefaxTelephone		*T)
{
	const gchar *TEL_TYPES[] = {
		N_(" (Preferred)"), N_(" (Work)"), N_(" (Home)"),
		N_(" (Voice)"), N_(" (Fax)"), N_(" (Message)"),
		N_(" (Cellular)"), N_(" (Pager)"), N_(" (Bulletin Board)"),
		N_(" (Modem)"), N_(" (Car)"), N_(" (ISDN)"), N_(" (Video)")
	};
	const gint TEL_TYPES_LENGTH = 13;
	gint i;
	
	gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font,
						&AddressBook.ViewText.scolor, NULL, "Telephone:", -1);
	for (i = 0; i < TEL_TYPES_LENGTH; i++) {
		if (T->type & 1 << i) {
			gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font,
						&AddressBook.ViewText.scolor, NULL, TEL_TYPES[i], -1);
		}
	}
	view_ViewText_subitem (T->number, "\n  Number:\t");
}

static void
view_ViewText_email					(LibGefaxEmail		*E)
{
	const gchar *EMAIL_TYPES[] = {
		N_(" (America On-Line)"), N_(" (Apple Link)"), N_(" (AT&T)"), N_(" (CIS)"),
		N_(" (e-World)"), N_(" (Internet)"), N_(" (IBM)"), N_(" (MCI)"),
		N_(" (Power Share)"), N_(" (Prodigy)"), N_(" (TLX)"), N_(" (X400)")
	};
	const gint EMAIL_TYPES_LENGTH = 12;
	gint i;
	
	gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font,
						&AddressBook.ViewText.scolor, NULL, "Email:", -1);
	for (i = 0; i < EMAIL_TYPES_LENGTH; i++) {
		if (E->type == i) {
			gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font,
						&AddressBook.ViewText.scolor, NULL, EMAIL_TYPES[i], -1);
		}
	}
	view_ViewText_subitem (E->number, "\n  Address:\t");
}

static void
diplay_contact_in_view_text			(gint			key)
{
	LibGefaxContact *C = lib_gefax_contact_get_by_key (AddressList, key);
	LibGefaxIdentity 	*I = C->Identity;
	LibGefaxBusiness 	*B = C->Business;
	/*LibGefaxTimeZone	*T = C->TimeZone;
	LibGefaxGeo		*G = C->GeoLocation;
	LibGefaxPublicKey	*K = C->PublicKey;*/
	GSList *List = NULL;
	gint list_length, i;
	
	g_return_if_fail (C != NULL);

	gtk_text_freeze (GTK_TEXT (AddressBook.ViewText.text));
	/*clear the text box*/
	gtk_text_set_point (GTK_TEXT (AddressBook.ViewText.text),0);
	gtk_text_forward_delete (GTK_TEXT (AddressBook.ViewText.text), gtk_text_get_length (GTK_TEXT (AddressBook.ViewText.text)));
	
	if(C->filename){
		gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.header_font, &AddressBook.ViewText.text->style->black, &AddressBook.ViewText.bgcolor, C->filename, -1);
	}else{
		gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.header_font, &AddressBook.ViewText.text->style->black, &AddressBook.ViewText.bgcolor, "??", -1);
	}
	
	gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.header_font, NULL, NULL, "\n", 1);
	gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.header_font, NULL, NULL, "\n", 1);
	/*Identity*/
	if ((I->prefix) || (I->suffix) || (I->given) || (I->additional) || (I->family)) {
		gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font,
						&AddressBook.ViewText.scolor, NULL, "Identity:\n", -1);
		view_ViewText_subitem (I->prefix, "  Prefix:\t");
		view_ViewText_subitem (I->suffix, "  Suffix:\t");
		view_ViewText_subitem (I->given, "  Given:\t");
		view_ViewText_subitem (I->additional, "  Additional:\t");
		view_ViewText_subitem (I->family, "  Family:\t");
	}
	/*Business*/
	if ((B->org_name) || (B->org_title)) {
		gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font,
						&AddressBook.ViewText.scolor, NULL, "Business:\n", -1);
		view_ViewText_subitem (B->org_name, "  Organization:\t");
		view_ViewText_subitem (B->org_title, "  Title:\t");
	}
	if (C->Address) {
		List = C->Address;
		list_length = g_slist_length (List);
		for (i = 0; i < list_length; i++) {
			LibGefaxAddress *A = List->data;
			
			view_ViewText_address (A);
			List = g_slist_next (List);
		}
	}
	if (C->Tel) {
		List = C->Tel;
		list_length = g_slist_length (List);
		for (i = 0; i < list_length; i++) {
			LibGefaxTelephone *T = List->data;
			
			view_ViewText_telephone (T);
			List = g_slist_next (List);
		}
	}
	if ((C->url) || (C->Email)) {
		gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font,
						&AddressBook.ViewText.scolor, NULL, "Network:\n", -1);
		view_ViewText_subitem (C->url, "  Web Page:\t");
		if (C->Email) {
			List = C->Email;
			list_length = g_slist_length (List);
			for (i = 0; i < list_length; i++) {
				LibGefaxEmail *E = List->data;
			
				view_ViewText_email (E);
				List = g_slist_next (List);
			}
		}
		
	}
	if (C->note) {
		gtk_text_insert (GTK_TEXT (AddressBook.ViewText.text), AddressBook.ViewText.section_font,
						&AddressBook.ViewText.scolor, NULL, "Note:\n", -1);
		view_ViewText_subitem (C->note, "");
	}
	gtk_text_thaw (GTK_TEXT (AddressBook.ViewText.text));
}





