/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "allow_mime.h"
#include "gefax.h"

struct allowed_mime{
	gchar	*type;				/*alowed mime type*/
	gchar	*command;				/*program to convert to tiffg3*/
	struct	allowed_mime *next_ptr;	/*pointer to the next element*/
};

struct allowed_mime *first_ptr = NULL;

void free_allowed_mime(void)
{
	while (first_ptr){
		g_free(first_ptr->type);
		g_free(first_ptr->command);
		first_ptr = first_ptr->next_ptr;
	}
	first_ptr = NULL;
		
}
static void load_allowed_mime(gchar *type);

void read_mime_file(void)
{
	FILE *mime_file;
	gchar *allow_mime;
	gchar aline[100];
	mime_file = fopen(GEFAX_FILE_MIME, "r");
	
#ifdef DEBUG
		printf("DEBUG:Loading config file\n");
#endif /* DEBUG*/

	if (mime_file == NULL) {
			g_error(GEFAX_FILE_MIME " does not exist\n");
	}else{
		while(1){
			allow_mime = fgets(aline, sizeof(aline), mime_file);
			if (allow_mime == NULL) break;
			if ((aline[0] != '#') && (aline[0] != '\n')) {		/*comment or empty line*/
				aline[(strlen(aline) - 1)] = '\0';
				load_allowed_mime(aline);
#				ifdef DEBUG
					printf("	DEBUG:Allowed mime type=%s\n", aline);
#				endif /* DEBUG*/
			}
		}
	}
	fclose(mime_file);
}
/*
 *here we get eg application/postscript=ghostscript
 *we have to split the string at = first
*/
void load_allowed_mime(gchar *type)
{

	struct allowed_mime *new_item_ptr;
	gchar *command;

	command = strchr(type, '=');
	if (command == NULL){
		g_error("Faulty " GEFAX_FILE_MIME " file - no = sign in string\nQuitting");
		gtk_main_quit();
	}
	*command = '\0';	/*now points to "=" */
	++command;
	new_item_ptr = g_malloc(sizeof(struct allowed_mime));	/*New Element*/
	new_item_ptr->type = g_strdup(type);
	new_item_ptr->command = g_strdup(command);
	new_item_ptr->next_ptr = first_ptr;
	first_ptr = new_item_ptr;
#	ifdef DEBUG
		printf("	DEBUG:Command=%s\n",command);
#	endif /*DEBUG*/
}

/*
 *finds a match and return 0 if none and 1 if yes 
*/
int find_mime(gchar *name)
{

	struct allowed_mime *current_ptr;
	
	current_ptr = first_ptr;
	while (current_ptr != NULL){
		if (strcmp(current_ptr->type, name) == 0){
			break;
			
		}
		current_ptr = current_ptr->next_ptr;
	}

	return (current_ptr != NULL);
}

/*
 *returns command given the mime type
*/
gchar *find_command	(gchar *name)
{
	struct allowed_mime *current_ptr;
	
	current_ptr = first_ptr;
	while (current_ptr != NULL){
		if (strcmp(current_ptr->type, name) == 0){
			break;
		}
		current_ptr = current_ptr->next_ptr;
	}
	if (current_ptr == NULL){
		return NULL;
	}else{
		return current_ptr->command;
	}
}



/*end*/






