/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <gnome.h>
#include <gal/e-paned/e-hpaned.h>

#include "gefax.h"
#include "browser.h"
#include "support.h"
#include "menu.h"
#include "toolbars.h"
#include "dialog_ret.h"
#include "dialog_num.h"
#include "exec_cmd.h"


static GtkWidget *clist_fax;
static GtkWidget *clist_nums;
static GtkWidget *scrolled_win_files;
static GtkWidget *h_paned;
static GtkWidget *scrolled_win_nums;
static GtkWidget *pixmap1;
static GtkWidget *event_box_nums;
static GtkWidget *event_box_fax;
static GtkWidget *browser_pop_menu;

static gint total_selected = 0;

/*
 *	Popup menu if right click
*/
static gint on_browser_click_event(GtkWidget *w, GdkEvent *event);

/*
 *	Selects or deselects the file
*/
static void on_clist_fax_select_row(GtkCList *clist, gint row, gint column, GdkEvent *event, GtkWidget *data);

/*
 *	Delete fax number
*/
static void on_clist_nums_select_row(GtkCList *clist, gint row, gint column, GdkEvent *event, GtkWidget *data);

/*
 *	Selects the first 9 files
*/
void set_all_selected(void);

/*
 *	Sets FileList->select to " " for all rows
*/
void set_all_unselected(void);

/*
 *	Finds row containing filename and sets FileList->select to next available number
*/
gint set_selected(gchar *fname);

/*
 *	Reorders numbering after unselecting a file
*/
void reorder_select_list(gchar *fname);

/*
 *	finds file name in FileList and returns 1 if FileList->select != " "
 *	0 if not
*/
gint is_row_selected(gchar *file_name);


void remove_file_browser(void)
{
#	ifdef DEBUG
		printf("DEBUG:Removing file browser\n");
#	endif /*DEBUG*/
	/*remove unused toolbars*/
	remove_MainToolbar();
	remove_browser_menu();
	gtk_widget_destroy(browser_pop_menu);
	browser_pop_menu = NULL;	
	gtk_widget_destroy(clist_nums);
	clist_nums = NULL;
	gtk_widget_destroy(scrolled_win_nums);
	scrolled_win_nums = NULL;
	gtk_widget_destroy(event_box_nums);
	event_box_nums = NULL;
	gtk_widget_destroy(pixmap1);
	pixmap1 = NULL;	
	gtk_widget_destroy(clist_fax);
	clist_fax = NULL;
	gtk_widget_destroy(scrolled_win_files);
	scrolled_win_files = NULL;
	gtk_widget_destroy(event_box_fax);
	event_box_fax = NULL;
	gtk_widget_destroy(h_paned);
	h_paned = NULL;
	return;
}

void create_file_browser(void)
{
	GdkCursor* cursor;
	
#	ifdef DEBUG
		printf("DEBUG:Building file browser\n");
#	endif /*DEBUG*/

	ActiveComponent = BROWSER_ON;
	create_MainToolbar();
	
	h_paned = e_hpaned_new();
	
	gtk_widget_show (h_paned);
	gefax_app_set_contents ( h_paned);
	if (ShortcutsToolbarOn) e_paned_set_position (E_PANED (h_paned), 290);
	else e_paned_set_position (E_PANED (h_paned), 352);
  
	event_box_fax = gtk_event_box_new();
	gtk_widget_show(event_box_fax);
	e_paned_pack1 (E_PANED (h_paned), event_box_fax, FALSE, TRUE);
  
	scrolled_win_files = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolled_win_files);
	gtk_container_add (GTK_CONTAINER (event_box_fax), scrolled_win_files);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win_files), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	clist_fax = gtk_clist_new (3);
	gtk_clist_set_shadow_type(GTK_CLIST (clist_fax),GTK_SHADOW_IN);
	gtk_widget_show (clist_fax);
	gtk_container_add (GTK_CONTAINER (scrolled_win_files), clist_fax);
	gtk_clist_set_column_auto_resize (GTK_CLIST (clist_fax),0,TRUE);
	gtk_clist_set_column_width (GTK_CLIST (clist_fax), 1, 100);
	gtk_clist_column_titles_show (GTK_CLIST (clist_fax));
	gtk_clist_set_sort_column(GTK_CLIST (clist_fax),0);
  
	pixmap1 = create_pixmap (gefax, GEFAX_ICON_BROWSER_CLIST_SELECT, FALSE);
	gtk_widget_show (pixmap1);
	gtk_clist_set_column_widget (GTK_CLIST (clist_fax), 0, pixmap1);
	gtk_widget_set_usize (pixmap1, 16, -2);

	gtk_clist_set_column_title (GTK_CLIST (clist_fax),1,_("File Name"));
	gtk_clist_set_column_title (GTK_CLIST (clist_fax),2,_("File Type"));
  
	event_box_nums = gtk_event_box_new();
	gtk_widget_show(event_box_nums);
	e_paned_pack2 (E_PANED (h_paned), event_box_nums, TRUE, TRUE);

	scrolled_win_nums = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolled_win_nums);
	gtk_container_add (GTK_CONTAINER (event_box_nums), scrolled_win_nums);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win_nums), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	clist_nums = gtk_clist_new (2);
	gtk_clist_set_shadow_type(GTK_CLIST (clist_nums),GTK_SHADOW_IN);
	gtk_widget_ref (clist_nums);
	gtk_object_set_data_full (GTK_OBJECT (gefax), "clist_nums", clist_nums,
                            (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (clist_nums);
	gtk_container_add (GTK_CONTAINER (scrolled_win_nums), clist_nums);
	gtk_clist_column_titles_show (GTK_CLIST (clist_nums));

	gtk_clist_set_column_title (GTK_CLIST (clist_nums),0,_("Contact"));
	gtk_clist_set_column_title (GTK_CLIST (clist_nums),1,_("Fax Number"));
	gtk_clist_set_column_width (GTK_CLIST (clist_nums), 0, 160);
	gtk_clist_column_titles_passive(GTK_CLIST (clist_nums));
	gtk_clist_column_titles_passive(GTK_CLIST (clist_fax));
  
	gtk_widget_realize(event_box_fax);
	gtk_widget_realize(event_box_nums);
	cursor = gdk_cursor_new(GDK_HAND2);
	gdk_window_set_cursor(event_box_fax->window, cursor);
	gdk_cursor_destroy(cursor);
	cursor = gdk_cursor_new(/*GDK_DRAPED_BOX*/GDK_PIRATE);
	gdk_window_set_cursor(event_box_nums->window, cursor);
	gdk_cursor_destroy(cursor);
	
	gtk_signal_connect (GTK_OBJECT (clist_fax), "select_row",
                      GTK_SIGNAL_FUNC (on_clist_fax_select_row), gefax);
	gtk_signal_connect (GTK_OBJECT (clist_nums), "select_row",
                      GTK_SIGNAL_FUNC (on_clist_nums_select_row), NULL);
     
	/*pop up menu*/
	browser_pop_menu = create_browser_pop_menu();
	gtk_signal_connect_object(GTK_OBJECT(clist_nums), "event",
  		 		 GTK_SIGNAL_FUNC (on_browser_click_event), NULL);
	gtk_signal_connect_object(GTK_OBJECT(clist_fax), "event",
  		 		 GTK_SIGNAL_FUNC (on_browser_click_event), NULL);

	
	if(fst_ptr){
		while (gtk_events_pending()) gtk_main_iteration();
		load_clist_fax();
		load_clist_nums();
	}else{
		load_dir();
	}

	while (gtk_events_pending()) gtk_main_iteration();
	restore_browser_menu();
  return;
}

void browser_adjust_your_dimensions(gboolean shortcuts_state, gint width)
{
	if(shortcuts_state == TRUE){
		e_paned_set_position(E_PANED(h_paned), (GTK_PANED(h_paned)->handle_xpos) - width);
	}else{
		e_paned_set_position(E_PANED(h_paned), (GTK_PANED(h_paned)->handle_xpos) + width);
	}
}

/*________________________________________Gui Events ______________________________*/

void on_clist_fax_select_row(GtkCList *clist, gint row, gint column, GdkEvent *event, GtkWidget *data)
{
	gchar *fname_ptr;
	gint sts = efax_status();
	
	if ((sts == SENDING) || (sts == EFIX_C) || (sts == GS_C)) return;
	
	gtk_clist_get_text( (GtkCList *)clist_fax, row, 1, &fname_ptr);
	set_selected(fname_ptr);
	load_clist_fax();
}

void on_clist_nums_select_row(GtkCList *clist, gint row, gint column, GdkEvent *event, GtkWidget *data)
{
	gchar *pnum;
	gchar *name;
	GtkWidget *ask;
	gchar *msg;
	gint sts = efax_status();
	
	if ((sts == SENDING) || (sts == EFIX_C) || (sts == GS_C)) return;
	
	gtk_clist_get_text( (GtkCList *) clist_nums, row, 0, &name);
	gtk_clist_get_text( (GtkCList *) clist_nums, row, 1, &pnum);
	if(remove_number_warn){
		msg = g_strdup_printf("Remove:\n%s\nFax number %s",name,pnum);
		ask = gnome_message_box_new(msg,GNOME_MESSAGE_BOX_QUESTION,
							GNOME_STOCK_BUTTON_OK,
							GNOME_STOCK_BUTTON_CLOSE,
							NULL);
		gtk_window_set_position (GTK_WINDOW (ask), GTK_WIN_POS_MOUSE);
		if(gnome_dialog_run_and_close(GNOME_DIALOG(ask)) == 0){
			if (remove_phone(pnum) == 0){
#				ifdef DEBUG
					puts("DEBUG:Error removing number from list callbacks.c:on_clist_nums_select_row");
#				endif DEBUG /* DEBUG*/
			}
		}
	}else{
		if (remove_phone(pnum) == 0){
#			ifdef DEBUG
				puts("DEBUG:Error removing number from list callbacks.c:on_clist_nums_select_row");
#			endif DEBUG /* DEBUG*/
		}
	}
	gtk_clist_unselect_row( (GtkCList *) clist_nums, row,0);
}

void on_btn_set_nums_clicked(GtkButton *button, gpointer user_data)
{
	gint sts = efax_status();
	
	if ((sts == SENDING) || (sts == EFIX_C) || (sts == GS_C)) return;
	
	create_dialog_fax_number(FALSE);
	
}

gint on_browser_click_event (GtkWidget *w, GdkEvent *event)
{
	if (event->type == GDK_BUTTON_PRESS){
		if(event->button.button == 3){
			GdkEventButton *bevent = (GdkEventButton *) event;
			gtk_menu_popup(GTK_MENU (browser_pop_menu), NULL, NULL, NULL, NULL,
							bevent->button, bevent->time);
			return TRUE;
		}
	}
	return FALSE;
}

void on_btn_select_all_clicked(GtkButton *button, gpointer data)
{
	gint sts = efax_status();
	
	if ((sts == SENDING) || (sts == EFIX_C) || (sts == GS_C)) return;
	
	set_all_selected();
	load_clist_fax();
}

void on_btn_clear_all_clicked(GtkButton *button, gpointer data)
{
	gint sts = efax_status();
	
	if ((sts == SENDING) || (sts == EFIX_C) || (sts == GS_C)) return;
	
	set_all_unselected();
	load_dir();
}

void on_btn_move_to_trash_clicked(GtkButton *button, gpointer user_data)
{

	gchar *old_filename;
	gchar *new_filename;
	struct FileList *current_ptr;
	gint sts = efax_status();
	
	if ((sts == SENDING) || (sts == EFIX_C) || (sts == GS_C)) return;
	
	current_ptr = fst_ptr;
	while (current_ptr != NULL){
		if (strcmp(current_ptr->select, UNSELECT) != 0){
			old_filename = g_strdup_printf("%s%s", fax_dir, current_ptr->file);
			new_filename = g_strdup_printf("%s%s", bin_dir, current_ptr->file);
			if (rename (old_filename, new_filename) == -1){
				g_warning("Error moving file");
			}
			g_free(old_filename);
			g_free(new_filename);
		}
		current_ptr = current_ptr->next_ptr;
	}
	set_all_unselected();
	load_dir	();
}



void on_btn_get_from_trash_clicked(GtkButton *button, gpointer user_data)
{
	gint sts = efax_status();
	
	if ((sts == SENDING) || (sts == EFIX_C) || (sts == GS_C)) return;
	
	create_dialog_retrieve ();
}

void on_btn_delete_trash_clicked(GtkButton *button, gpointer user_data)
{
	GtkWidget *rm;
	gint no_files;
	gint fcount;
	gchar *fname;
	gchar *msg;
	struct dirent **filelist;
	no_files = scandir(bin_dir, &filelist, 0,alphasort ); /*count num of files*/
	if(no_files > 2){
		msg = g_strdup_printf("WARNING\n%d files in your trash bin\nwill be permanently deleted",(no_files-2));
		rm = gnome_message_box_new(msg,
							GNOME_MESSAGE_BOX_WARNING,
							GNOME_STOCK_BUTTON_OK,
							GNOME_STOCK_BUTTON_CLOSE,
							NULL);
		gtk_window_set_position (GTK_WINDOW (rm), GTK_WIN_POS_MOUSE);
		if(gnome_dialog_run_and_close(GNOME_DIALOG(rm)) == 0){
			fcount = 0;
			while (fcount < no_files){
				fname = g_strdup_printf("%s%s", bin_dir, filelist[fcount]->d_name);
				if(g_file_test(fname, G_FILE_TEST_ISFILE)){
					if(remove(fname) != 0){
					puts("Cannot delete file");
					}
				}
				g_free(fname);
				++fcount;
			}
		}
		g_free(msg);
	}else{
		rm = gnome_message_box_new(_("Your trash bin is empty"),
							GNOME_MESSAGE_BOX_INFO,
							GNOME_STOCK_BUTTON_CLOSE,
							NULL);
		gtk_window_set_position (GTK_WINDOW (rm), GTK_WIN_POS_MOUSE);
		gnome_dialog_run_and_close(GNOME_DIALOG(rm));
	}
}


/*________________________________________Load up files, Phone numbers______________________________*/
void load_clist_nums(void)
{

	struct PhoneNumber *current_phone_ptr;
	gchar *add_rows[2];
	
#	ifdef DEBUG
		printf("DEBUG:Reloading clist_nums\n");
#	endif /* DEBUG*/
	
	gtk_clist_freeze((GtkCList *)clist_nums);
	
	current_phone_ptr = phone_fst_ptr;
	while(current_phone_ptr != NULL){
		add_rows[1] = current_phone_ptr->phone;
		add_rows[0] = current_phone_ptr->person;
		gtk_clist_append( (GtkCList *) clist_nums, add_rows);
		current_phone_ptr = current_phone_ptr->next_ptr;
	}
	gtk_clist_thaw((GtkCList *)clist_nums);
}

void load_clist_fax(void)
{
	extern int find_mime(gchar *name);
	struct FileList *current_ptr;
	gchar *title;
	gint row;
	gint allowed_row;
	gchar *add_rows[3];
	GdkColormap *cmap;
	GdkColor red;

#	ifdef DEBUG
		printf("DEBUG:Reloading clist_fax\n");
#	endif /* DEBUG*/
	

	gtk_clist_freeze((GtkCList *) clist_fax);
	gtk_clist_clear((GtkCList *) clist_fax);

	/* Get the system color map and allocate the color red */
	/* Mabye could make colours a user pref one day*/
	cmap = gdk_colormap_get_system();
	red.red = 140*255;
	red.green = 9*255;
	red.blue = 0;
	if (!gdk_color_alloc(cmap, &red)) {
		g_error("couldn't allocate color\n");
	}
	
	if(load_ordered){
			allowed_row = 0;
	current_ptr = fst_ptr;
		while(current_ptr != NULL){
			if (strcmp(current_ptr->select, UNSELECT) != 0){
				add_rows[0] = current_ptr->select;
				add_rows[1] = current_ptr->file;
				add_rows[2] = current_ptr->mtype;
				gtk_clist_append( (GtkCList *) clist_fax, add_rows);
				++allowed_row;
			}
			 current_ptr = current_ptr->next_ptr;
		}
		/*sort then out*/
		gtk_clist_sort(GTK_CLIST(clist_fax));
	
		row = allowed_row;
		current_ptr = fst_ptr;
		while(current_ptr != NULL){
			if (strcmp(current_ptr->select, UNSELECT) == 0){
				add_rows[0] = UNSELECT;
				add_rows[1] = current_ptr->file;
				add_rows[2] = current_ptr->mtype;
		
				if (find_mime(current_ptr->mtype)){
					gtk_clist_insert( (GtkCList *) clist_fax, allowed_row, add_rows);
					++allowed_row;
				}else{
					gtk_clist_append( (GtkCList *) clist_fax, add_rows);
					gtk_clist_set_foreground( (GtkCList *) clist_fax, row, &red);
				}
				++row;
			}
			 current_ptr = current_ptr->next_ptr;
		}
	}else{
		allowed_row = 0;
		row = 0;
		current_ptr = fst_ptr;
		while(current_ptr != NULL){
			if (strcmp(current_ptr->select, UNSELECT) != 0){
				add_rows[0] = current_ptr->select;
			}else{
				add_rows[0] = UNSELECT;
			}
			add_rows[1] = current_ptr->file;
			add_rows[2] = current_ptr->mtype;
		
			if (find_mime(current_ptr->mtype)){
				gtk_clist_insert( (GtkCList *) clist_fax, allowed_row, add_rows);
				++allowed_row;
			}else{
				gtk_clist_append( (GtkCList *) clist_fax, add_rows);
				gtk_clist_set_foreground( (GtkCList *) clist_fax, row, &red);
			}
			++row;
			 current_ptr = current_ptr->next_ptr;
		}
	}
	gtk_clist_thaw((GtkCList *)clist_fax);
	
	title = g_strdup_printf("gefax - Directory: %s", fax_dir);
	gtk_window_set_title(GTK_WINDOW(gefax), title);
	g_free(title);
}

void load_dir(void)
{
	extern int find_mime(gchar *name);
	extern gchar *find_command (gchar *name);

	gchar *a_fname;
	gchar *fname;
	gchar *ftype;
	gchar *prog;
	struct dirent **namelist;
	gint no_of_files;
	gint is_file;
	gint count;
	struct FileList *new_item_ptr;

	
   	free_FileList();					/*destroy whole list if any*/
	no_clist_rows = 0;
	total_selected = 0;
	no_of_files = scandir(fax_dir, &namelist, 0, alphasort);
	if (no_of_files < 0)
		perror("scandir");
	else{
		fst_ptr = NULL;
		for (count = no_of_files - 1; count > 1; --count) {
			fname = g_strdup(namelist[count]->d_name);
			a_fname = g_strdup_printf("%s%s", fax_dir, fname);
			is_file = g_file_test(a_fname, G_FILE_TEST_ISFILE);
			if ((is_file == 1) && (fname[0] != '.')){
				new_item_ptr = g_malloc(sizeof(struct FileList));
				ftype = g_strdup(gnome_mime_type_or_default_of_file(a_fname, "UNKNOWN"));
				
				/*for some reason gnome_mime_type_or_default_of_file() returns "PDF document"
				instead of "application/pdf on my system. maybe installing acroread changed it??*/
				if (strcmp(ftype, "PDF document") == 0){
					g_free(ftype);
					ftype = g_strdup("application/pdf");
				}
				if (((find_mime(ftype) == 1) && (load_all == 0)) || (load_all == 1)){
					new_item_ptr->select = g_strdup(UNSELECT);
					new_item_ptr->file = g_strdup(fname);
					new_item_ptr->mtype = g_strdup(ftype);
					prog = g_strdup(find_command(ftype));
					new_item_ptr->program = prog;
					new_item_ptr->next_ptr = fst_ptr;
					fst_ptr = new_item_ptr;
					++no_clist_rows;
				}
			}
			g_free(a_fname);
			g_free(fname);
		}
	}
	/*load files in clist*/
	if (ActiveComponent == BROWSER_ON) load_clist_fax();
	return;
}

/*________________________________________Functions for struct FileList______________________________*/

void set_all_unselected(void)
{
	struct FileList *current_ptr;

	current_ptr = fst_ptr;
	while (current_ptr != NULL){
		g_free(current_ptr->select);
		current_ptr->select = g_strdup(UNSELECT);
		current_ptr = current_ptr->next_ptr;
	}
	total_selected = 0;

}

void set_all_selected(void)
{
	extern int find_mime(gchar *name);
	struct FileList *current_ptr;
	current_ptr = fst_ptr;
	while ((current_ptr != NULL) && (total_selected < 9)){						/*fell off the end*/
		if (((find_mime(current_ptr->mtype)) != 0) && (strcmp(current_ptr->select, UNSELECT) == 0)){
			++total_selected;
			g_free(current_ptr->select);
			current_ptr->select = g_strdup_printf("%d",total_selected);
		}
		current_ptr = current_ptr->next_ptr;
	}

}

/*
 *sets selected to X given file name
 *Returns 1 if the row was selected 2 if not 0 on err
*/
gint set_selected(gchar *fname)
{
	extern int find_mime(gchar *name);
	int ret = 0;
	struct FileList *current_ptr;
	
	current_ptr = fst_ptr;
	while (current_ptr != NULL){							/*fell off the end*/
		if (strcmp(current_ptr->file, fname) == 0){
			if (strcmp(current_ptr->select, UNSELECT) != 0){
				reorder_select_list(fname);
				g_free(current_ptr->select);
				current_ptr->select = g_strdup(UNSELECT);
				--total_selected;
				ret = 1;	
				break;
			}else{
				if ((find_mime(current_ptr->mtype) == 1) && (total_selected < 9)){
					++total_selected;
					g_free(current_ptr->select);
					current_ptr->select = g_strdup_printf("%d",total_selected);
					ret = 0;	
					break;
				}else{
					ret = 1;
					break;
				}
			}
		}
		current_ptr = current_ptr->next_ptr;
	}

	return (ret);
}

void reorder_select_list(gchar *fname)
{
	struct FileList *current_ptr;
	gint num, bad_num;
	
	/*get the number of the file*/
	num = 0;
	current_ptr = fst_ptr;
	while (current_ptr != NULL){
		if (strcmp(current_ptr->file, fname) == 0){
			num = atoi(current_ptr->select);
		}
		current_ptr = current_ptr->next_ptr;
	}
	/*adjust all above it*/
	if(num > 0){
		current_ptr = fst_ptr;
		while (current_ptr != NULL){
			if ((strcmp(current_ptr->file, fname) != 0) && (strcmp(current_ptr->select, UNSELECT) != 0)){
				bad_num = atoi(current_ptr->select);
				if(bad_num >= num){
					--bad_num;
					g_free(current_ptr->select);
					current_ptr->select = g_strdup_printf("%d",bad_num);
				}
			}
			current_ptr = current_ptr->next_ptr;
		}
	}
}


gint is_row_selected(gchar *file_name)
{
	struct FileList *current_ptr;
	int ok;
	
	ok = 0;
	current_ptr = fst_ptr;
	while (current_ptr != NULL){
		if (strcmp(current_ptr->file, file_name) == 0){
			if(strcmp(current_ptr->select, UNSELECT) != 0){
				ok = 1;
				break;
			}else{
				ok = 0;
				break;
			}
		}
		current_ptr = current_ptr->next_ptr;
	}
	return(ok);
}

void free_FileList(void)
{
	struct FileList *next;
	
	while (fst_ptr != NULL){
		next = fst_ptr->next_ptr;
		g_free(fst_ptr->select);
		g_free(fst_ptr->file);
		g_free(fst_ptr->mtype);
		g_free(fst_ptr->program);
		g_free(fst_ptr);
		fst_ptr = next;
	}
	return;
}

/*________________________________________Functions for struct phone______________________________*/


gint has_phone_num(gchar *phone_number)
{
	struct PhoneNumber *current_ptr;
	int ok;
	
	ok = 0;
	current_ptr = phone_fst_ptr;
	while (current_ptr != NULL){
		if (strcmp(current_ptr->phone, phone_number) == 0){
			ok = 1;
			break;
		}else{
			ok = 0;
		}
		current_ptr = current_ptr->next_ptr;
	}
	return(ok);
}


gint add_phone(gchar *number, gchar *person)
{
	gint ok;
	gchar *num[2];
	struct PhoneNumber *new_phone_ptr;
	
	if ((ok = has_phone_num(number)) == 0){
		new_phone_ptr = g_malloc(sizeof(struct PhoneNumber));
		new_phone_ptr->phone = g_strdup(number);
		new_phone_ptr->person = g_strdup(person);
		new_phone_ptr->next_ptr = phone_fst_ptr;
		phone_fst_ptr = new_phone_ptr;
		num[0] = person;
		num[1] = number;
		gtk_clist_prepend( (GtkCList *) clist_nums,num );
		 ++no_phone_nums;
	}
	return (ok);
}

/*
 *This is pretty huge any better way to do it? took me ages to hack this up
 *phone numbers kept in order
*/
gint remove_phone(gchar *number)
{
	gint ok;
	gchar *pnum;
	gint row;
	struct PhoneNumber *phone_current_ptr;
	struct PhoneNumber *phone_nxt_ptr;
	struct PhoneNumber *phone_prev_ptr;

	ok = 0;
	phone_current_ptr = phone_fst_ptr;
	if (strcmp(phone_current_ptr->phone, number) == 0){	/*Number is first*/
		if (phone_current_ptr->next_ptr != NULL){		/*there are more than 1 item*/
			phone_fst_ptr = phone_fst_ptr->next_ptr; /*move first to next before remove*/
			g_free(phone_current_ptr->phone);
			g_free(phone_current_ptr->person);
			g_free(phone_current_ptr);
			ok = 1;
		}else{
			g_free(phone_current_ptr->phone);
			g_free(phone_current_ptr->person);
			g_free(phone_current_ptr);
			phone_fst_ptr = NULL;
			no_phone_nums = 0;
			ok = 1;
		}
	}else{	/*search through the rest*/
		phone_prev_ptr = phone_fst_ptr;			/*so we can link back up*/
		phone_current_ptr = phone_fst_ptr->next_ptr; /*start at second in list*/
		while (phone_current_ptr != NULL){
			if (strcmp(phone_current_ptr->phone, number) == 0){
				phone_nxt_ptr = phone_current_ptr->next_ptr;
				g_free(phone_current_ptr->phone);
				g_free(phone_current_ptr->person);
				g_free(phone_current_ptr);
				if (phone_nxt_ptr != NULL){		/*it was not the last*/
					phone_prev_ptr->next_ptr = phone_nxt_ptr; /*link the gap*/
				}else{
					phone_prev_ptr->next_ptr = NULL;		/*so we know where the end is*/
				}
				ok = 1;
				break;
			}
			phone_current_ptr = phone_current_ptr->next_ptr;	/*increment to next*/
			phone_prev_ptr = phone_prev_ptr->next_ptr;
		}
	}
	
	/*remove the number from the list */
	if(ok){
		for (row = 0; row < 1000; ++row){
			gtk_clist_get_text( (GtkCList *)clist_nums, row, 1, &pnum);
			if (strcmp(number, pnum) == 0){
				gtk_clist_remove( (GtkCList *)clist_nums, row);
				break;
			}
		}
	}
	return (ok);
}




/*end*/









