/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <ctype.h>
#include <gnome.h>
#include <dirent.h>
#include <gefax/gefax.h>

#include "control.h"
#include "gefax.h"
#include "allow_mime.h"
#include "exec_cmd.h"
#include "about.h"
#include "dialog_dir.h"
#include "dialog_num.h"
#include "dialog_receive.h"
#include "setup.h"
#include "browser.h"
#include "support.h"
#include "viewer.h"
#include "hist.h"
#include "address_b.h"

 
static GnomeHelpMenuEntry help_ref = {"gefax", "index.html"};

static void free_data_lists(void);

void on_startup_execute(GtkWidget *gefax)
{
	read_mime_file();
	address_book_load_config ();
	history_load_config ();
	fax_viewer_load_yourself ();
	/*catch exit signals*/
	if(setup_sig_handler()){
		switch(ActiveComponent){
			case HISTORY_ON:
				create_hist_viewer();
				break;
			case CONTACTS_ON:
				create_address_book();
				break;
			default:
				create_file_browser();
				/*start waiting if we were waiting when "save_state" or user exited while waiting and selected OK*/
				if (receive_on) receive_fax(TRUE,NULL,TRUE);
				break;
		}
	}else{
		g_error("Can't continue failed to setup a signal handler\n");
	}
	
}



void on_mnu_about_activate(GtkMenuItem *menuitem, gpointer user_data)
{
	create_about_gefax();
}



void on_btn_send_fax_clicked(GtkButton *button, gpointer user_data)
{
	if(efax_status()) return;
	
	if (no_phone_nums > 0){
		send_fax(FALSE);	/*all happens in exec_cmd.c*/
	}
}

void on_btn_receive_fax_clicked(GtkButton *button, gpointer user_data)
{
	if(efax_status()) return;
	
	/*create dialog receive - it will do the rest*/
	create_dialog_receive();
}

void on_btn_poll_fax_clicked(GtkButton *button, gpointer user_data)
{
	
	if(efax_status()) return;
	create_dialog_fax_number(TRUE);
	return;
}


void on_btn_options_clicked(GtkButton *button, gpointer user_data)
{
  	create_dialog_prop ();
}


void on_btn_help_clicked(GtkButton *button, gpointer user_data)
{
    gnome_help_display("ignored", &help_ref);

}

void free_data_lists(void)
{
	free_FileList();
	free_allowed_mime();
	delete_temp_files();
	if (auto_save_cardfile) {
		address_book_save_yourself (TRUE);
	} else {
		address_book_save_yourself (FALSE);
	}
	fax_viewer_save_yourself ();
	history_save_config ();	
}

void on_btn_exit_clicked(GtkButton *button, gpointer user_data)
{
	GtkWidget *action_dialog;
	gint user_action;
	gint status;
	gchar *msg = NULL;

	status = efax_status();
	/*Kill processes if any and find out if we are waiting for a fax
	 so we can automatically start waiting on startup*/
	if (status == WAIT_M){
		msg = g_strdup_printf(_("Gefax is waiting for a fax\n\n"
							"Do you want gefax to automatically start\n"
							"receiving the next time you run it?"));
		action_dialog = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_QUESTION, GNOME_STOCK_BUTTON_OK,
						GNOME_STOCK_BUTTON_CLOSE, GNOME_STOCK_BUTTON_CANCEL, NULL);
		gtk_window_set_position (GTK_WINDOW (action_dialog), GTK_WIN_POS_MOUSE);
		user_action = gnome_dialog_run_and_close(GNOME_DIALOG(action_dialog));
		g_free(msg);
		msg = NULL;
		if (user_action == 0){
			gtk_widget_hide(gefax);
			receive_on = TRUE;
			save_config();
			free_data_lists();
			kill_fax_operation(NULL,(gint *)EXIT_NOW);
		}else if (user_action == 1){
			gtk_widget_hide(gefax);
			receive_on = FALSE;
			save_config();
			free_data_lists();
			kill_fax_operation(NULL,(gint *)EXIT_NOW);			
		}
	}else{
		receive_on = FALSE;
		if (status == RECEIVING){
			msg = g_strdup_printf(_("Cancel receive fax and Quit?"));
		}else if (status == SENDING){
			msg = g_strdup_printf(_("Cancel send fax and Quit?"));
		}else if (status == POLLING){
			msg = g_strdup_printf(_("Cancel poll fax and Quit?"));
		}else if (status == WAIT){
			msg = g_strdup_printf(_("Cancel waiting for fax and Quit?"));
		}else{
			save_config();
			free_data_lists();
			gtk_main_quit();
			return;
		}
		action_dialog = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_QUESTION, GNOME_STOCK_BUTTON_OK,
							GNOME_STOCK_BUTTON_CANCEL, NULL);
		gtk_window_set_position (GTK_WINDOW (action_dialog), GTK_WIN_POS_MOUSE);
		user_action = gnome_dialog_run_and_close(GNOME_DIALOG(action_dialog));
			g_free(msg);
			msg = NULL;
		if (user_action == 0){
			save_config();
			gtk_widget_hide(gefax);
			free_data_lists();
			kill_fax_operation(NULL,(gint *)EXIT_NOW);
		}
	}
}


void on_btn_view_clicked(GtkButton *button, gpointer user_data)
{
	if(efax_status()) return;
	
	change_to_viewer(NULL,NULL);
	send_fax(TRUE);	/*all happens in exec_cmd.c*/
}

void on_btn_open_clicked (GtkButton *button, gpointer user_data)
{

	create_dialog_dir (fax_dir);
}


void change_to_browser(GtkWidget *w, gpointer data)
{
	if(efax_status()) return;
	
	switch(ActiveComponent){
		case HISTORY_ON:
			history_destroy_yourself();
			create_file_browser();
			break;
		case VIEWER_ON:
			remove_image_viewer();
			create_file_browser();
			break;
		case CONTACTS_ON:
			remove_address_book();
			create_file_browser();
			break;
		default:
			gdk_beep();
			break;
	}
}
void change_to_history(GtkWidget *w, gpointer data)
{
	if(efax_status()) return;
	
	switch(ActiveComponent){
		case BROWSER_ON:
			remove_file_browser();
			create_hist_viewer();
			break;
		case VIEWER_ON:
			remove_image_viewer();
			create_hist_viewer();
			break;
		case CONTACTS_ON:
			remove_address_book();
			create_hist_viewer();
			break;
		default:
			gdk_beep();
			break;
	}
}

void change_to_viewer(GtkWidget *w, gpointer data)
{
	if(efax_status()) return;
	
	switch(ActiveComponent){
		case BROWSER_ON:
			remove_file_browser();
			create_image_viewer(data);
			break;
		case HISTORY_ON:
			history_destroy_yourself();
			create_image_viewer(data);
			break;
		case CONTACTS_ON:
			remove_address_book();
			create_image_viewer(data);
			break;
		default:
			gdk_beep();
			break;
	}
	gtk_window_set_title(GTK_WINDOW(gefax),"gefax - Fax Viewer");
}

void change_to_contacts(GtkWidget *w, gpointer data)
{
	if(efax_status()) return;
	
		switch(ActiveComponent){
		case BROWSER_ON:
			remove_file_browser();
			create_address_book();
			break;
		case HISTORY_ON:
			history_destroy_yourself();
			create_address_book();
			break;
		case VIEWER_ON:
			remove_image_viewer();
			create_address_book();
			break;
		default:
			gdk_beep();
			break;
	}
}



/*end*/





