/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>

#include <gnome.h>

#include "dialog_dir.h"
#include "support.h"
#include "exec_cmd.h"
#include "gefax.h"

extern gchar *fax_dir;
extern gchar *home;

/* XPM */
static char *folder[] = {
"16 16 9 1",
"g c #808080",
"b c #ffa858",
"e c #c0c0c0",
"# c #000000",
"c c #ffdca8",
". c None",
"a c #585858",
"f c #a0a0a4",
"d c #ffffff",
"..#a#...........",
".#abc##.........",
".#daabc#####....",
".#ddeaabcccb#...",
".#dedeeabccca...",
".#edeeeeaaaab#..",
".#deeeeeeefe#ba.",
".#eeeeeeefef#ba.",
".#eeeeeefeff#ba.",
".#eeeeefefff#ba.",
".##geefeffff#ba.",
"...##gefffff#ba.",
".....##fffff#ba.",
".......##fff#b##",
".........##f#b##",
"...........####."};

static GtkWidget *dialog_dir = NULL;
static GtkWidget *btn_dir_bakk;
static GtkWidget *clist_dir;
static GtkWidget *entry_dir_loc;
static gchar *dname = NULL;
static gchar *prev = NULL;
static gboolean show_hidden_files = FALSE;

extern void load_dir(void);

static void load_clist_dir(void);
static void on_check_all_files_toggled(GtkToggleButton *togglebutton, gpointer user_data);
static void on_btn_dir_up_clicked(GtkButton *button, gpointer btn);
static void on_btn_dir_back_clicked(GtkButton *button, gpointer btn);
static void on_btn_dir_home_clicked (GtkButton *button, gpointer btn);
static void on_btn_dir_refresh_clicked (GtkButton *button, gpointer user_data);
static void on_btn_dir_ok_clicked(GtkButton *button, gpointer user_data);
static void on_btn_dir_cancel_clicked(GtkButton *button, gpointer user_data);
static void on_clist_dir_select_row(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer btn);

void on_btn_dir_up_clicked(GtkButton *button, gpointer btn)
{	
	gint len;
	const gchar sep = '/';
	gchar *tmp;

	len = (strlen(dname) - 1);
	tmp = g_strdup(dname);
	if (len > 1){
		tmp[len] = '\0';
		while ((len > 0) && (sep != tmp[len])){
			--len;
		}
		tmp[len + 1] = '\0';
		if (prev != NULL) g_free(prev);
		prev = g_strdup(dname);
		gtk_widget_set_sensitive (btn_dir_bakk, TRUE);
		g_free(dname);
		dname = g_strdup(tmp); 	 
		g_free(tmp);
		load_clist_dir();
	}
}

void on_btn_dir_back_clicked (GtkButton *button, gpointer btn)
{
	gtk_widget_set_sensitive (btn_dir_bakk, FALSE);
	g_free(dname);
	dname = g_strdup(prev);
	g_free(prev);
	prev = NULL;
	load_clist_dir();
}

void on_btn_dir_home_clicked(GtkButton *button, gpointer btn)
{
	if (prev != NULL) g_free(prev);
	prev = g_strdup(dname);
	if (dname != NULL) g_free(dname);
	dname = g_strdup_printf("%s/", home);
	load_clist_dir();
	gtk_widget_set_sensitive (btn_dir_bakk, TRUE);
}

void on_btn_dir_refresh_clicked (GtkButton *button, gpointer user_data)
{
	load_clist_dir();
}

void on_btn_dir_ok_clicked (GtkButton *button, gpointer user_data)
{
	g_free(fax_dir);
	fax_dir = g_strdup(dname);

	if (prev != NULL) g_free(prev);
	prev = NULL;
	g_free(dname);
	dname = NULL;
	gtk_widget_destroy(dialog_dir);
	dialog_dir = NULL;
	
	/*let the dialog box close quickly while loading file names etc*/
	while (gtk_events_pending()) gtk_main_iteration();

	load_dir();
	return;
}

void on_btn_dir_cancel_clicked(GtkButton *button, gpointer user_data)
{
	gtk_widget_destroy(dialog_dir);
	dialog_dir = NULL;
	
	if (prev != NULL){
		g_free(prev);
		prev = NULL;
	}
	if (dname != NULL){
		g_free(dname);
		dname = NULL;
	}
}

void on_clist_dir_select_row(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer btn)
{
	gchar *tmp;
	gchar *tmp2;
	gchar *dir_name;
	GdkPixmap *pmap;
	GdkBitmap *mask;
	
	if ((event->type == GDK_2BUTTON_PRESS) && (event->button == 1 )){
		/*for some reason we have to get the pixmap & mask or segfault?? - not according to gtkclist.h*/
		gtk_clist_get_pixtext( (GtkCList *)clist_dir, row, 0, &dir_name, NULL, &pmap, &mask);
	
#		ifdef DEBUG
			printf("DEBUG:clist_dir text=%s\n", dir_name);
#		endif DEBUG

		if (strcmp(dir_name, "..") == 0){
			on_btn_dir_up_clicked(NULL,NULL);
		}else{
			if (strcmp(dir_name, ".") != 0){
				tmp2 = g_strdup(dname);
				if (prev != NULL) g_free(prev);
				prev = g_strdup(dname);
				gtk_widget_set_sensitive (btn_dir_bakk, TRUE);
				g_free(dname);
				tmp = g_strdup_printf("%s%s/", tmp2, dir_name);
				dname = g_strdup(tmp);
				g_free(tmp);
				load_clist_dir();
			}
		}
	}
}

void load_clist_dir(void)
{
	extern GtkWidget *gefax;
	gint no_files;
	gint fcount;
	gint ok;
	gint row;
	gchar *tmp;
	gchar *add_rows[1];
	struct dirent **filelist;
	GdkPixmap *pmap;
	GdkBitmap *mask;
	GtkStyle *style;
	
	style = gtk_widget_get_style(GTK_WIDGET(gefax));
	pmap = gdk_pixmap_create_from_xpm_d(gefax->window,&mask,&style->bg[GTK_STATE_NORMAL],folder);
	
	gtk_clist_freeze( (GtkCList *) clist_dir);
	gtk_clist_clear( (GtkCList *) clist_dir);
	no_files = scandir(dname, &filelist, 0,alphasort ); /*count num of files*/
	if (show_hidden_files){
		row = 0;
		for (fcount = 0; fcount < no_files; ++fcount){
			tmp = g_strdup_printf("%s%s",dname,filelist[fcount]->d_name);
			ok = g_file_test(tmp, G_FILE_TEST_ISDIR);
			if ((ok == TRUE) && (filelist[fcount]->d_name[0] == '.')){
				add_rows[0]= filelist[fcount]->d_name;
				gtk_clist_append( (GtkCList *) clist_dir, add_rows);
				++row;			
			}
			g_free(tmp);
		}
	}else{ /*add .and ..*/
		row = 0;
		add_rows[0]= ".";
		gtk_clist_append( (GtkCList *) clist_dir, add_rows);
		add_rows[0]= "..";
		gtk_clist_append( (GtkCList *) clist_dir, add_rows);
		row += 2;
	}
	for (fcount = 0; fcount < no_files; ++fcount){
		
		tmp = g_strdup_printf("%s%s",dname,filelist[fcount]->d_name);
		ok = g_file_test(tmp, G_FILE_TEST_ISDIR);
		if ((ok == TRUE) && (filelist[fcount]->d_name[0] != '.')){
			add_rows[0]= filelist[fcount]->d_name;
			gtk_clist_append( (GtkCList *) clist_dir, add_rows);
			++row;			
		}
		g_free(tmp);
	}
	/*add folder pixmap*/
	for (fcount = 0; fcount < row; ++fcount){
		gchar *text;
		gchar *txt;
		gtk_clist_get_text(GTK_CLIST(clist_dir), fcount,0,&text);
		txt = g_strdup(text);
		gtk_clist_set_pixtext (GTK_CLIST(clist_dir), fcount, 0,txt,8, pmap, mask);
		g_free(txt);
	}
	gtk_clist_thaw( (GtkCList *) clist_dir);
	gtk_entry_set_text(GTK_ENTRY(entry_dir_loc), dname);
}

void on_check_all_files_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	if (GTK_TOGGLE_BUTTON(togglebutton)->active){
		show_hidden_files = TRUE;
		load_clist_dir();
	}else{
		show_hidden_files = FALSE;
		load_clist_dir();
	}
}

void create_dialog_dir(gchar *dir)
{
	GtkWidget *dock1;
	GtkWidget *toolbar_dock_dir;
	GtkWidget *toolbar1;
	GtkWidget *tmp_toolbar_icon;
	GtkWidget *btn_dir_open;
	GtkWidget *btn_dir_home;
	GtkWidget *btn_dir_refresh;
	GtkWidget *vbox2;
	GtkWidget *scrolledwindow1;
	GtkWidget *label5;
	GtkWidget *check_all_files;
	GtkWidget *hbox1;
	GtkWidget *label4;
	GtkWidget *hseparator1;
	GtkWidget *hbuttonbox1;
	GtkWidget *btn_dir_ok;
	GtkWidget *btn_dir_cancel;
	GtkTooltips *tooltips;
	gint sts = efax_status();
	
	if(dialog_dir){
		gdk_window_raise(dialog_dir->window);
		return;
	}
	if ((sts == SENDING) || (sts == EFIX_C) || (sts == GS_C)) return;
	
	dname = g_strdup(dir);
	tooltips = gtk_tooltips_new ();

	dialog_dir = gnome_app_new ("Dialog_dir", _("gefax: Open Directory"));
	GTK_WINDOW (dialog_dir)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_transient_for(GTK_WINDOW (dialog_dir),GTK_WINDOW (gefax));

	dock1 = GNOME_APP (dialog_dir)->dock;
	gtk_widget_show (dock1);
	
	toolbar_dock_dir = gnome_dock_item_new("toolbar_dock_dir",GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL | GNOME_DOCK_ITEM_BEH_LOCKED | GNOME_DOCK_ITEM_BEH_EXCLUSIVE);
	gtk_widget_show (toolbar_dock_dir);
	gnome_dock_add_item (GNOME_DOCK (dock1), GNOME_DOCK_ITEM(toolbar_dock_dir), GNOME_DOCK_TOP, 1, 0, 0,TRUE);

	toolbar1 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	gtk_widget_show (toolbar1);
	gtk_container_set_border_width (GTK_CONTAINER (toolbar1), 1);
	gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar1), 18);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar1), GTK_RELIEF_NONE);
	gtk_container_add (GTK_CONTAINER (toolbar_dock_dir), toolbar1);
	gnome_dock_item_set_shadow_type(GNOME_DOCK_ITEM(toolbar_dock_dir), GTK_SHADOW_NONE);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (dialog_dir, GNOME_STOCK_PIXMAP_BACK);
	btn_dir_bakk = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Back"),
                                _("Previous Directory"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_set_sensitive (btn_dir_bakk, FALSE);
	gtk_widget_show (btn_dir_bakk);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (dialog_dir, GNOME_STOCK_PIXMAP_UP);
	btn_dir_open = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Open"),
                                _("Parent Directory"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (btn_dir_open);

	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar1));

	tmp_toolbar_icon = gnome_stock_pixmap_widget (dialog_dir, GNOME_STOCK_PIXMAP_HOME);
	btn_dir_home = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Home"),
                                _("Home Directory"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (btn_dir_home);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (dialog_dir, GNOME_STOCK_PIXMAP_REFRESH);
	btn_dir_refresh = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Refresh"),
                                _("Refresh"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (btn_dir_refresh);
  
	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar1));
  
	label5 = gtk_label_new ("");
	gtk_widget_show (label5);
	gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar1), label5, NULL, NULL);
	gtk_widget_set_usize (label5, 10, -2);

	check_all_files = gtk_check_button_new_with_label (_("Show Hidden Directories"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_all_files), show_hidden_files);
	gtk_widget_show (check_all_files);
	gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar1), check_all_files, NULL, NULL);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox2);
	gnome_app_set_contents (GNOME_APP (dialog_dir), vbox2);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow1, TRUE, TRUE, 0);
	gtk_widget_set_usize (scrolledwindow1, -2, 280);
	gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow1), 6);

	clist_dir = gtk_clist_new (1);
	gtk_widget_show (clist_dir);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), clist_dir);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_clist_set_shadow_type (GTK_CLIST (clist_dir), GTK_SHADOW_IN);
	gtk_tooltips_set_tip (tooltips, clist_dir, _("Select A Directory"), NULL);
	gtk_clist_set_column_width (GTK_CLIST (clist_dir), 0, 80);
	gtk_clist_column_titles_show (GTK_CLIST (clist_dir));
	gtk_clist_set_column_title (GTK_CLIST (clist_dir),0,_("Directory"));
	gtk_clist_column_titles_passive(GTK_CLIST (clist_dir));
	gtk_clist_set_row_height (GTK_CLIST (clist_dir),18);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);

	label4 = gtk_label_new (_("Selection:"));
	gtk_widget_show (label4);
	gtk_box_pack_start (GTK_BOX (hbox1), label4, FALSE, FALSE, 0);
	gtk_misc_set_alignment (GTK_MISC (label4), 0.06, 0.5);
	gtk_misc_set_padding (GTK_MISC (label4), 7, 0);

	entry_dir_loc = gtk_entry_new ();
	gtk_widget_show (entry_dir_loc);
	gtk_box_pack_start (GTK_BOX (hbox1), entry_dir_loc, TRUE, TRUE, 6);
	gtk_entry_set_editable (GTK_ENTRY (entry_dir_loc), FALSE);
	gtk_entry_set_text (GTK_ENTRY (entry_dir_loc), dir);

	hseparator1 = gtk_hseparator_new ();
	gtk_widget_show (hseparator1);
	gtk_box_pack_start (GTK_BOX (vbox2), hseparator1, TRUE, TRUE, 4);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (vbox2), hbuttonbox1, TRUE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox1), 5);
	gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox1), 85, 10);

	btn_dir_ok = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
	gtk_widget_show (btn_dir_ok);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), btn_dir_ok);
	GTK_WIDGET_SET_FLAGS (btn_dir_ok, GTK_CAN_DEFAULT);

	btn_dir_cancel = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
	gtk_widget_show (btn_dir_cancel);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), btn_dir_cancel);
	GTK_WIDGET_SET_FLAGS (btn_dir_cancel, GTK_CAN_DEFAULT);

	gtk_signal_connect_object (GTK_OBJECT (dialog_dir), "destroy",
					GTK_SIGNAL_FUNC (on_btn_dir_cancel_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_dir_bakk), "clicked",
					GTK_SIGNAL_FUNC (on_btn_dir_back_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_dir_open), "clicked",
					GTK_SIGNAL_FUNC (on_btn_dir_up_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_dir_home), "clicked",
					GTK_SIGNAL_FUNC (on_btn_dir_home_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_dir_refresh), "clicked",
					GTK_SIGNAL_FUNC (on_btn_dir_refresh_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_dir_ok), "clicked",
					GTK_SIGNAL_FUNC (on_btn_dir_ok_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_dir_cancel), "clicked",
					GTK_SIGNAL_FUNC (on_btn_dir_cancel_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_dir), "select_row",
					GTK_SIGNAL_FUNC (on_clist_dir_select_row), NULL);
	gtk_signal_connect (GTK_OBJECT (check_all_files), "toggled",
					GTK_SIGNAL_FUNC (on_check_all_files_toggled), NULL);

	gtk_object_set_data (GTK_OBJECT (dialog_dir), "tooltips", tooltips);


	load_clist_dir();
	gtk_widget_show(dialog_dir);
	return;
}


/*end*/





