/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <gnome.h>
#include <gefax/gefax.h>

#include "dialog_num.h"
#include "exec_cmd.h"
#include "gefax.h"
#include "address_b.h"
#include "browser.h"
#include "exec_cmd.h"

/* XPM */
static char * UncheckedClistXpm[] = {
"12 12 2 1",
" 	c None",
".	c #000000",
"............",
".          .",
".          .",
".          .",
".          .",
".          .",
".          .",
".          .",
".          .",
".          .",
".          .",
"............"};

/* XPM */
static char * CheckedClistXpm[] = {
"12 12 2 1",
" 	c None",
".	c #000000",
"............",
".          .",
".        . .",
".       .. .",
".      ... .",
". .   ...  .",
". .. ...   .",
". .....    .",
".  ...     .",
".   .      .",
".          .",
"............"};

static GtkWidget *dialog_num;
static GtkWidget *clist_numbers;
static GtkWidget *table1;
static GtkWidget *entry_r_number;
static GtkWidget *entry_r_name;
static gint total_rows;
static gboolean polling;

static GnomeHelpMenuEntry numbers_help_ref =
{
	"gefax",
	"fax.html#sr"
};

static GnomeHelpMenuEntry numbers_poll_help_ref =
{
	"gefax",
	"fax.html#pfax"
};

static void on_tog_from_address_toggled(GtkToggleButton *togglebutton, gpointer user_data);
static void on_btn_num_OK_clicked(GtkButton *button,  gpointer tb);
static void on_btn_num_cancel_clicked(GtkButton *button,  gpointer data);
static void on_btn_num_help_clicked(GtkButton *button,  gpointer data);
static void load_clist_numbers_with_pixtext(GtkCList *clist_numbers);
static void set_clist_numbers_row(GtkCList *clist, gint row, gint column, GdkEvent *event, GtkWidget *W);
static void unset_clist_numbers_row(GtkCList *clist, gint row, gint column, GdkEvent *event, GtkWidget *W);

void on_btn_num_help_clicked(GtkButton *button,  gpointer data)
{
	if (polling)
		gnome_help_display("ignored", &numbers_poll_help_ref);
	else
		gnome_help_display("ignored", &numbers_help_ref);
}

void on_btn_num_OK_clicked(GtkButton *button,  gpointer tb)
{
	gchar *number, *name, *tmp;
	gint i;
	
	if (GTK_TOGGLE_BUTTON(tb)->active){
		for (i = 0; i < total_rows; i++) {
			gtk_clist_get_text(GTK_CLIST(clist_numbers), i,3,&tmp);
			if (strcmp(tmp,"1") == 0){
				gtk_clist_get_text(GTK_CLIST(clist_numbers), i,2,&tmp);
				number = g_strdup(tmp);
				if(polling){
					poll_fax(number);
					break;
				}else{
					gtk_clist_get_text(GTK_CLIST(clist_numbers), i,1,&tmp);
					name = g_strdup(tmp);
					if(!add_phone(number,name)){
					/*error msg*/
					}
					g_free(number);
					g_free(name);
				}
			}
		}
	}else{
		number = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_r_number)));
		if (number[0] == '\0'){
			g_free(number);
			return;
		}
		if(polling){
			poll_fax(number);
		}else{
			name = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_r_name)));
			if(!add_phone(number,name)){
				/*error msg*/
			}
			g_free(number);
			g_free(name);
				
		}
	}
	on_btn_num_cancel_clicked(NULL,NULL);
}

static void on_btn_num_cancel_clicked(GtkButton *button,  gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(dialog_num));
	dialog_num = NULL;
}

void create_dialog_fax_number(gboolean poll)
{
	GtkWidget *dialog_vbox;
	GtkWidget *lbl_r_number;
	GtkWidget *lbl_r_name;
	GtkWidget *tog_from_address;
	GtkWidget *vbox_address;
	GtkWidget *lbl_select;
	GtkWidget *sw_ab;
	GtkWidget *dialog_action_area;
	GtkWidget *btn_ok;
	GtkWidget *btn_cancel;
	GtkWidget *btn_help;
  	gint sts = efax_status();
	
	if ((sts == SENDING) || (sts == EFIX_C) || (sts == GS_C)) return;
	if (dialog_num){
		gdk_window_raise(dialog_num->window);
		return;
	}

	polling = poll;

	if(poll)
		dialog_num = gnome_dialog_new (_("Gefax - Select number to poll"), NULL);
	else
		dialog_num = gnome_dialog_new (_("Gefax - Select fax recipent/s"), NULL);
		
	gtk_window_set_transient_for(GTK_WINDOW (dialog_num),GTK_WINDOW (gefax));

	dialog_vbox = GNOME_DIALOG (dialog_num)->vbox;
	gtk_widget_show (dialog_vbox);
	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), 5);

	table1 = gtk_table_new (2, 2, FALSE);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox), table1, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 5);
	gtk_table_set_col_spacings (GTK_TABLE (table1), 5);

	lbl_r_number = gtk_label_new (_("Fax Number:"));
	gtk_widget_show (lbl_r_number);
	gtk_table_attach (GTK_TABLE (table1), lbl_r_number, 0, 1, 1, 2,
		(GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (lbl_r_number), 1, 0.5);
 
	entry_r_number = gtk_entry_new_with_max_length (30);
	gtk_widget_show (entry_r_number);
	gtk_table_attach (GTK_TABLE (table1), entry_r_number, 1, 2, 1, 2,
		(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	if (!poll) {
		entry_r_name = gtk_entry_new_with_max_length (100);
		gtk_widget_show (entry_r_name);
		gtk_table_attach (GTK_TABLE (table1), entry_r_name, 1, 2, 0, 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (0), 0, 0);

		lbl_r_name = gtk_label_new (_("Recipients Name:"));
		gtk_widget_show (lbl_r_name);
		gtk_table_attach (GTK_TABLE (table1), lbl_r_name, 0, 1, 0, 1,
			(GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
			(GtkAttachOptions) (0), 0, 0);
		gtk_misc_set_alignment (GTK_MISC (lbl_r_name), 1, 0.5);
	}

	tog_from_address = gtk_toggle_button_new_with_label (_("Select from address book...."));
	gtk_widget_show (tog_from_address);
	gtk_box_pack_start (GTK_BOX (dialog_vbox), tog_from_address, FALSE, FALSE, 0);

	vbox_address = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (dialog_vbox), vbox_address, TRUE, TRUE, 0);

	lbl_select = gtk_label_new (_("Select available recipients:"));
	gtk_box_pack_start (GTK_BOX (vbox_address), lbl_select, FALSE, FALSE, 0);
	gtk_misc_set_alignment (GTK_MISC (lbl_select), 1.49012e-08, 0.5);
	gtk_widget_show (lbl_select);

	sw_ab = gtk_scrolled_window_new (NULL, NULL);
	gtk_box_pack_start (GTK_BOX (vbox_address), sw_ab, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw_ab), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_widget_show (sw_ab);
	clist_numbers = gtk_clist_new (4);
	gtk_container_add (GTK_CONTAINER (sw_ab), clist_numbers);
	gtk_widget_show (clist_numbers);
	gtk_widget_set_usize (clist_numbers, -2, 180);
	gtk_clist_set_column_width (GTK_CLIST (clist_numbers), 0, 15);
	gtk_clist_set_shadow_type (GTK_CLIST (clist_numbers), GTK_SHADOW_IN);
	gtk_clist_set_column_auto_resize(GTK_CLIST (clist_numbers),1,TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST (clist_numbers),2,TRUE);
	gtk_clist_set_column_width (GTK_CLIST (clist_numbers), 3, 0);
	gtk_clist_column_titles_hide (GTK_CLIST (clist_numbers));
	gtk_clist_set_column_visibility (GTK_CLIST (clist_numbers),3,FALSE);
	gtk_clist_set_sort_column (GTK_CLIST (clist_numbers), 1);
	gtk_clist_set_sort_type (GTK_CLIST (clist_numbers), GTK_SORT_ASCENDING);
	gtk_clist_set_auto_sort (GTK_CLIST (clist_numbers), TRUE);

	dialog_action_area = GNOME_DIALOG (dialog_num)->action_area;
	gtk_widget_show (dialog_action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area), GTK_BUTTONBOX_SPREAD);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area), 8);
	gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (dialog_action_area), 0, 0);

	gnome_dialog_append_button (GNOME_DIALOG (dialog_num), GNOME_STOCK_BUTTON_OK);
	btn_ok = g_list_last (GNOME_DIALOG (dialog_num)->buttons)->data;
	gtk_widget_show (btn_ok);
	GTK_WIDGET_SET_FLAGS (btn_ok, GTK_CAN_DEFAULT);

	gnome_dialog_append_button (GNOME_DIALOG (dialog_num), GNOME_STOCK_BUTTON_CANCEL);
	btn_cancel = g_list_last (GNOME_DIALOG (dialog_num)->buttons)->data;
	gtk_widget_show (btn_cancel);

	gnome_dialog_append_button (GNOME_DIALOG (dialog_num), GNOME_STOCK_BUTTON_HELP);
	btn_help = g_list_last (GNOME_DIALOG (dialog_num)->buttons)->data;
	gtk_widget_show (btn_help);

	gtk_signal_connect (GTK_OBJECT (tog_from_address), "toggled",
				GTK_SIGNAL_FUNC (on_tog_from_address_toggled), vbox_address);
	gtk_signal_connect (GTK_OBJECT (dialog_num), "delete_event",
				GTK_SIGNAL_FUNC (on_btn_num_cancel_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_ok), "clicked",
				GTK_SIGNAL_FUNC (on_btn_num_OK_clicked), tog_from_address);
	gtk_signal_connect (GTK_OBJECT (btn_cancel), "clicked",
				GTK_SIGNAL_FUNC (on_btn_num_cancel_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_help), "clicked",
				GTK_SIGNAL_FUNC (on_btn_num_help_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_numbers), "select_row",
				GTK_SIGNAL_FUNC (set_clist_numbers_row), NULL);
	if (poll) gtk_signal_connect (GTK_OBJECT (clist_numbers), "unselect_row",
				GTK_SIGNAL_FUNC (unset_clist_numbers_row), NULL);

	gtk_widget_show(dialog_num);
	load_clist_numbers_with_pixtext(GTK_CLIST(clist_numbers));
	return;
}
	
static gchar
*string_from_phone_types				(LibGefaxTelephone		*T)
{
	const gchar *TEL_TYPES[] = {
		N_(" (Preferred)"), N_(" (Work)"), N_(" (Home)"),
		N_(" (Voice)"), N_(" (Fax)"), N_(" (Message)"),
		N_(" (Cellular)"), N_(" (Pager)"), N_(" (Bulletin Board)"),
		N_(" (Modem)"), N_(" (Car)"), N_(" (ISDN)"), N_(" (Video)")
	};
	const gint TEL_TYPES_LENGTH = 13;
	gint i;
	gchar *string = NULL;
	gchar *tmp = NULL;
	
	for (i = 0; i < TEL_TYPES_LENGTH; i++) {		
		if (T->type & 1 << i) {
			
			tmp = string;
			if (tmp) {
				string = g_strdup_printf ("%s%s", tmp,	TEL_TYPES[i]);
				g_free (tmp);
				tmp = NULL;
			} else {
				string = g_strdup (TEL_TYPES[i]);
			}
		}
	}
	if (tmp) {
		g_free (tmp);
	}
	return string;
}

void
load_clist_numbers_with_pixtext			(GtkCList			*clist_numbers)
{
	GdkPixmap *pmap;
	GdkBitmap *mask;
	GtkStyle *style;
	gchar *add_row[4];
	gint row, i, list_len;
	GList *Ab1, *Ab2;
	
	gtk_clist_freeze(clist_numbers);
	gtk_clist_clear(clist_numbers);
	style = gtk_widget_get_style(GTK_WIDGET(gefax));
	pmap = gdk_pixmap_create_from_xpm_d(gefax->window,&mask,&style->bg[GTK_STATE_NORMAL],UncheckedClistXpm);
	
	total_rows = 0;
	Ab1 = address_book_get_AddressList ();
	Ab2 = lib_gefax_filter_address_book (Ab1, LIBGEFAX_FILTER_FAX, FALSE);
	g_list_free (Ab1);
	Ab1 = Ab2;
	list_len = g_list_length (Ab1);
	for (i = 0; i < list_len; i++) {
		LibGefaxContact *C = LIB_GEFAX_CONTACT (Ab1);
		GSList *Pl = C->Tel;
		gint ll, ii;
		
		ll = g_slist_length (Pl);
		for (ii = 0; ii < ll; ii++) {
			LibGefaxTelephone *T = Pl->data;
			gchar *types_str;
		
			if (T->type & LIBGEFAX_PHONE_FAX) {
				add_row[0] = " ";
				add_row[3] = "0";
				types_str = string_from_phone_types (T);
				add_row[1] = g_strdup_printf("%s %s", C->filename, types_str);
				g_free (types_str);
				add_row[2] = T->number;
				row = gtk_clist_append (clist_numbers, add_row);
				gtk_clist_set_pixmap (clist_numbers, row, 0, pmap, mask);
				g_free (add_row[1]);
				++total_rows;
			}
			Pl = g_slist_next (Pl);
		}
		
		Ab1 = g_list_next (Ab1);
	}
	g_list_free (Ab2);
	gtk_clist_thaw(clist_numbers);
	return;
}

void set_clist_numbers_row(GtkCList *clist, gint row, gint column, GdkEvent *event, GtkWidget *W)
{
	GdkPixmap *pmap;
	GdkBitmap *mask;
	GtkStyle *style;
	gchar *text;
	
	style = gtk_widget_get_style(GTK_WIDGET(gefax));
	gtk_clist_get_text(clist, row, 3, &text);
	if (strcmp(text,"0") == 0){
		pmap = gdk_pixmap_create_from_xpm_d(gefax->window,&mask,&style->bg[GTK_STATE_NORMAL],CheckedClistXpm);
		gtk_clist_set_text (clist,row,3,"1");
	}else {
		pmap = gdk_pixmap_create_from_xpm_d(gefax->window,&mask,&style->bg[GTK_STATE_NORMAL],UncheckedClistXpm);
		gtk_clist_set_text (clist,row,3,"0");
	}
	gtk_clist_set_pixmap (clist, row, 0, pmap, mask);
}

void unset_clist_numbers_row(GtkCList *clist, gint row, gint column, GdkEvent *event, GtkWidget *W)
{
	GdkPixmap *pmap;
	GdkBitmap *mask;
	GtkStyle *style;
	
	style = gtk_widget_get_style(GTK_WIDGET(gefax));
	pmap = gdk_pixmap_create_from_xpm_d(gefax->window,&mask,&style->bg[GTK_STATE_NORMAL],UncheckedClistXpm);
	gtk_clist_set_pixmap (clist, row, 0, pmap, mask);
	gtk_clist_set_text (clist,row,3,"0");
	
}

void
on_tog_from_address_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (togglebutton->active){
		gtk_widget_show(GTK_WIDGET(user_data));
		gtk_widget_set_sensitive(table1, FALSE);
	
	} else {
		gtk_widget_hide(GTK_WIDGET(user_data));
		gtk_widget_set_sensitive(table1, TRUE);
	}
}

/*end*/





