/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gnome.h>

#include "gefax.h"
#include "dialog_receive.h"
#include "exec_cmd.h"


typedef struct _receive_d{
	GtkWidget *dialog_receive;
	GtkWidget *dialog_rec_vbox;
	GtkWidget *vbox;
	GtkWidget *lbl_msg;
	GSList *rec_group;
	GtkWidget *rad_dont_pick_up;
	GtkWidget *hbox;
	GtkObject *spin_answer_no_adj;
	GtkWidget *action_area;
	GtkWidget *btn_ok;
	GtkWidget *btn_cancel;
	GtkWidget *btn_help;
	GtkWidget *rad_pick_up;
	GtkWidget *spin_answer_no;
	GtkWidget *check_lck;
	GtkWidget *lbl_rings;
	guint destroy_signal;
} receive_d;

static GnomeHelpMenuEntry receive_help_ref =
{
	"gefax",
	"fax.html#rfax"
};

/*static gboolean dialog_receive_on = FALSE;*/

static void on_rad_pick_up_toggled(GtkButton *button, receive_d *ReceiveDialog);
static void on_btn_dialog_receive_ok_clicked(GtkButton *button, receive_d *ReceiveDialog);
static void on_btn_dialog_receive_cancel_clicked(GtkButton *button, receive_d *ReceiveDialog);
static void on_btn_dialog_receive_help_clicked(GtkButton *button, gpointer data);

void on_btn_dialog_receive_help_clicked(GtkButton *button, gpointer data)
{
	gnome_help_display("ignored", &receive_help_ref);
}

void on_btn_dialog_receive_ok_clicked(GtkButton *button, receive_d *ReceiveDialog)
{
	gchar rings_c[3];
	gint use_lock;

	use_lock = (GTK_TOGGLE_BUTTON(ReceiveDialog->check_lck)->active);
	
	if (GTK_TOGGLE_BUTTON(ReceiveDialog->rad_pick_up)->active){		/*Wait for fax*/
		/*need rings as char*/
		sprintf(rings_c, "%d", (gtk_spin_button_get_value_as_int(
							GTK_SPIN_BUTTON(ReceiveDialog->spin_answer_no))));
		on_btn_dialog_receive_cancel_clicked(NULL,ReceiveDialog);
		receive_fax(TRUE, rings_c, use_lock);
	}else{									/*Attempt to receive fax immediately*/
		on_btn_dialog_receive_cancel_clicked(NULL,ReceiveDialog);
		receive_fax(FALSE, NULL, use_lock);
	}
	return;
}

void on_btn_dialog_receive_cancel_clicked(GtkButton *button, receive_d *ReceiveDialog)
{
	/*if the user hit the cancel or ok button, remove the 
	"destroy" handler to prevent executing this twice*/
	gtk_signal_disconnect (GTK_OBJECT (ReceiveDialog->dialog_receive), ReceiveDialog->destroy_signal);
#	ifdef DEBUG
		puts("DEBUG:destroying receive dialog");
#	endif DEBUG

	gtk_widget_destroy(ReceiveDialog->dialog_receive);
	g_free(ReceiveDialog);
	return;
}
void create_dialog_receive(void)
{
	receive_d *ReceiveDialog = NULL;
	
	ReceiveDialog = g_malloc(sizeof(receive_d));
	ReceiveDialog->dialog_receive = gnome_dialog_new (_("gefax - Receive fax"), NULL);
	GTK_WINDOW (ReceiveDialog->dialog_receive)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_modal (GTK_WINDOW (ReceiveDialog->dialog_receive), TRUE);
	gtk_window_set_default_size (GTK_WINDOW (ReceiveDialog->dialog_receive), 269, -1);
	gtk_window_set_policy (GTK_WINDOW (ReceiveDialog->dialog_receive), FALSE, FALSE, FALSE);
	gtk_window_set_transient_for(GTK_WINDOW (ReceiveDialog->dialog_receive),GTK_WINDOW (gefax));

	ReceiveDialog->dialog_rec_vbox = GNOME_DIALOG (ReceiveDialog->dialog_receive)->vbox;
	gtk_widget_show (ReceiveDialog->dialog_rec_vbox);

	ReceiveDialog->vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (ReceiveDialog->vbox);
	gtk_box_pack_start (GTK_BOX (ReceiveDialog->dialog_rec_vbox), ReceiveDialog->vbox, TRUE, TRUE, 0);

	ReceiveDialog->lbl_msg = gtk_label_new (_("Accept an incoming fax Immediately or,\n wait and answer calls."));
	gtk_widget_show (ReceiveDialog->lbl_msg);
	gtk_box_pack_start (GTK_BOX (ReceiveDialog->vbox), ReceiveDialog->lbl_msg, FALSE, FALSE, 0);
	gtk_misc_set_padding (GTK_MISC (ReceiveDialog->lbl_msg), 19, 11);

	ReceiveDialog->rec_group = NULL;
  
	ReceiveDialog->rad_dont_pick_up = gtk_radio_button_new_with_label (ReceiveDialog->rec_group,
												_("Phone has already been picked up"));
	ReceiveDialog->rec_group = gtk_radio_button_group (GTK_RADIO_BUTTON (ReceiveDialog->rad_dont_pick_up));
	gtk_widget_show (ReceiveDialog->rad_dont_pick_up);
	gtk_box_pack_start (GTK_BOX (ReceiveDialog->vbox), ReceiveDialog->rad_dont_pick_up, FALSE, FALSE, 5);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ReceiveDialog->rad_dont_pick_up), TRUE);

	ReceiveDialog->rad_pick_up = gtk_radio_button_new_with_label (ReceiveDialog->rec_group,
												_("Wait and automatically pick up the phone"));
	ReceiveDialog->rec_group = gtk_radio_button_group (GTK_RADIO_BUTTON (ReceiveDialog->rad_pick_up));
	gtk_widget_show (ReceiveDialog->rad_pick_up);
	gtk_box_pack_start (GTK_BOX (ReceiveDialog->vbox), ReceiveDialog->rad_pick_up, FALSE, FALSE, 5);

	ReceiveDialog->hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (ReceiveDialog->hbox);
	gtk_box_pack_start (GTK_BOX (ReceiveDialog->vbox), ReceiveDialog->hbox, TRUE, TRUE, 5);

	ReceiveDialog->lbl_rings = gtk_label_new (_("Number of rings before answering:"));
	gtk_widget_show (ReceiveDialog->lbl_rings);
	gtk_box_pack_start (GTK_BOX (ReceiveDialog->hbox), ReceiveDialog->lbl_rings, FALSE, FALSE, 5);
	gtk_widget_set_sensitive (ReceiveDialog->lbl_rings, FALSE);

	ReceiveDialog->spin_answer_no_adj = gtk_adjustment_new (3, 1, 10, 1, 10, 10);
	ReceiveDialog->spin_answer_no = gtk_spin_button_new (GTK_ADJUSTMENT (ReceiveDialog->spin_answer_no_adj), 1, 0);
	gtk_widget_show (ReceiveDialog->spin_answer_no);
	gtk_box_pack_start (GTK_BOX (ReceiveDialog->hbox), ReceiveDialog->spin_answer_no, TRUE, TRUE, 5);
	gtk_widget_set_sensitive (ReceiveDialog->spin_answer_no, FALSE);
	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (ReceiveDialog->spin_answer_no), GTK_UPDATE_IF_VALID);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (ReceiveDialog->spin_answer_no), TRUE);

	ReceiveDialog->check_lck = gtk_check_button_new_with_label (_("Receive more than 1 fax"));
	gtk_widget_show (ReceiveDialog->check_lck);
	gtk_box_pack_start (GTK_BOX (ReceiveDialog->vbox), ReceiveDialog->check_lck, FALSE, FALSE, 5);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ReceiveDialog->check_lck), TRUE);
	gtk_widget_set_sensitive (ReceiveDialog->check_lck, FALSE);

	ReceiveDialog->action_area = GNOME_DIALOG (ReceiveDialog->dialog_receive)->action_area;
	gtk_widget_show (ReceiveDialog->action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (ReceiveDialog->action_area), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (ReceiveDialog->action_area), 8);

	gnome_dialog_append_button (GNOME_DIALOG (ReceiveDialog->dialog_receive), GNOME_STOCK_BUTTON_OK);
	ReceiveDialog->btn_ok = g_list_last (GNOME_DIALOG (ReceiveDialog->dialog_receive)->buttons)->data;
	gtk_widget_show (ReceiveDialog->btn_ok);
	GTK_WIDGET_SET_FLAGS (ReceiveDialog->btn_ok, GTK_CAN_DEFAULT);

	gnome_dialog_append_button (GNOME_DIALOG (ReceiveDialog->dialog_receive), GNOME_STOCK_BUTTON_CANCEL);
	ReceiveDialog->btn_cancel = g_list_last (GNOME_DIALOG (ReceiveDialog->dialog_receive)->buttons)->data;
	gtk_widget_show (ReceiveDialog->btn_cancel);
	GTK_WIDGET_SET_FLAGS (ReceiveDialog->btn_cancel, GTK_CAN_DEFAULT);
	
	gnome_dialog_append_button (GNOME_DIALOG (ReceiveDialog->dialog_receive), GNOME_STOCK_BUTTON_HELP);
	ReceiveDialog->btn_help = g_list_last (GNOME_DIALOG (ReceiveDialog->dialog_receive)->buttons)->data;
	gtk_widget_show (ReceiveDialog->btn_help);

	gtk_signal_connect (GTK_OBJECT (ReceiveDialog->rad_pick_up), "toggled",
					GTK_SIGNAL_FUNC (on_rad_pick_up_toggled), (receive_d *) ReceiveDialog);
	gtk_signal_connect (GTK_OBJECT (ReceiveDialog->btn_ok), "clicked",
					GTK_SIGNAL_FUNC (on_btn_dialog_receive_ok_clicked), (receive_d *) ReceiveDialog);
	gtk_signal_connect (GTK_OBJECT (ReceiveDialog->btn_cancel), "clicked",
					GTK_SIGNAL_FUNC (on_btn_dialog_receive_cancel_clicked), (receive_d *) ReceiveDialog);
	gtk_signal_connect (GTK_OBJECT (ReceiveDialog->btn_help), "clicked",
					GTK_SIGNAL_FUNC (on_btn_dialog_receive_help_clicked), NULL);
	ReceiveDialog->destroy_signal = gtk_signal_connect (GTK_OBJECT (ReceiveDialog->dialog_receive), "destroy",
					GTK_SIGNAL_FUNC (on_btn_dialog_receive_cancel_clicked), (receive_d *) ReceiveDialog);
                      
	gtk_widget_show(ReceiveDialog->dialog_receive);
	return;
}

void on_rad_pick_up_toggled(GtkButton *button, receive_d *ReceiveDialog)
{
	if (GTK_TOGGLE_BUTTON(ReceiveDialog->rad_pick_up)->active){
		gtk_widget_set_sensitive (ReceiveDialog->lbl_rings, TRUE);
		gtk_widget_set_sensitive (ReceiveDialog->check_lck, TRUE);
		gtk_widget_set_sensitive (ReceiveDialog->spin_answer_no, TRUE);
	}else{
		gtk_widget_set_sensitive (ReceiveDialog->lbl_rings, FALSE);
		gtk_widget_set_sensitive (ReceiveDialog->check_lck, FALSE);
		gtk_widget_set_sensitive (ReceiveDialog->spin_answer_no, FALSE);
	}
	return;
}


/*end*/





