/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>

#include <gnome.h>

#include "dialog_ret.h"
#include "support.h"
#include "gefax.h"

extern gchar *fax_dir;
extern gchar *bin_dir;
extern void load_dir(void);

static GnomeHelpMenuEntry trash_help_ref =
{
	"gefax",
	"fax.html#rt"
};

static GtkWidget *dialog_retrieve = NULL;
static gboolean select_all_on = FALSE;

static void on_clist_trash_select_row(GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer user_data);
static void on_clist_trash_unselect_row(GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer user_data);
static void on_clist_trash_select_all(GtkCList *clist, gint column, gpointer user_data);
static void on_btn_retrieve_ok_clicked(GtkButton *button, GtkCList *clist_ret);									
static void on_btn_retrieve_cancel_clicked(GtkButton *button,gpointer	data);
static void on_btn_retrieve_help_clicked(GtkButton *button, gpointer data);
static gint load_file_list(GtkCList *clist_trash);

struct SelectFileList{
	gint row;
	gboolean select;
	struct SelectFileList *next_ptr;
};

struct SelectFileList *fl_first_ptr = NULL;

void on_clist_trash_select_row(GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer user_data)
{
	struct SelectFileList *find_ptr = NULL;
	find_ptr = fl_first_ptr;
	while (find_ptr != NULL){
		if (find_ptr->row == row){
			find_ptr->select = TRUE;
			break;
		}
		find_ptr = find_ptr->next_ptr;
	}
}

void on_clist_trash_unselect_row(GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer user_data)
{
	struct SelectFileList *find_ptr = NULL;
	find_ptr = fl_first_ptr;
	while (find_ptr != NULL){
		if (find_ptr->row == row){
			find_ptr->select = FALSE;
			break;
		}
		find_ptr = find_ptr->next_ptr;
	}
}

void on_clist_trash_select_all(GtkCList *clist, gint column, gpointer user_data)
{

	if (!select_all_on){
		select_all_on = TRUE;
		gtk_clist_select_all(clist);
	}else{
		select_all_on = FALSE;
		gtk_clist_unselect_all(clist);
	}
}

void on_btn_retrieve_ok_clicked(GtkButton *button, GtkCList *clist_ret)
{
	gchar *old_filename;
	gchar *new_filename;
	gchar *trash_fname_ptr;
	struct SelectFileList *find_ptr = NULL;
	struct SelectFileList *next;
	
	find_ptr = fl_first_ptr;
	while (find_ptr != NULL){
		next = find_ptr->next_ptr;
		if (find_ptr->select == TRUE){
			gtk_clist_get_text( (GtkCList *)clist_ret, find_ptr->row, 0, &trash_fname_ptr);	
			old_filename = g_strdup_printf("%s%s", bin_dir, trash_fname_ptr);
			new_filename = g_strdup_printf("%s%s", fax_dir, trash_fname_ptr);
			if (rename (old_filename, new_filename) == -1){
				g_warning("Encountered an Error");
			}
			g_free(old_filename);
			g_free(new_filename);
		}
		g_free(find_ptr);
		find_ptr = next;
	}
	fl_first_ptr = NULL;
		gtk_widget_destroy(dialog_retrieve);
		dialog_retrieve = NULL;
		
		/*let the dialog close while loading files*/
		while (gtk_events_pending()) gtk_main_iteration();
		load_dir();
}

void on_btn_retrieve_cancel_clicked(GtkButton *button, gpointer data)
{
	if(dialog_retrieve) gtk_widget_destroy(dialog_retrieve);
	dialog_retrieve = NULL;	
}

void on_btn_retrieve_help_clicked(GtkButton *button, gpointer data)
{
	gnome_help_display("ignored", &trash_help_ref);
}

void create_dialog_retrieve(void)
{

	GtkWidget *dialog_vbox1;
	GtkWidget *scrolledwindow1;
	GtkWidget *clist_trash;
	GtkWidget *dialog_action_area1;
	GtkWidget *btn_retrieve_ok;
	GtkWidget *btn_retrieve_cancel;
	GtkWidget *btn_retrieve_help;
	GtkWidget *no_trash;
	
	if(dialog_retrieve){
		gdk_window_raise(dialog_retrieve->window);
		return;
	}
	
	select_all_on = FALSE;
  
	dialog_retrieve = gnome_dialog_new (_("gefax - Trash "), NULL);
	gtk_object_set_data (GTK_OBJECT (dialog_retrieve), "dialog_retrieve", dialog_retrieve);
	gtk_window_set_transient_for(GTK_WINDOW (dialog_retrieve),GTK_WINDOW (gefax));
	gtk_window_set_policy (GTK_WINDOW (dialog_retrieve), TRUE, TRUE, TRUE);

	dialog_vbox1 = GNOME_DIALOG (dialog_retrieve)->vbox;
	gtk_object_set_data (GTK_OBJECT (dialog_retrieve), "dialog_vbox1", dialog_vbox1);
	gtk_widget_show (dialog_vbox1);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), scrolledwindow1, TRUE, TRUE, 0);
	gtk_widget_set_usize (scrolledwindow1, 280, 263);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	clist_trash = gtk_clist_new (1);
	gtk_widget_show (clist_trash);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), clist_trash);
	gtk_clist_set_column_width (GTK_CLIST (clist_trash), 0, 80);
	gtk_clist_set_selection_mode (GTK_CLIST (clist_trash), GTK_SELECTION_MULTIPLE);
	gtk_clist_column_titles_show (GTK_CLIST (clist_trash));
	gtk_clist_set_column_title (GTK_CLIST (clist_trash),0,_("Select File To Retrieve"));

	dialog_action_area1 = GNOME_DIALOG (dialog_retrieve)->action_area;
	gtk_widget_show (dialog_action_area1);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

	gnome_dialog_append_button (GNOME_DIALOG (dialog_retrieve), GNOME_STOCK_BUTTON_OK);
	btn_retrieve_ok = g_list_last (GNOME_DIALOG (dialog_retrieve)->buttons)->data;
	gtk_widget_show (btn_retrieve_ok);

	gnome_dialog_append_button (GNOME_DIALOG (dialog_retrieve), GNOME_STOCK_BUTTON_CANCEL);
	btn_retrieve_cancel = g_list_last (GNOME_DIALOG (dialog_retrieve)->buttons)->data;
	gtk_widget_show (btn_retrieve_cancel);
	GTK_WIDGET_SET_FLAGS (btn_retrieve_cancel, GTK_CAN_DEFAULT);

	gnome_dialog_append_button (GNOME_DIALOG (dialog_retrieve), GNOME_STOCK_BUTTON_HELP);
	btn_retrieve_help = g_list_last (GNOME_DIALOG (dialog_retrieve)->buttons)->data;
	gtk_widget_show (btn_retrieve_help);

	gtk_signal_connect (GTK_OBJECT (dialog_retrieve), "destroy",
					GTK_SIGNAL_FUNC (on_btn_retrieve_cancel_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_retrieve_ok), "clicked",
					GTK_SIGNAL_FUNC (on_btn_retrieve_ok_clicked), (gpointer)clist_trash);
	gtk_signal_connect (GTK_OBJECT (btn_retrieve_help), "clicked",
					GTK_SIGNAL_FUNC (on_btn_retrieve_help_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_retrieve_cancel), "clicked",
					GTK_SIGNAL_FUNC (on_btn_retrieve_cancel_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_trash), "select_row",
					GTK_SIGNAL_FUNC (on_clist_trash_select_row), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_trash), "unselect_row",
					GTK_SIGNAL_FUNC (on_clist_trash_unselect_row), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_trash), "click_column",
					GTK_SIGNAL_FUNC (on_clist_trash_select_all), NULL);

	if(load_file_list((gpointer)clist_trash)){
		gtk_widget_grab_focus (btn_retrieve_cancel);  
		gtk_widget_show(dialog_retrieve);
	}else{
		on_btn_retrieve_cancel_clicked (NULL, NULL);
		no_trash = gnome_message_box_new (_("Your trash bin is empty"),
							GNOME_MESSAGE_BOX_INFO,
							GNOME_STOCK_BUTTON_CLOSE,
							NULL);
		gtk_window_set_position (GTK_WINDOW (no_trash), GTK_WIN_POS_MOUSE);
		gnome_dialog_run_and_close(GNOME_DIALOG(no_trash));
	}
	return; 
}

gint load_file_list(GtkCList *clist_trash)
{
	
	gchar *absolute_file_name;
	gchar *add_rows[1];
	struct dirent **namelist;
	gint no_of_files;
	gint count;
	struct SelectFileList *fl_new_ptr;
	
	/*Load filenames in clist*/
	no_of_files = scandir(bin_dir, &namelist, 0, alphasort);
	if (no_of_files < 3){
		return FALSE;
	}else{
		for(count = 1; count < no_of_files; ++count) {
			absolute_file_name = g_strdup_printf("%s%s", bin_dir,namelist[count]->d_name);
			if ((g_file_test(absolute_file_name, G_FILE_TEST_ISFILE)) && (namelist[count]->d_name[0] != '.')){
				add_rows[0] = namelist[count]->d_name;
				gtk_clist_append( clist_trash, add_rows);

				fl_new_ptr = g_malloc(sizeof (struct SelectFileList));
				fl_new_ptr->row = count - 2;
				fl_new_ptr->select = FALSE;
				fl_new_ptr->next_ptr = fl_first_ptr;
				fl_first_ptr = fl_new_ptr;
				g_free(absolute_file_name);
			}
		}
	}
	
  return TRUE;
}


/*end*/





