/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>  /* include this before any other sys headers */
#include <sys/wait.h>   /* header for waitpid() and various macros */
#include <signal.h>     /* header for signal functions */
#include <stdio.h>  
#include <unistd.h>     /* header for fork() */
#include <dirent.h>
#include <sys/stat.h>
#include <string.h>

#include <gnome.h>
#include <gefax/gefax.h>

#include "gefax.h"
#include "browser.h"
#include "control.h"
#include "exec_cmd.h"
#include "viewer.h"
#include "appbar.h"
#include "sig_err.h"
#include "menu.h"
#include "toolbars.h"
#include "print.h"
#include "term.h"


/*constants*/
static const gchar EFAX_PATH []= "/usr/share/gefax/efax/efax";
static const gchar EFIX_PATH []= "/usr/share/gefax/efax/efix";
static const gchar EFAX []= "efax";

						
/*structures*/
struct EfixGsExec{					/* args for creating fax pages - I don't like this at all -FIX*/
	const gchar	*command;				/*command to execute*/
	gchar	*argv[10];			/*arg1*/
	gchar	*in_fname;
	gchar	*out_fname;
	gchar	*file;
	gboolean temp;
	struct	EfixGsExec *next_ptr;		/*pointer to the next element*/
};
static struct EfixGsExec *exec_command_ptr = NULL;
static struct EfixGsExec *exec_fst_ptr = NULL;

struct PageFile{					/*list of page files for sending*/
	gchar	*fname;				/*Phone number*/
	struct	PageFile *next_ptr;		/*pointer to the next element*/
};
static struct PageFile *page_fst_ptr = NULL;
static struct PageFile *page_command_ptr = NULL;


static struct PhoneNumber *current_phone_ptr = NULL;

/*other*/
static struct sigaction act;		/*I dont understand this very well yet*/
static gint cont_ex;			/*what to do with a received program exit code*/
static gint start_viewer;		/*if we were creating pages for a preview*/
static gchar *argvec[200];		/*arg vector for efax sending faxes*/
static gint argvec_phone;		/*the argvec[X] where the phone num is to be inserted*/
static gint view_timer;
static gint wait_m_timer;
static gint exec_timer;			/*starts sending faxes after tiff files are made -This is UGLY*/
static pid_t answer_pid = NONE; 	/*pid of efax when recieving or waiting for a fax*/
static pid_t poll_pid = NONE; 		/*pid of efax when polling a fax*/
static pid_t send_pid = NONE;		/*pid of efax when sending*/
static pid_t make_pid = NONE;		/*pid of gs or efix when making pages*/
static int answer_pipe[2];
static int poll_pipe[2];
static int send_pipe[2];

static gchar *receive_dir;		/*newly created directory for recieving a fax*/


/*
 *Debug functions
*/
#	ifdef DEBUG
		void	print_nums (void);
#	endif /* DEBUG */

static void gefax_answer_control_get_data(gpointer data, gint source, GdkInputCondition condition );
static void gefax_poll_control_get_data(gpointer data, gint source, GdkInputCondition condition );
static void gefax_send_control_get_data(gpointer data, gint source, GdkInputCondition condition );
/*
 *Cleanup struct exec after tiff files have been made
*/
static void free_EfixGsExec(void);

/*
 *The UGLY timers - need to jump out of the signal handler
*/
static gint start_viewer_t(gpointer data);
static gint start_efax(gpointer data);
static gint start_wait_m(gpointer data);
/*
 *Cleanup struct page
*/
static void free_PageFile(void);


/*
 *Forks and runs command in struct exec
*/
static void make_tiffg3(void);

/*
 *Forks and sends commands to efax
*/
static void exec_fax_cmd(void);

/*
 * prototype for our SIGCHLD handler
*/
static void sig_chld(int);

/*
 *Gives the user some information about which files are being converted to fax pages
 *Builds commands to fax Tiffg3 files to the filename of dir + prefix etc
*/
static gint prepare_to_send(void);

/*
 *restore menus etc
*/
static void restore_interface(gint action);
				

/*________________________________________Create Tiffg3 Files___________________________________*/


/*
 *save all commands to exec gs or efix in struct exec
*/
void send_fax(gint action)
{


	struct FileList *current_ptr;
	struct EfixGsExec *exec_new_ptr;
	gint no_commands;				/*number of different phone numbers*/
	gint argc;
	gchar *temp_filename;
	LibGefaxWrapTextResult *result = NULL;
	exec_fst_ptr = NULL;
	no_commands = 0;
	current_ptr = fst_ptr;

	delete_temp_files(); /*old pages are left there till next time*/
	
	current_ptr = fst_ptr;
	while (current_ptr != NULL){
		
		if (strcmp(current_ptr->select, UNSELECT) != 0){	/*has a number*/
			if (strcmp(current_ptr->program, "ghostscript") == 0){
				
#				ifdef DEBUG
					printf("DEBUG:gefaxmime.conf program=ghostscript\n");
#				endif /* DEBUG*/
				
				argc = 0;
				exec_new_ptr = g_malloc(sizeof(struct EfixGsExec));
				exec_new_ptr->command = "gs"; ++argc;
				exec_new_ptr->argv[argc] = "-dQUIET"; ++argc;
				exec_new_ptr->argv[argc] = "-sDEVICE=tiffg3"; ++argc;
				exec_new_ptr->argv[argc] = "-dNOPAUSE"; ++argc;
				exec_new_ptr->argv[argc] = GsArgs.paper; ++argc;
				exec_new_ptr->out_fname = g_strdup_printf("-sOutputFile=%s%s.%%03d",temp_dir,current_ptr->select);
				exec_new_ptr->argv[argc] = exec_new_ptr->out_fname; ++argc;
				exec_new_ptr->argv[argc] = "-- filename"; ++argc;
				exec_new_ptr->in_fname = g_strdup_printf("%s%s", fax_dir, current_ptr->file);
				exec_new_ptr->argv[argc] = exec_new_ptr->in_fname; ++argc;
				exec_new_ptr->argv[argc] = NULL;
				exec_new_ptr->file = current_ptr->file;
				exec_new_ptr->temp = FALSE;
				exec_new_ptr->next_ptr = exec_fst_ptr;
				exec_fst_ptr = exec_new_ptr;
				++no_commands;
			}else{
				exec_new_ptr = g_malloc(sizeof(struct EfixGsExec));
				exec_new_ptr->command = EFIX_PATH;
				argc = 0;
				
#				ifdef DEBUG
					printf("DEBUG:gefaxmime.conf program=text\n");
					exec_new_ptr->argv[argc] = "-vewiaf"; ++argc;
#				endif /* DEBUG*/
				
				exec_new_ptr->argv[argc] = "-l66"; ++argc;
				exec_new_ptr->argv[argc] = "-d0.2,0.2"; ++argc;
				exec_new_ptr->argv[argc] = "-f"GEFAX_FILE_FONT; ++argc;				
				exec_new_ptr->out_fname = g_strdup_printf("-n%s%s.%%03d",temp_dir,current_ptr->select);
				exec_new_ptr->argv[argc] = exec_new_ptr->out_fname; ++argc;
				
				temp_filename = g_strdup_printf("%s%s", fax_dir, current_ptr->file);
				if (strcmp(current_ptr->program, "text") == 0){
					exec_new_ptr->argv[argc] = "-itext"; ++argc;	/*efix ver 0.3 seems to ignore this*/
					if (TextWrapArgs.do_wrap){
						result = libGefax_wrap_text_file(temp_filename, TextWrapArgs.tab_len, TextWrapArgs.line_len);
						if (result->error == LIBGEFAX_WRAPTEXTFILE_ERROR_MIMETYPE) {
							gnome_error_dialog(_("Gefax was unable to determine the file mime type\n"
										"The file may not be a valid text file"));
						} else if (result->error == LIBGEFAX_WRAPTEXTFILE_ERROR_LINELENGTH) {
							gnome_error_dialog(_("Gefax can not wrap the text file\n Invalid line length"));
						} else if (result->error == LIBGEFAX_WRAPTEXTFILE_ERROR_NOFILE) {
							gnome_error_dialog(_("Gefax can not find one of the text files"));
						} else if (result->error == LIBGEFAX_WRAPTEXTFILE_ERROR_NOFILEACCESS) {
							gnome_error_dialog(_("You do not have the correct file permission to access the file"));
						} else if (result->error == LIBGEFAX_WRAPTEXTFILE_ERROR_OUTFILE) {
							gnome_error_dialog(_("Gefax was unable to generate a valid temporary file\n"
										"Please try again"));
						} else {
							printf("FILE+%s\n",result->return_file);
							printf("Line Count = %d \n wrapped= %d \n words= %d \n", result->file_linecount,result->lines_wrapped,result->words);
							exec_new_ptr->temp = TRUE;
							exec_new_ptr->in_fname = result->return_file;
							g_free (result);
						}
					
						g_free (temp_filename);
					}else{
						exec_new_ptr->in_fname = temp_filename;
						exec_new_ptr->temp = FALSE;
					}
				}else{
					exec_new_ptr->in_fname = temp_filename;
					exec_new_ptr->temp = FALSE;
				}
				exec_new_ptr->argv[argc] = exec_new_ptr->in_fname; ++argc;
				
				exec_new_ptr->argv[argc] = NULL;
				exec_new_ptr->file = current_ptr->file;
				exec_new_ptr->next_ptr = exec_fst_ptr;
				exec_fst_ptr = exec_new_ptr;				
				++no_commands;			

			}
		}
		current_ptr = current_ptr->next_ptr;
	}
	
	/*continue on and make fax pages*/
	if (no_commands > 0){
		start_progress_motion(30,FALSE);
		if (!action){
			adjust_toolbar(SEND_A);
			adjust_menu(SEND_A);
		}
		set_action_cursor();
		exec_command_ptr = exec_fst_ptr;			
		make_tiffg3();
		start_viewer = action;
	}else{
		change_to_browser(NULL,NULL);
	}
	return;
}
void make_tiffg3(void)
{
	
	gchar *status_msg;
	
	if (strcmp(exec_command_ptr->command, "gs") == 0){
		cont_ex = GS_C;
	}else{
		cont_ex = EFIX_C;
	}
	status_msg = g_strdup_printf("Processing File: %s",exec_command_ptr->file);
	static_status_msg(status_msg);
	g_free(status_msg);
	switch (make_pid = fork()){		/* Fork */
		case -1:				/*ERROR Fork Failed*/
			fprintf(stderr, "fork failed\n");
			return;
		case 0:
			execvp( exec_command_ptr->command, exec_command_ptr->argv);
		 	_exit(7);           /*  exit status = 7 - The Execution Failed*/
		default:        		/*we are the parent*/
	}     
	return;
}

/*________________________________________send fax___________________________________*/



gint prepare_to_send(void)
{

	int pcount;
	int files;
	int cont;		/*Return TRUE if ok*/
	gchar *fname;
	struct dirent **namelist;
	struct PageFile *new_page_ptr;
	struct PageFile *current_page_ptr;
	gint arg_no;
	
	while(1){
		files = scandir(temp_dir, &namelist, 0, alphasort); /*count num of files*/
		if (files > 180){	/*can't think of any reason for more?*/
			cont = FALSE;
			static_status_msg(_("TRANSMISSION ABORTED: Fax contained more than 180 pages!"));
			break;
		}
	
		for (pcount = files - 1; pcount > 1; --pcount){ /*start at the end so pages are sent in order*/
			fname = g_strdup_printf("%s%s", temp_dir, namelist[pcount]->d_name);
			new_page_ptr = g_malloc(sizeof(struct PageFile));
			new_page_ptr->fname = g_strdup(fname);
			new_page_ptr->next_ptr = page_fst_ptr;
			page_fst_ptr = new_page_ptr;
			g_free(fname);		
		}
		if (page_fst_ptr == NULL){	/*no pages*/
			cont = FALSE;
			break;
		}
		current_page_ptr = NULL;
		arg_no = 0;
		argvec[arg_no] = "efax"; ++arg_no;
		argvec[arg_no] = EfaxArgs.dev; ++arg_no;
		if (strcmp(EfaxArgs.locktype, "none") != 0){
			argvec[arg_no] = EfaxArgs.lock; ++arg_no;
		}
		argvec[arg_no] = "-f"GEFAX_FILE_FONT; ++arg_no;
		argvec[arg_no] = EfaxArgs.header; ++arg_no;
		argvec[arg_no] = EfaxArgs.log; ++arg_no;
		argvec[arg_no] = EfaxArgs.mclass; ++arg_no;
		if (strcmp(EfaxArgs.xonxoff, "none") != 0){
			argvec[arg_no] = EfaxArgs.xonxoff; ++arg_no;
		}
		if (strcmp(EfaxArgs.cmddelay, "default") != 0){
			argvec[arg_no] = EfaxArgs.cmddelay; ++arg_no;
		}
		argvec[arg_no] = EfaxArgs.init; ++arg_no;
		argvec[arg_no] = EfaxArgs.vol; ++arg_no;
		argvec[arg_no] = EfaxArgs.caps; ++arg_no;
		argvec[arg_no] = EfaxArgs.ident; ++arg_no;
		argvec[arg_no] = EfaxArgs.reset; ++arg_no;
		argvec_phone = arg_no; ++arg_no; /*we add the phone no before executing
									just have to know where*/
		if (EfaxArgs.usecover == 1){
			argvec[arg_no] = EfaxArgs.cover; ++arg_no;
		}
		/*add pages*/
		current_page_ptr = page_fst_ptr;
		while (current_page_ptr != NULL){				/*fell off the end*/
			argvec[arg_no] = current_page_ptr->fname;
			current_page_ptr = current_page_ptr->next_ptr;
			++arg_no;		
		}
		argvec[arg_no] = NULL;
		current_phone_ptr = phone_fst_ptr;
		cont = TRUE;
		break;
	}
	return(cont);
}

gint start_efax(gpointer data)
{
	free_EfixGsExec();
	if(prepare_to_send()){			/*setup args for efax*/
		if (debug_window) create_gefax_terminal();
		exec_fax_cmd();			/*send if there were not too many files*/
	}else{
		static_status_msg(_("Failed No Pages Created"));
		restore_interface(SENDING);
	}
	return (FALSE);
}
gint start_wait_m(gpointer data)
{
	receive_fax(TRUE,NULL,TRUE);
	
	return (FALSE);
}

gint start_viewer_t(gpointer data)
{
	
	viewer_load_directory(temp_dir);
	free_EfixGsExec();
	
	return (FALSE);
}

void exec_fax_cmd(void)
{
	gchar *tmp;
	gchar *fnum = NULL;

#	ifdef DEBUG
		int arg_no;
		printf("DEBUG:Executing Efax for send fax\n");
#	endif /* DEBUG */

	cont_ex = SENDING;
	
	/*add the number with switch to argvec*/
	fnum = g_strdup_printf("%s%s", EfaxArgs.prefix, current_phone_ptr->phone);
	argvec[argvec_phone] = fnum;
	
#	ifdef DEBUG
		/* print out arguments*/
		for (arg_no = 0; ((arg_no < 200) && (argvec[arg_no] != NULL)); ++arg_no){
			printf("	DEBUG:argvec[%d]=%s\n", arg_no, argvec[arg_no]);
		}
#	endif /* DEBUG */
	
	/*give user some info*/
	tmp = g_strdup_printf("Sending Fax To: %s - %s", current_phone_ptr->person, current_phone_ptr->phone);
	static_status_msg(tmp);
	g_free(tmp);

	/*execute*/
	switch (send_pid = fork()){		/* Fork */
		case -1:				/*ERROR Fork Failed*/
			fprintf(stderr, "fork failed\n");
			return;
		case 0:
			dup2( send_pipe[1], STDERR_FILENO );
			/*We are the child	Run the command*/
			execv( EFAX_PATH, argvec);
		 	_exit(7);           /*  exit status = 7 - The Execution Failed if we get to this*/
		default:        		/*we are the parent*/			
			return;
	}    
	if(fnum) g_free (fnum);
	fnum = NULL;
	return;
}

/*________________________________________receive & poll fax___________________________________*/

void receive_fax(gint pick_up, gchar *rings,	int lck)
{
	extern gchar *get_receive_directory(void);
	gchar *r_file_names = NULL;
	gchar *rec_args[20];
	gint arg_no;
	static gchar ring_str[8];
	
#	ifdef DEBUG
		printf("DEBUG:Receiving Fax\n");
#	endif /*DEBUG*/

	adjust_toolbar(RECEIVE_A);
	adjust_menu(RECEIVE_A);

	arg_no = 0;
	if ((pick_up) && (lck)){
		cont_ex = WAIT_M;
	}else if (pick_up){
		cont_ex = WAIT;
	}else{
		cont_ex = RECEIVING;
	}
	
	start_progress_motion(130, TRUE);

	if (rings) sprintf(ring_str,"-iS0=%s", rings);
#	ifdef DEBUG
		printf("	DEBUG:Rings before pickup=%s\n", ring_str);
		printf("	DEBUG:Wait for more than 1 fax=%d\n", lck);
#	endif /*DEBUG*/
		
	receive_dir = get_receive_directory();
	printf("%s\n",receive_dir);
	r_file_names = g_strdup_printf("-r%s/%%b%%d-%%H.%%I.%%S.tiff", receive_dir);
#	ifdef DEBUG
		printf("	DEBUG:Receive filename arg=%s\n", r_file_names);
#	endif /* DEBUG*/
	rec_args[arg_no] = "efax"; ++arg_no;
	rec_args[arg_no] = EfaxArgs.dev; ++arg_no;
	rec_args[arg_no] = EfaxArgs.caps; ++arg_no;
	rec_args[arg_no] = EfaxArgs.lock; ++arg_no;
	rec_args[arg_no] = EfaxArgs.log; ++arg_no;
	rec_args[arg_no] = EfaxArgs.mclass; ++arg_no;
	rec_args[arg_no] = EfaxArgs.reset; ++arg_no;
	rec_args[arg_no] = EfaxArgs.vol; ++arg_no;
	if (strcmp(EfaxArgs.xonxoff, "none") != 0){
		rec_args[arg_no] = EfaxArgs.xonxoff; ++arg_no;
	}
	if (strcmp(EfaxArgs.cmddelay, "default") != 0){
		rec_args[arg_no] = EfaxArgs.cmddelay; ++arg_no;
	}
	if (pick_up){		/*Wait for call and answer*/
		static_status_msg(_("Waiting for incoming Fax"));
		rec_args[arg_no] = "-w"; ++arg_no;
		rec_args[arg_no] = ring_str; ++arg_no;
	}else{			/*Already answered just receive*/
		static_status_msg(_("Attempting to receive incoming Fax"));
	}
	rec_args[arg_no] = EfaxArgs.ident; ++arg_no;
	rec_args[arg_no] = r_file_names; ++arg_no;
	rec_args[arg_no] = NULL;
	if (debug_window) create_gefax_terminal();
	/*execute efax*/
	switch (answer_pid = fork()){		/* Fork */
		case -1:				/*ERROR Fork Failed*/
			fprintf(stderr, "fork failed\n");
			return;
		case 0:
			dup2( answer_pipe[1], STDERR_FILENO );
			/*We are the child	Run the command*/
			execv( EFAX_PATH, rec_args);
		 	_exit(7);           /*  exit status = 7 - The Execution Failed*/
		default:        		/*we are the parent*/
	}

	g_free(r_file_names);
	r_file_names = NULL;
	return;

}
/*Not USED
gchar *parse_control_data(gchar *line, gint bytes)
{
	gint index = 18;
	gint str_len = 0;
	gchar *str;
	
	str = (gchar *) g_malloc0 (bytes - 18);
	while ((line[index] != '\n') && (line[index] != '\0')){
		if((line[index] != ' ') || (index != 18)){
			str[str_len] = line[index];
			++str_len;
		}
		++index;
	}
	if (strlen(str) > 1){
		return(str);
	}
	g_free(str);
	return NULL;
}*/

void gefax_answer_control_get_data(gpointer data, gint source, GdkInputCondition condition )
{
	char line[1024];
	gint bytes;
	/*gchar *str;*/

	bytes = read( source, line, 1024);
    	if (!debug_window) return;	
	
	line[bytes-1] = '\0';
	gefax_terminal_add_text(line);
	/*
	if (bytes > 18){
		str = parse_control_data(line, bytes);
		if (str){
			static_status_msg(str);
			printf("START%sEND\n",str);
			g_free(str);
		}
	}*/
}

void gefax_send_control_get_data(gpointer data, gint source, GdkInputCondition condition )
{
	char line[1024];
	gint bytes;
	/*gchar *str;*/

	bytes = read( source, line, 1024);
	if (!debug_window) return;	
	
	line[bytes-1] = '\0';
	gefax_terminal_add_text(line);
	/*
	if (bytes > 18){
		str = parse_control_data(line, bytes);
		if (str){
			static_status_msg(str);
			printf("START%sEND\n",str);
			g_free(str);
		}
	}*/
}

void gefax_poll_control_get_data(gpointer data, gint source, GdkInputCondition condition )
{
	char line[1024];
	gint bytes;
	/*gchar *str;*/

	bytes = read( source, line, 1024);
	if (!debug_window) return;	
	
	line[bytes-1] = '\0';
	gefax_terminal_add_text(line);
	/*
	if (bytes > 18){
		str = parse_control_data(line, bytes);
		if (str){
			static_status_msg(str);
			printf("START%sEND\n",str);
			g_free(str);
		}
	}*/
}

void poll_fax(gchar *fax_no)
{
	extern gchar *get_receive_directory(void);
	gchar *stat_msg;
	gchar *fnum = NULL;
	gchar *r_file_names = NULL;
	gchar *poll_args[21];
	gint arg_no;

#	ifdef DEBUG
		printf("DEBUG:Polling Fax\n");
#	endif /*DEBUG*/

	
	adjust_toolbar(POLL_A);
	adjust_menu(POLL_A);
	
	cont_ex = POLLING;
	
	arg_no = 0;
	
	/*give user some info*/
	stat_msg = g_strdup_printf("Polling Fax Number: %s",fax_no);
	static_status_msg(stat_msg);
	g_free(stat_msg);
	start_progress_motion(130,TRUE);
	
	/*add dial switch to number*/
	fnum = g_strdup_printf("%s%s", EfaxArgs.prefix, fax_no);
	
	/*make arg for received file names*/
	receive_dir = get_receive_directory();
#	ifdef DEBUG
		printf("	DEBUG: receive dir=%s\n",receive_dir);
#	endif /*DEBUG*/
	r_file_names = g_strdup_printf("-r%s/%%b%%d-%%H.%%I.%%S.tiff", receive_dir);
	
	/*point poll_args to switches*/
	poll_args[arg_no] = "efax"; ++arg_no;
	poll_args[arg_no] = EfaxArgs.dev; ++arg_no;
	if (strcmp(EfaxArgs.locktype, "none") != 0){
		poll_args[arg_no] = EfaxArgs.lock; ++arg_no;
	}
	poll_args[arg_no] = EfaxArgs.mclass; ++arg_no;
	if (strcmp(EfaxArgs.xonxoff, "none") != 0){
		poll_args[arg_no] = EfaxArgs.xonxoff; ++arg_no;
	}
	if (strcmp(EfaxArgs.cmddelay, "default") != 0){
		poll_args[arg_no] = EfaxArgs.cmddelay; ++arg_no;
	}
	poll_args[arg_no] = EfaxArgs.log; ++arg_no;
	poll_args[arg_no] = EfaxArgs.init; ++arg_no;
	poll_args[arg_no] = EfaxArgs.vol; ++arg_no;
	poll_args[arg_no] = EfaxArgs.reset; ++arg_no;
	poll_args[arg_no] = EfaxArgs.caps; ++arg_no;
	poll_args[arg_no] = EfaxArgs.ident; ++arg_no;
	poll_args[arg_no] = r_file_names; ++arg_no;
	poll_args[arg_no] = fnum; ++arg_no;
	poll_args[arg_no] = NULL;	/*be sure of it*/
	if (debug_window) create_gefax_terminal();
	/*execute*/
	switch (poll_pid = fork()){		/* Fork */
		case -1:				/*ERROR Fork Failed*/
			fprintf(stderr, "fork failed\n");
			return;
		case 0:
			dup2( poll_pipe[1], STDERR_FILENO );
			/*We are the child	Run the command*/
			execv( EFAX_PATH, poll_args);
		 	_exit(7);           /*  exit status = 7 - The Execution Failed if we get to this*/
		default:        		/*we are the parent*/
	}
	
	/*free mem */
	g_free(fnum);
	fnum = NULL;
	g_free(r_file_names);
	r_file_names = NULL;
	g_free(fax_no);
	return;
}

/*___________________________________signal handling for all exec's____________________________*/

/*
 *I really don't understand this too well yet
*/
gint setup_sig_handler(void)
{
	pipe (poll_pipe);
	pipe (send_pipe);
	pipe (answer_pipe);
	
	gdk_input_add( poll_pipe[0], GDK_INPUT_READ, gefax_poll_control_get_data , NULL);
	gdk_input_add( send_pipe[0], GDK_INPUT_READ, gefax_send_control_get_data , NULL);
	gdk_input_add( answer_pipe[0], GDK_INPUT_READ, gefax_answer_control_get_data , NULL);
	
		/* Assign sig_chld as our SIGCHLD handler */
	act.sa_handler = sig_chld;
     
	/* We don't want to block any other signals */
	sigemptyset(&act.sa_mask);
     
	/*
	* We're only interested in children that have terminated, not ones
	* which have been stopped (eg user pressing control-Z at terminal ?? maybe not! FIX)
	*/
	act.sa_flags = SA_NOCLDSTOP;
        
	/*
	* Make these values effective. If we were writing a real
	* application, we would probably save the old value instead of
	* passing NULL.	?? how what why-does this allow us to remove the sig handler?
	*/

	if (sigaction(SIGCHLD, &act, NULL) < 0){
		fprintf(stderr, "sigaction failed\n");
		return (0);
	}else{
		return (1);
	}

}

void sig_chld(int signo)
{
	extern void save_fax_data_to_directory(gchar *dir, gint autoprint, gint polled);
	gint status;
	gint child_val;
	int pid;
	
#	ifdef DEBUG
		printf("DEBUG:Received Exit Signal\n");
#	endif /*DEBUG*/
	

	
	/* Wait for any child without blocking */
	pid = waitpid(WAIT_ANY, &status, WNOHANG);
	
	/*don't do anything if it was a system() - used for printing*/
	if ((pid == make_pid) || (pid == send_pid) || (pid == answer_pid) || (pid == poll_pid)){
		
		/*make sure we don't try to kill the finished process*/
		poll_pid = 0;
		send_pid = 0;
		make_pid = 0;
		
		/* We now have the info in 'status' and can manipulate it using
		the macros in wait.h.*/
		if (WIFEXITED(status)) {               		/* did child exit normally? */
			child_val = WEXITSTATUS(status);		/* get child's exit status */
#			ifdef DEBUG
				printf("	DEBUG:Recieved exit sig child_val=%d\n",child_val);
				printf("	DEBUG:cont_ex=%d\n",cont_ex);
#			endif /* DEBUG */
			switch (cont_ex){
				case EFIX_C:
					cont_ex = NONE;	/*don't act on any more sigs till asked*/
					if (child_val == 0){		/*Normal exit*/
						if (exec_command_ptr->temp) {
							remove(exec_command_ptr->in_fname);
						}
						exec_command_ptr = exec_command_ptr->next_ptr;
						if (exec_command_ptr != NULL){	/*Have to make more pages*/
							make_tiffg3(); 
						}else if (start_viewer){			/*have to start viewer*/
							view_timer = gtk_timeout_add (500, start_viewer_t ,NULL);
						}else{						/*send the faxes*/
							exec_timer = gtk_timeout_add (500, start_efax ,NULL);
						}
					}else{					/*Error*/
						efix_error(child_val);
						restore_interface(EFIX_C);
					}
					break;
				case GS_C:
					cont_ex = NONE;	/*don't act on any more sigs till asked*/
					if (child_val == 0){		/*Normal exit*/
						exec_command_ptr = exec_command_ptr->next_ptr;
						if (exec_command_ptr != NULL){	/*Have to make more pages*/
							make_tiffg3(); 
						}else if (start_viewer){			/*have to start viewer*/
							view_timer = gtk_timeout_add (500, start_viewer_t ,NULL);
						}else{						/*send the faxes*/
							exec_timer = gtk_timeout_add (500, start_efax ,NULL);
						}
					}else{				/*Error*/
						gs_error(child_val);
						restore_interface(GS_C);
					}
					break;
				case SENDING:
					cont_ex = NONE;	/*don't act on any more sigs till asked*/
					if (child_val == 0){		/*Normal exit*/
						current_phone_ptr = current_phone_ptr->next_ptr;
						if (current_phone_ptr != NULL){/*Have to send more faxes*/
							exec_fax_cmd();
						}else{
							static_status_msg(_("Success! Completed Fax Transmission"));
							restore_interface(SENDING);
						}			
					}else if(child_val == 5){	/*Killed*/
						static_status_msg(_("Send Fax Cancelled"));
						restore_interface(SENDING);
					}else{					/*Error*/
						efax_error(child_val);
						restore_interface(SENDING);
					}
					break;
				case POLLING:
					cont_ex = NONE;	/*don't act on any more sigs till asked*/
					if (child_val == 0){		/*Normal exit*/
						gchar *fixed_dir = g_strdup_printf("%s/",receive_dir);
						/*save history file*/
						save_fax_data_to_directory(fixed_dir, Print.auto_print, TRUE);
						g_free(fixed_dir);
						g_free (receive_dir);
						receive_dir = NULL;
						static_status_msg(_("Success! Completed Polling Fax"));
					}else if(child_val == 5){	/*Killed*/
						static_status_msg(_("Poll Fax Cancelled"));
					}else{					/*Error*/
						efax_error(child_val);
					}
					restore_interface(POLLING);
					break;
				case RECEIVING:
					cont_ex = NONE;	/*don't act on any more sigs till asked*/
					if (child_val == 0){		/*Normal exit*/
						gchar *fixed_dir = g_strdup_printf("%s/",receive_dir);
						/*save history file*/
						save_fax_data_to_directory(fixed_dir, Print.auto_print, FALSE);
						g_free(fixed_dir);
						g_free (receive_dir);
						receive_dir = NULL;
						static_status_msg(_("Success! Received fax"));
					}else if(child_val == 5){	/*Killed*/
						static_status_msg(_("Recieve Fax Cancelled"));
					}else{					/*Error*/
						efax_error(child_val);
					}
					restore_interface(RECEIVING);
					answer_pid = 0;
					break;
				case WAIT_M:
					cont_ex = NONE;	/*don't act on any more sigs till asked*/
					if (child_val == 0){		/*Normal exit*/
						gchar *fixed_dir = g_strdup_printf("%s/",receive_dir);
						/*save history file*/
						save_fax_data_to_directory(fixed_dir, Print.auto_print, FALSE);
						g_free(fixed_dir);
						g_free (receive_dir);
						receive_dir = NULL;
						reset_progress();
						static_status_msg(_("Received at least 1 fax"));
						restore_interface(WAIT_M);
						if(pid == answer_pid)wait_m_timer = gtk_timeout_add (500, start_wait_m ,NULL);
					}else if(child_val == 5){	/*Killed*/
						static_status_msg(_("Recieve Fax Cancelled"));
						restore_interface(WAIT_M);
						answer_pid = 0;
					}else{					/*Error*/
						efax_error(child_val);
						restore_interface(WAIT_M);
						answer_pid = 0;
					}
					break;
				case WAIT:
					cont_ex = NONE;	/*don't act on any more sigs till asked*/
					if (child_val == 0){		/*Normal exit*/
						
						gchar *fixed_dir = g_strdup_printf("%s/",receive_dir);
						/*save history file*/
						save_fax_data_to_directory(fixed_dir, Print.auto_print, FALSE);
						g_free(fixed_dir);
						g_free (receive_dir);
						receive_dir = NULL;
						static_status_msg(_("Received a fax"));
					}else if(child_val == 5){	/*Killed*/
						static_status_msg(_("Recieve Fax Cancelled"));
					}else{					/*Error*/
						efax_error(child_val);
					}
					restore_interface(WAIT);
					answer_pid = 0;
					break;
				case EXIT_NOW:
					gtk_main_quit();
					break;
				default:
					break;
			}
		}
	}else if(pid < 0){
	/*	perror("waitpid");*/
	}
	return;
}



/*___________________________________kill exec functions____________________________*/

gint efax_status(void)
{
	return cont_ex;
}
void kill_fax_operation(GtkButton *button, gpointer data)
{
	
	extern void remove_receive_directory(gchar *dir);
	const gchar *reset_msg = N_("Please Wait: Resetting Modem");
	
	/*set cont_ex to EXIT_NOW if we are exiting*/
	if ((gint) data == EXIT_NOW) cont_ex = EXIT_NOW;
#	ifdef DEBUG
		printf("DEBUG:Killing child processes\n");
#	endif /*DEBUG*/

	if (send_pid > 1){	/*efax is running*/
#	ifdef DEBUG
			printf("	DEBUG:KILLING send_pid:%d\n",send_pid);
#	endif /*DEBUG*/
		kill(send_pid,1);
		static_status_msg(_(reset_msg));
		disable_kill_button();
		disable_kill_menu();
	}
	if (make_pid > 0){	/*efix or gs is running*/
#	ifdef DEBUG
			printf("	DEBUG:KILLING make_pid:%d\n",make_pid);
#	endif /*DEBUG*/
		kill(make_pid,1);
		static_status_msg(_("Send Fax Terminated"));
		disable_kill_button();
		disable_kill_menu();
		restore_interface(SENDING);
		cont_ex = NONE;
	}
	if (answer_pid > 1){	/*we are executing efax*/
#	ifdef DEBUG
			printf("	DEBUG:KILLING pid:%d\n",answer_pid);
#	endif /* DEBUG */
		kill(answer_pid,1);
		static_status_msg(_(reset_msg));
		disable_kill_button();
		disable_kill_menu();
	}
		if (poll_pid > 1){	/*we are executing efax*/
#	ifdef DEBUG
			printf("	DEBUG:KILLING pid:%d\n",poll_pid);
#	endif /* DEBUG */
		kill(poll_pid,1);
		disable_kill_button();
		disable_kill_menu();
		static_status_msg(_(reset_msg));
	}
	delete_temp_files();
	if(receive_dir){
		remove_receive_directory(receive_dir);
		g_free (receive_dir);
		receive_dir = NULL;
	}
	return;
}

/*________________________________________Functions for cleaning up______________________________*/

void restore_interface(gint action)
{
	extern void remove_receive_directory(gchar *dir);
	if(ActiveComponent == BROWSER_ON){
		restore_toolbar();
		remove_kill_menu();
	}
	reset_progress();
	if(receive_dir){
		remove_receive_directory(receive_dir);
		g_free (receive_dir);
		receive_dir = NULL;
	}
	switch (action){
		case SENDING:
			free_PageFile();
			reset_cursor();
			break;
		case EFIX_C:
			free_EfixGsExec();
			reset_cursor();
			break;
		case GS_C:
			free_EfixGsExec();
			reset_cursor();
			break;
		default:
			break;
	}
}
/*
 *Cleanup mem after execlp
*/
void free_EfixGsExec(void)
{
	struct EfixGsExec *next_exec;
	while (exec_fst_ptr != NULL){
		 next_exec = exec_fst_ptr->next_ptr;
		if (exec_fst_ptr->in_fname != NULL) g_free(exec_fst_ptr->in_fname);
		exec_fst_ptr->in_fname = NULL;
		if (exec_fst_ptr->out_fname != NULL) g_free(exec_fst_ptr->out_fname);
		exec_fst_ptr->out_fname = NULL;
		g_free(exec_fst_ptr);
		exec_fst_ptr = next_exec;
	}
	exec_fst_ptr = NULL;
	exec_command_ptr = NULL;

}


/*
void
del_phone							(void)
{
	struct phone *next;
	
	while (phone_fst_ptr != NULL){
		next = phone_fst_ptr->next_ptr;
		g_free(phone_fst_ptr->phone);
		g_free(phone_fst_ptr);
		phone_fst_ptr = next;
	}
}*/

void free_PageFile(void)
{
	struct PageFile *next;
	
	while (page_fst_ptr != NULL){
		next = page_fst_ptr->next_ptr;
		g_free(page_fst_ptr->fname);
		g_free(page_fst_ptr);
		page_fst_ptr = next;
	}
	page_command_ptr = NULL;
}

void delete_temp_files(void) 
{
	int ok;
	int no_files;
	int fcount;
	gchar *fname;
	struct dirent **filelist;

#	ifdef DEBUG
		printf("DEBUG:Deleting temp files\n");
#	endif /*DEBUG*/
	
	/*delete any temp files*/
	no_files = scandir(temp_dir, &filelist, 0,alphasort ); /*count num of files*/
#	ifdef DEBUG
		printf("DEBUG:	Removing %d Files from temp dir\n", (no_files - 2));
#	endif /* DEBUG*/
	for (fcount = 0; fcount < no_files; ++fcount){
		fname = g_strdup_printf("%s%s", temp_dir, filelist[fcount]->d_name);
		ok = g_file_test(fname, G_FILE_TEST_ISFILE);
		if (ok == TRUE){
			if((ok = remove(fname)) != 0){
				g_error("Cannot delete file in temp dir\n");
				break;
			}
		}
		g_free(fname);
	}
}

/*_________________________________debug functions_________________________________*/

#ifdef DEBUG
void print_nums(void)
{
	struct PhoneNumber *cur_phone_ptr;
	printf("\n\nDEBUG:Printing phone numbers\n");
	cur_phone_ptr = phone_fst_ptr;
	while (cur_phone_ptr != NULL){
		printf("	DEBUG:Phone number=%s\n",cur_phone_ptr->phone);
		cur_phone_ptr = cur_phone_ptr->next_ptr;
	}
	printf("DEBUG:Printing phone numbers done\n");
}
#endif /*DEBUG*/
