/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <gnome.h>
#include <gal/e-paned/e-hpaned.h>

#include "gefax.h"
#include "browser.h"
#include "support.h"
#include "control.h"
#include "menu.h"
#include "toolbars.h"
#include "appbar.h"
#include "splash.h"

GtkWidget *app_e_paned;

/*check executables exist and can be executed.*/
gint check_for_executables(void);

gint check_for_executables(void)
{
	const gchar first_path[] = "/usr/bin/";
	const gchar second_path[] = "/usr/local/bin/";
	gchar *executable[2] = {"gs","fax2ps"};
	gchar *error_msg =	N_("QUITTING Gefax can not find one or\n"
					"more of the following executables\n\n"
					"           fax2ps\n"
					"           gs\n\n"
					"You need to have ghostscript and\n"
					"libTiff installed on your system");
	GtkWidget *error_dialog;
	gchar *tmp;
	gint count;

	/*test each executable with both paths*/
	for (count = 0; count < 2; count++){
		tmp = g_strdup_printf("%s%s", first_path,executable[count]);
		if (access(tmp , X_OK) == -1){
			g_free(tmp);
			tmp = g_strdup_printf("%s%s", second_path,executable[count]);
			if (access(tmp , X_OK) == -1){
				/*error msg*/
				error_dialog = gnome_message_box_new(error_msg,
							GNOME_MESSAGE_BOX_ERROR,
							GNOME_STOCK_BUTTON_CLOSE,
							NULL);
				gnome_dialog_run_and_close(GNOME_DIALOG(error_dialog));
				g_warning("missing executable");
				g_free(tmp);
				return FALSE;
			}
		}
		g_free(tmp);
	}
	return TRUE;
}
/*
 *Stole session management from gnoamba
*/
/* Session management */
gint save_state (GnomeClient *client, gint phase, GnomeRestartStyle save_style, gint shutdown,
					GnomeInteractStyle	interact_style, gint fast, gpointer client_data)
{
	extern gint efax_status(void);
	gchar *argv[3];
	/*gint x, y, w, h;
	gchar *session_id;
	
	session_id= gnome_client_get_id (client);
	
	gdk_window_get_geometry (gefax->window, &x, &y, &w, &h, NULL);
	gnome_config_push_prefix (gnome_client_get_config_prefix (client));
	gnome_config_set_int ("Geometry/x", x);
	gnome_config_set_int ("Geometry/y", y);
	gnome_config_set_int ("Geometry/w", w);
	gnome_config_set_int ("Geometry/h", h);

	gnome_config_pop_prefix ();
	gnome_config_sync();*/
	
	/*find out if we are waiting for a fax
	 so we can automatically start waiting on startup*/
	if(efax_status() == WAIT_M){
		receive_on = TRUE;
	}else{
		receive_on = FALSE;
	}
	/*save config data*/
	save_config();
	
	argv[0] = (char*) client_data;
	argv[1] = "--discard-session";
	argv[2] = gnome_client_get_config_prefix (client);
	gnome_client_set_discard_command (client, 3, argv);
	gnome_client_set_clone_command (client, 1, argv); 
	gnome_client_set_restart_command (client, 1, argv);

	return TRUE;
}

static gint die (GnomeClient *client, gpointer client_data)
{
	extern void kill_fax_operation(GtkButton *button, gpointer data);
	/*Nuke any child processes*/
	kill_fax_operation(NULL,(gint *)EXIT_NOW);
	gtk_exit (0);
	return FALSE;
}

/*someday we might have an option or two*/
static void parse_an_arg (poptContext state,enum poptCallbackReason reason,
					const struct poptOption *opt, const char *arg, void *data)
{
	
}

static struct poptOption cb_options [] = {
	{ NULL, '\0', POPT_ARG_CALLBACK, parse_an_arg, 0},
	
	{ NULL, '\0', 0, NULL, 0}
};

void gefax_app_set_contents (GtkWidget *component)
{
	e_paned_add2 (E_PANED (app_e_paned), component);
}

int main (int argc, char *argv[])
{
	gboolean first_run;
	GtkWidget *gefax_dock;
	gchar *tmp;
	GnomeClient *client;
	poptContext ctx;


#	ifdef ENABLE_NLS
  		bindtextdomain (PACKAGE, GNOMELOCALEDIR);
  		textdomain (PACKAGE);
#	endif

	/* Init Stuff that will be updated by opts */
	gnome_init_with_popt_table("gefax", VERSION, argc, argv, cb_options, 0, &ctx);
	gdk_rgb_init();
	client = gnome_master_client();

	/* Arrange to be told when something interesting happens.	*/
	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
					GTK_SIGNAL_FUNC (save_state), (gpointer) argv[0]);
	gtk_signal_connect (GTK_OBJECT (client), "die",
					GTK_SIGNAL_FUNC (die), NULL);
	
	/*clone stuff, still not sure how any of this works */
	if (GNOME_CLIENT_CONNECTED (client)) { }
	
	display_splash_window();
	while (gtk_events_pending()) gtk_main_iteration();

	/*check for the efax,efix and gs executables*/
	if (!check_for_executables()){
		gtk_exit(1);
		return(1);
	}

	/*Get users Home Dir*/
	home = gnome_util_user_home();
	
	/* set and check temp directory*/
	temp_dir = g_strdup_printf("%s/.gefax/",home);
	if (g_file_test(temp_dir, G_FILE_TEST_ISDIR) != TRUE){
		if(mkdir( temp_dir, S_IRWXU) == -1){
			g_error(_("ERROR accessing temp dir"), "%s\n",temp_dir);
		}
	}
	
	/*Load Prefs*/
	gnome_config_push_prefix ("Gefax/");
	
	first_run = gnome_config_get_bool ("Options/FirstTime=TRUE");
	
	/*Default directory for rubbish bin*/
	tmp = g_strdup_printf("Options/BinDir=%s/.gnome-desktop/Trash.gmc",home);
	bin_dir = gnome_config_get_string (tmp);
	g_free(tmp);
	
	/*Default fax directory is ~/ */
	tmp = g_strdup_printf("Options/FaxDir=%s/",home);
	fax_dir = gnome_config_get_string (tmp);
	g_free(tmp);
	
	
	/*Default directory for received fax*/
	tmp = g_strdup_printf("Options/ReceiveDir=%s",home);
	recvd_dir = gnome_config_get_string (tmp);
	g_free(tmp);
	
	
	gefax_e_paned_pos = gnome_config_get_int ("Options/PanedPos=80");
	load_all = gnome_config_get_int ("Options/LoadAll=0");
	ShortcutsToolbarOn = gnome_config_get_bool ("Options/ShortcutsToolbarOn=TRUE");
	ActiveComponent = gnome_config_get_int ("Options/ActiveComponent=0");
	load_ordered = gnome_config_get_bool ("Options/LoadInOrder=TRUE");
	remove_number_warn = gnome_config_get_bool ("Options/WarnBeforeNumberRemove=TRUE");
	auto_save_cardfile = gnome_config_get_bool ("Options/AutoSaveAddressbook=FALSE");
	EfaxArgs.usecover = gnome_config_get_int ("Options/UseCoverPage=0");
	EfaxArgs.cover = gnome_config_get_string ("Options/CoverPage=");
	EfaxArgs.header = gnome_config_get_string ("Options/Header=-hFrom Linux Box. Page %d of %d");
	EfaxArgs.ident = gnome_config_get_string ("Options/Ident=-l+00 00 000000");
	receive_on = gnome_config_get_int ("Options/AutoReceive=FALSE");
	debug_window = gnome_config_get_bool ("Options/ShowDebug=TRUE");
	
	EfaxArgs.dev = gnome_config_get_string ("Efax/Device=-d/dev/modem");
	EfaxArgs.lock = gnome_config_get_string ("Efax/Lock=-x/var/lock/LCK..modem");
	EfaxArgs.locktype = gnome_config_get_string ("Efax/LockType=modern");

	EfaxArgs.mclass = gnome_config_get_string ("Efax/ModemClass=-o2");
	EfaxArgs.xonxoff = gnome_config_get_string ("Efax/ForceXonXoff=none");
	EfaxArgs.cmddelay = gnome_config_get_string ("Efax/CommandDelay=default");
	EfaxArgs.log = gnome_config_get_string ("Efax/Log=-vi");
	EfaxArgs.init = gnome_config_get_string ("Efax/Init=-iZ&FE&D2S7=120&C0");
	EfaxArgs.vol = gnome_config_get_string ("Efax/Vol=-iM0L0");
	EfaxArgs.caps = gnome_config_get_string ("Efax/Caps=-c1,5,0,0,0,0,0,0");/*legal=-c1,5,0,1,0,0,0,0*/
	EfaxArgs.reset = gnome_config_get_string ("Efax/Reset=-kZ");
	EfaxArgs.prefix = gnome_config_get_string ("Efax/Prefix=-tT");

	/*Text file line wrapping*/
	TextWrapArgs.do_wrap = gnome_config_get_bool ("TextWrap/DoWrap=TRUE");
	TextWrapArgs.line_len = gnome_config_get_int ("TextWrap/LineLength=70");
	TextWrapArgs.tab_len = gnome_config_get_int ("TextWrap/TabLength=4");
	
	GsArgs.paper = gnome_config_get_string ("Gs/PaperSize=-sPAPERSIZE=a4");/*letter, legal, a4*/
	
	/*Printing (using fax2ps)*/
	Print.page_width = gnome_config_get_float ("Printing/PageWidth=7.0");
	Print.page_height = gnome_config_get_float ("Printing/PageHeight=10.0");
	Print.x_res = gnome_config_get_int ("Printing/X_Res=300");
	Print.y_res = gnome_config_get_int ("Printing/Y_Res=300");
	Print.convert = gnome_config_get_bool ("Printing/ConvertPostscript=TRUE");
	Print.command = gnome_config_get_string ("Printing/PrintCommand=lpr");
	Print.auto_print = gnome_config_get_bool ("Printing/AutoPrint=TRUE");

	gnome_config_sync();
	gnome_config_pop_prefix();



	
	if (first_run){	/*will have a setup wizard one day - any one?*/
		first_run = FALSE;
		gnome_config_push_prefix ("Gefax/");
		gnome_config_set_bool ("Options/FirstTime", first_run);
		gnome_config_sync();
		gnome_config_pop_prefix();
		on_btn_options_clicked(NULL,NULL);
	}
#	ifdef DEBUG
		printf("DEBUG BUILD gefax Ver %s Copyright 2000 Daryl Gray\n\n"
		"gefax comes with ABSOLUTELY NO WARRANTY.\n"
    		"This is free software, and you are welcome to redistribute it "
    		"under certain conditions.\n"
    		"For details see the GNU General Public Licence Ver 2 or above.\n\n", VERSION);
#	else DEBUG
		printf("gefax Ver %s Copyright 2000 Daryl Gray\n", VERSION);
#	endif DEBUG
	
	/*Create Main window*/
	gefax = gnome_app_new ("gefax", _("gefax"));
	
	/*set the default window height according to the screen size.
	The width is the width when the main toolbar is enabled*/
	if(gdk_screen_height() > 600){
		gtk_window_set_default_size (GTK_WINDOW (gefax), 650, 450);
	}else{
		gtk_window_set_default_size (GTK_WINDOW (gefax), 514, 350);
	}

	gefax_dock = GNOME_APP (gefax)->dock;
	gtk_widget_show (gefax_dock);
	toolbar_dock = gnome_dock_item_new("toolbar_dock",GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL | GNOME_DOCK_ITEM_BEH_LOCKED | GNOME_DOCK_ITEM_BEH_EXCLUSIVE);
	gtk_widget_show (toolbar_dock);
	
	gnome_dock_add_item (GNOME_DOCK (gefax_dock), GNOME_DOCK_ITEM(toolbar_dock), GNOME_DOCK_TOP, 1, 0, 0,TRUE);
	create_appbar();

	/*create apps menu*/
	create_main_menu(gefax);
	
	app_e_paned = e_hpaned_new();
	gtk_widget_show (app_e_paned);
	gnome_app_set_contents (GNOME_APP (gefax), app_e_paned);
	
	/*load mime types and files & Vcard*/
	on_startup_execute(gefax);
	
	create_ShortcutsToolbar(app_e_paned);
	e_paned_set_position (E_PANED (app_e_paned), gefax_e_paned_pos);
	/*catch window delete*/
	gtk_signal_connect (GTK_OBJECT (gefax), "delete_event",
		GTK_SIGNAL_FUNC (on_btn_exit_clicked), NULL);
                      
	/*Show main Window*/
	gtk_widget_show (gefax);
	
	destroy_spash(GEFAX_SPLASH_TIMEOUT);
	/*Wait for the fun and games*/
	gtk_main ();
	return 0;
}

void save_config(void)
{
	gnome_config_push_prefix("Gefax/");
	
	gnome_config_set_int ("Options/PanedPos",e_paned_get_position (E_PANED (app_e_paned)));
	gnome_config_set_string("Options/ReceiveDir", recvd_dir);
	gnome_config_set_string("Options/BinDir", bin_dir);
	gnome_config_set_string ("Options/FaxDir", fax_dir);
	gnome_config_set_bool ("Options/AutoSaveAddressbook", auto_save_cardfile);
	gnome_config_set_int ("Options/LoadAll", load_all);
	gnome_config_set_bool ("Options/ShortcutsToolbarOn", ShortcutsToolbarOn);
	gnome_config_set_int ("Options/ActiveComponent", ActiveComponent);
	gnome_config_set_bool ("Options/LoadInOrder", load_ordered);
	gnome_config_set_bool ("Options/WarnBeforeNumberRemove", remove_number_warn);
	gnome_config_set_int ("Options/AutoReceive", receive_on);
	gnome_config_set_bool ("Options/ShowDebug", debug_window);
	gnome_config_set_string ("Efax/Device", EfaxArgs.dev);
	gnome_config_set_string ("Efax/Lock", EfaxArgs.lock);
	gnome_config_set_string ("Efax/ModemClass", EfaxArgs.mclass);
	gnome_config_set_string ("Efax/Caps", EfaxArgs.caps);
	gnome_config_set_string ("Efax/Log", EfaxArgs.log);
	gnome_config_set_string ("Efax/Vol", EfaxArgs.vol);
	gnome_config_set_string ("Efax/Prefix", EfaxArgs.prefix);
	gnome_config_set_string ("Efax/Init", EfaxArgs.init);
	gnome_config_set_string ("Efax/Reset", EfaxArgs.reset);
	gnome_config_set_string ("Efax/ForceXonXoff", EfaxArgs.xonxoff);
	gnome_config_set_string ("Efax/CommandDelay", EfaxArgs.cmddelay);
	gnome_config_set_bool ("TextWrap/DoWrap", TextWrapArgs.do_wrap);
	gnome_config_set_int ("TextWrap/LineLength", TextWrapArgs.line_len);
	gnome_config_set_int ("TextWrap/TabLength", TextWrapArgs.tab_len);
	gnome_config_set_string ("Options/Header", EfaxArgs.header);
	gnome_config_set_string ("Options/Ident", EfaxArgs.ident);
	gnome_config_set_int ("Options/UseCoverPage", EfaxArgs.usecover);
	gnome_config_set_string ("Options/CoverPage", EfaxArgs.cover);
	gnome_config_set_bool ("Printing/ConvertPostscript", Print.convert);
	gnome_config_set_float ("Printing/PageWidth", Print.page_width);
	gnome_config_set_float ("Printing/PageHeight", Print.page_height);
	gnome_config_set_int ("Printing/X_Res", Print.x_res);
	gnome_config_set_int ("Printing/Y_Res", Print.y_res);
	gnome_config_set_string ("Printing/PrintCommand", Print.command);
	gnome_config_set_bool ("Printing/AutoPrint", Print.auto_print);

	gnome_config_sync();
	gnome_config_pop_prefix();

}






