/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <time.h>
#include <string.h>
#include <dirent.h>
#include <stdio.h>  
#include <sys/stat.h>
#include <gnome.h>
#include <gefax/gefax.h>
#include <gal/e-paned/e-hpaned.h>


#include "hist.h"
#include "gefax.h"
#include "browser.h"
#include "menu.h"
#include "appbar.h"
#include "print.h"
#include "viewer.h"
#include "control.h"

typedef struct _History_Config {
	gint		paned_pos;
	gint		clist_pos[4];
} History_Config;

History_Config HistoryConfig;
/*toolbar*/
typedef struct _toolbar_h {
	GtkWidget	*toolbar;
	GtkWidget	*btn_delete;
	GtkWidget	*btn_print;
	GtkWidget	*btn_view;
	GtkWidget	*btn_help;
	GtkWidget	*btn_close;
} toolbar_h;


typedef struct _History_Data {
	gint 		selected_row_key;
	gint		pages;
	gint		thumb;
	GtkWidget 	*sw_hist;
	GtkWidget 	*e_paned;
	GtkWidget 	*thumb_image;
	GtkWidget	*btn_prev;
	GtkWidget	*btn_next;
  	GtkWidget 	*clist_inbox;
  	GtkWidget 	*sort_h1;
  	GtkWidget 	*sort_h2;
  	GtkWidget 	*sort_h3;
  	GtkWidget 	*sort_h4;
	GtkWidget 	*pop_menu;
	toolbar_h 	*HistoryToolbar;
} History_Data;

History_Data *HistoryData = NULL;
GList *HistList = NULL;

/*help reference*/
static GnomeHelpMenuEntry help_ref =
{
	"gefax",
	"history.html"
};


/*gui callbacks*/
static void
on_viewer_btn_prev_clicked			(GtkButton		*b,
						History_Data		*hd);

static void
on_viewer_btn_next_clicked			(GtkButton		*b,
						History_Data		*hd);

static void
on_viewer_btn_view_clicked			(GtkButton		*b,
						History_Data		*hd);

static void
on_viewer_btn_print_clicked			(GtkButton		*b,
						History_Data		*hd);

static void
on_viewer_btn_delete_clicked			(GtkButton		*b,
						History_Data		*hd);

static void
on_viewer_btn_help_clicked			(GtkButton		*b,
						History_Data		*hd);


static void
on_clist_inbox_select_row			(GtkCList		*clist,
						gint			row,
						gint			column,
						GdkEvent		*event,
						History_Data		*hd);

static void
on_clist_inbox_unselect_row			(GtkCList		*clist,
						gint			row,
						gint			column,
						GdkEvent		*event,
						History_Data		*hd);

static void
on_clist_history_resize_column			(GtkCList			*clist,
						gint				col,
						gint				width,
						gpointer			data);

static gint
on_history_click_event				(GtkWidget		*w,
						GdkEvent		*event,
						History_Data		*hd);

/*create and destroy widgets*/
static void
create_history_menus				(History_Data		*hd);

static void
create_history_toolbar				(History_Data		*hd);

static void
destroy_history_toolbar				(History_Data		*hd);

static void
destroy_hist_viewer				(GtkButton		*b,
						gpointer		data);

/*the rest*/
static void
load_clist_in_order				(History_Data		*HistoryData);

static gchar
*get_current_date				(time_t			time);


/*main menus - add signals when it's made*/
static GnomeUIInfo history_action_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Delete Fax"), N_("Permanently delete selected fax"),
		NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Print Fax"), N_("Print selected fax"),
		NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PRINT, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_View Fax"), N_("View selected fax"),
		NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH, 0, 0, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo history_file_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Close History"), N_("Close fax history"),
		NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 0, 0, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo history_menu[] =
{
	GNOMEUIINFO_SUBTREE (N_("_Fax"),history_action_menu),
	GNOMEUIINFO_END
};

/*popup menu - add signals when it's made*/
static GnomeUIInfo history_pop_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Delete Fax"), N_("Permanently delete selected fax"),
		NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Print Fax"), N_("Print selected fax"),
		NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PRINT, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_View Fax"), N_("View selected fax"),
		NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH, 0, 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,  
	{
		GNOME_APP_UI_ITEM, N_("_Close History"), N_("Close fax history"),
		NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 0, 0, NULL
	},
	GNOMEUIINFO_END
};

static void
on_viewer_btn_prev_clicked			(GtkButton		*b,
						History_Data		*hd)
{
	Fax_Data *Fd;

	if ((hd->selected_row_key != -1) && (HistList)) {
		if (hd->thumb > 1) {
			gtk_widget_set_sensitive (hd->btn_next, TRUE);
			--hd->thumb;
			Fd = lib_gefax_history_data_get_fax_data_from_key (HistList, hd->selected_row_key);
			if (Fd) {
				gchar *thumb_path;
		
				thumb_path = lib_gefax_history_data_get_fax_page_thumb (Fd, hd->thumb);
				if (thumb_path) {
					lib_gefax_fax_viewer_load_file (LIBGEFAX_FAX_VIEWER (hd->thumb_image), thumb_path);
					g_free (thumb_path);
				}
			}
			if (hd->thumb > 1) {
				gtk_widget_set_sensitive (hd->btn_prev, TRUE);
			} else {
				gtk_widget_set_sensitive (hd->btn_prev, FALSE);
			}
		}
	}
}

static void
on_viewer_btn_next_clicked			(GtkButton		*b,
						History_Data		*hd)
{
	Fax_Data *Fd;

	if ((hd->selected_row_key != -1) && (HistList)) {
		if (hd->thumb < hd->pages) {
			gtk_widget_set_sensitive (hd->btn_prev, TRUE);
			++hd->thumb;
			Fd = lib_gefax_history_data_get_fax_data_from_key (HistList, hd->selected_row_key);
			if (Fd) {
				gchar *thumb_path;
		
				thumb_path = lib_gefax_history_data_get_fax_page_thumb (Fd, hd->thumb);
				if (thumb_path) {
					lib_gefax_fax_viewer_load_file (LIBGEFAX_FAX_VIEWER (hd->thumb_image), thumb_path);
					g_free (thumb_path);
				}
			}
			if (hd->pages > hd->thumb) {
				gtk_widget_set_sensitive (hd->btn_next, TRUE);
			} else {
				gtk_widget_set_sensitive (hd->btn_next, FALSE);
			}
		}
	}
}

static void
on_viewer_btn_view_clicked			(GtkButton		*b,
						History_Data		*hd)
{
	if ((hd->selected_row_key != -1) && (HistList)) {
		GList *Fl = NULL;
		
		Fl = lib_gefax_history_data_get_file_list_from_key (HistList, hd->selected_row_key);
		if (Fl) {
			change_to_viewer(NULL, Fl);
		}
	}
}

static void
on_viewer_btn_print_clicked			(GtkButton		*b,
						History_Data		*hd)
{
	Fax_Data *Fd = NULL;
	time_t curtime;
	
	if ((hd->selected_row_key != -1) && (HistList)) {
		Fd = lib_gefax_history_data_get_fax_data_from_key (HistList, hd->selected_row_key);
		if (Fd) {
			print_fax_pages (Fd->directory, FALSE);
			curtime = time (NULL);
			Fd->print_time = curtime;
			load_clist_in_order (HistoryData);
			hd->selected_row_key = -1;
		}
	}
}

static void
on_viewer_btn_delete_clicked			(GtkButton		*b,
						History_Data		*hd)
{

	GtkWidget *fwarn;
	
	if (hd->selected_row_key == -1) {
		return;
	}

	/*confirm first*/
	fwarn = gnome_message_box_new (_("Permenently Delete Fax?"),
					GNOME_MESSAGE_BOX_QUESTION,
					GNOME_STOCK_BUTTON_OK,
					GNOME_STOCK_BUTTON_CANCEL, NULL);
	gtk_window_set_position (GTK_WINDOW (fwarn), GTK_WIN_POS_MOUSE);
	if ((gnome_dialog_run_and_close (GNOME_DIALOG (fwarn))) == 0) {
		gint row;
		Fax_Data *Fd;
	
		row = gtk_clist_find_row_from_data (GTK_CLIST (hd->clist_inbox), GINT_TO_POINTER (hd->selected_row_key));
		Fd = lib_gefax_history_data_get_fax_data_from_key (HistList , hd->selected_row_key);
		remove_receive_directory (Fd->directory);
		HistList = lib_gefax_history_data_remove_fax_data_from_key (HistList , hd->selected_row_key);
		gtk_clist_remove (GTK_CLIST (hd->clist_inbox), row);
		hd->selected_row_key = -1;
		if (!lib_gefax_history_data_save_xml_file (recvd_dir, HistList)) {
			g_warning ("Saving gefax.metadata failed");
		}
	}

}

static void
on_viewer_btn_help_clicked			(GtkButton		*b,
						History_Data		*hd)
{
	gnome_help_display("ignored", &help_ref);

}

static void
on_clist_inbox_select_row			(GtkCList		*clist,
						gint			row,
						gint			column,
						GdkEvent		*event,
						History_Data		*hd)
{
	Fax_Data *Fd;
	hd->selected_row_key = GPOINTER_TO_INT (gtk_clist_get_row_data (clist, row));
	
	Fd = lib_gefax_history_data_get_fax_data_from_key (HistList, hd->selected_row_key);
	if (Fd) {
		gchar *thumb_path;
		
		thumb_path = lib_gefax_history_data_get_fax_page_thumb (Fd, 1);
		if (thumb_path) {
			hd->thumb = 1;
			lib_gefax_fax_viewer_load_file (LIBGEFAX_FAX_VIEWER (hd->thumb_image), thumb_path);
			g_free (thumb_path);
			hd->pages = Fd->pages;
			if (hd->pages > 1) {
				gtk_widget_set_sensitive (hd->btn_next, TRUE);
			}
		}
	}
}

static void
on_clist_inbox_unselect_row			(GtkCList		*clist,
						gint			row,
						gint			column,
						GdkEvent		*event,
						History_Data		*hd)
{
	hd->selected_row_key = -1;
	hd->pages = 0;
	hd->thumb = 0;
	gtk_widget_set_sensitive (hd->btn_prev, FALSE);
	gtk_widget_set_sensitive (hd->btn_next, FALSE);
}

static gint
on_history_click_event				(GtkWidget		*w,
						GdkEvent		*event,
						History_Data		*hd)
{
	if (event->type == GDK_BUTTON_PRESS) {
		if(event->button.button == 3) {
			GdkEventButton *bevent = (GdkEventButton *) event;
			gtk_menu_popup (GTK_MENU (hd->pop_menu), NULL, NULL, NULL, NULL,
							bevent->button, bevent->time);
			return TRUE;
		}
	}
	return FALSE;
}


/*create and destroy widgets*/
static void
create_history_menus				(History_Data		*hd)
{
	
	hd->pop_menu = gtk_menu_new ();
	gnome_app_fill_menu (GTK_MENU_SHELL (hd->pop_menu), history_pop_menu, NULL, FALSE, 0);
	
	/*poup menu signals*/
	gtk_signal_connect(GTK_OBJECT (history_pop_menu[4].widget), "activate",
                      GTK_SIGNAL_FUNC (change_to_browser), NULL);
	gtk_signal_connect(GTK_OBJECT (history_pop_menu[1].widget), "activate",
                      GTK_SIGNAL_FUNC (on_viewer_btn_print_clicked), hd);
	gtk_signal_connect (GTK_OBJECT (history_pop_menu[2].widget), "activate",
                      GTK_SIGNAL_FUNC (on_viewer_btn_view_clicked), hd);
	gtk_signal_connect (GTK_OBJECT (history_pop_menu[0].widget), "activate",
                      GTK_SIGNAL_FUNC (on_viewer_btn_delete_clicked), hd);
                      
	/*add the menus to main app*/
	gnome_app_insert_menus (GNOME_APP (gefax),"File/",history_file_menu);
	gnome_app_insert_menus (GNOME_APP (gefax),"File",history_menu);
	
	/*action menu signals*/
	gtk_signal_connect(GTK_OBJECT (history_file_menu[0].widget), "activate",
                      GTK_SIGNAL_FUNC (change_to_browser), NULL);
	gtk_signal_connect(GTK_OBJECT (history_action_menu[1].widget), "activate",
                      GTK_SIGNAL_FUNC (on_viewer_btn_print_clicked), hd);
	gtk_signal_connect (GTK_OBJECT (history_action_menu[2].widget), "activate",
                      GTK_SIGNAL_FUNC (on_viewer_btn_view_clicked), hd);
	gtk_signal_connect (GTK_OBJECT (history_action_menu[0].widget), "activate",
                      GTK_SIGNAL_FUNC (on_viewer_btn_delete_clicked), hd);
                      
	/*menu hints*/
	gnome_app_install_menu_hints(GNOME_APP (gefax),history_file_menu);
	gnome_app_install_menu_hints(GNOME_APP (gefax),history_menu);
	gnome_app_install_menu_hints(GNOME_APP (gefax),history_pop_menu);
	
	/*window title*/
	gtk_window_set_title(GTK_WINDOW(gefax),"gefax - History");

}

static void
create_history_toolbar				(History_Data		*hd)
{
	GtkWidget *tmp_toolbar_icon;
	GtkAccelGroup *accel_group;
	
	accel_group = gtk_accel_group_new ();
	hd->HistoryToolbar = g_malloc(sizeof(toolbar_h));

	hd->HistoryToolbar->toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  
	gtk_container_add (GTK_CONTAINER (toolbar_dock), hd->HistoryToolbar->toolbar);
	gtk_container_set_border_width (GTK_CONTAINER (hd->HistoryToolbar->toolbar), 2);
	gtk_toolbar_set_space_style (GTK_TOOLBAR (hd->HistoryToolbar->toolbar), GTK_TOOLBAR_SPACE_LINE);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (hd->HistoryToolbar->toolbar), GTK_RELIEF_NONE);
  
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_TRASH);
	hd->HistoryToolbar->btn_delete = gtk_toolbar_append_element (GTK_TOOLBAR (hd->HistoryToolbar->toolbar),
		GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Delete"), _("Delete Fax"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (hd->HistoryToolbar->btn_delete);
	gtk_widget_show(hd->HistoryToolbar->toolbar);
	while (gtk_events_pending()) gtk_main_iteration();
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_PRINT);
	hd->HistoryToolbar->btn_print = gtk_toolbar_append_element (GTK_TOOLBAR (hd->HistoryToolbar->toolbar),
		GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Print"), _("Print Fax"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (hd->HistoryToolbar->btn_print);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_SEARCH);
	hd->HistoryToolbar->btn_view = gtk_toolbar_append_element (GTK_TOOLBAR (hd->HistoryToolbar->toolbar),
		GTK_TOOLBAR_CHILD_BUTTON, NULL, _("View"), _("View Fax"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (hd->HistoryToolbar->btn_view);

	gtk_toolbar_append_space (GTK_TOOLBAR (hd->HistoryToolbar->toolbar));

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_HELP);
	hd->HistoryToolbar->btn_help = gtk_toolbar_append_element (GTK_TOOLBAR (hd->HistoryToolbar->toolbar),
		GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Help"), _("History Help"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (hd->HistoryToolbar->btn_help);
	gtk_widget_add_accelerator (hd->HistoryToolbar->btn_help, "clicked", accel_group, GDK_F1, 0, GTK_ACCEL_VISIBLE);

	gtk_toolbar_append_space (GTK_TOOLBAR (hd->HistoryToolbar->toolbar));

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_CLOSE);
	hd->HistoryToolbar->btn_close = gtk_toolbar_append_element (GTK_TOOLBAR (hd->HistoryToolbar->toolbar),
		GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Close"), _("Close"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (hd->HistoryToolbar->btn_close);
                            
	gtk_signal_connect (GTK_OBJECT (hd->HistoryToolbar->btn_close), "clicked",
					GTK_SIGNAL_FUNC (change_to_browser), NULL);
	gtk_signal_connect (GTK_OBJECT (hd->HistoryToolbar->btn_print), "clicked",
					GTK_SIGNAL_FUNC (on_viewer_btn_print_clicked), hd);
	gtk_signal_connect (GTK_OBJECT (hd->HistoryToolbar->btn_view), "clicked",
					GTK_SIGNAL_FUNC (on_viewer_btn_view_clicked), hd);
	gtk_signal_connect (GTK_OBJECT (hd->HistoryToolbar->btn_delete), "clicked",
					GTK_SIGNAL_FUNC (on_viewer_btn_delete_clicked), hd);
	gtk_signal_connect (GTK_OBJECT (hd->HistoryToolbar->btn_help), "clicked",
					GTK_SIGNAL_FUNC (on_viewer_btn_help_clicked), hd);
	gtk_window_add_accel_group (GTK_WINDOW (gefax), accel_group);
	
}

static void
destroy_history_toolbar				(History_Data		*hd)
{
	
	gtk_widget_destroy (hd->HistoryToolbar->toolbar);
	hd->HistoryToolbar->toolbar = NULL;
	g_free (hd->HistoryToolbar);
	hd->HistoryToolbar = NULL;
}

static void
destroy_hist_viewer				(GtkButton		*b,
						gpointer		data)
{
	
	if (!HistoryData) {
		return;
	}

	destroy_history_toolbar (HistoryData);
	
	/*remove the menus - have to destroy pop menu first!*/
	gtk_widget_destroy (GTK_WIDGET (HistoryData->pop_menu));
	HistoryConfig.paned_pos = e_paned_get_position (E_PANED (HistoryData->e_paned));
	gtk_widget_destroy (GTK_WIDGET (HistoryData->e_paned));
	HistoryData = NULL;
	gnome_app_remove_menus (GNOME_APP (gefax),"File/Close History", 1);
	gnome_app_remove_menus (GNOME_APP (gefax),"Fax", 1);
	return;
}

/*the rest*/
static void
load_clist_in_order				(History_Data		*HistoryData)
{
	GList *Hl = HistList;
	gint row = 0;

	gtk_clist_freeze ((GtkCList *) HistoryData->clist_inbox);
	gtk_clist_clear ((GtkCList *) HistoryData->clist_inbox);
	
	while (Hl) {
		gchar *add_rows[6];
		gint i, r;
		Fax_Data *Fd = Hl->data;
		
		add_rows[0] = g_strdup_printf ("%ld",Fd->save_time);
		add_rows[1] = g_strdup_printf ("%ld",Fd->print_time);
  		add_rows[2] = get_current_date ((time_t) Fd->save_time);
  		add_rows[3] = g_strdup_printf ("%d",Fd->pages);
  		if (Fd->print_time == -1) {
  			add_rows[4] = g_strdup ("Not yet printed");
  		} else {
  			add_rows[4] = get_current_date ((time_t) Fd->print_time);
  		}
  		add_rows[5] = g_strdup (Fd->polled);
  		r = gtk_clist_insert (GTK_CLIST (HistoryData->clist_inbox), row, add_rows);
  		gtk_clist_set_row_data (GTK_CLIST (HistoryData->clist_inbox), r, GINT_TO_POINTER (Fd->key));
		++row;
  		for (i = 0; i < 6; i++) {
  			g_free (add_rows[i]);
  		}
  		Hl = g_list_next (Hl);
  	}
	gtk_clist_thaw ((GtkCList *) HistoryData->clist_inbox);
}

static gchar
*get_current_date				(time_t			time)
{
	const gint  SIZE = 256;
	gchar buff[SIZE];
	struct tm *loctime;
	
	loctime = localtime (&time);
	strftime (buff, SIZE, "%a %d %b %Y %l%p", loctime);
	return (g_strdup (buff));
}


/*==============================================Public=======================*/

void
create_hist_viewer				(void)
{
	GSList *ab_group = NULL;
	GtkWidget *vbox;
	GtkWidget *hbox_nav;
	
 
	HistoryData = (History_Data *) g_malloc0(sizeof (History_Data));
	create_history_toolbar(HistoryData);
	ActiveComponent = HISTORY_ON;
	
	HistoryData->e_paned = e_hpaned_new();
	
	gtk_widget_show (HistoryData->e_paned);
	gefax_app_set_contents (HistoryData->e_paned);
  
	HistoryData->sw_hist = gtk_scrolled_window_new (NULL, NULL);
	e_paned_pack1 (E_PANED (HistoryData->e_paned), HistoryData->sw_hist, FALSE, TRUE);
	
	HistoryData->thumb_image = lib_gefax_fax_viewer_new ();
	gtk_widget_show (HistoryData->thumb_image);
	lib_gefax_fax_viewer_set_scale (LIBGEFAX_FAX_VIEWER (HistoryData->thumb_image), 1.0);
	
	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);

	e_paned_pack2 (E_PANED (HistoryData->e_paned), vbox, FALSE, TRUE);
	gtk_box_pack_start (GTK_BOX (vbox), HistoryData->thumb_image, TRUE, TRUE, 0);
	
	hbox_nav = gtk_hbox_new (TRUE, 0);
	gtk_widget_show (hbox_nav);
	gtk_box_pack_start (GTK_BOX (vbox), hbox_nav, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox_nav), 5);

	HistoryData->btn_prev = gnome_stock_button (GNOME_STOCK_BUTTON_PREV);
	gtk_widget_show (HistoryData->btn_prev);
	gtk_box_pack_start (GTK_BOX (hbox_nav), HistoryData->btn_prev, TRUE, FALSE, 0);
	gtk_button_set_relief (GTK_BUTTON (HistoryData->btn_prev), GTK_RELIEF_NONE);
	gtk_widget_set_sensitive (HistoryData->btn_prev, FALSE);

	HistoryData->btn_next = gnome_stock_button (GNOME_STOCK_BUTTON_NEXT);
	gtk_widget_show (HistoryData->btn_next);
	gtk_box_pack_start (GTK_BOX (hbox_nav), HistoryData->btn_next, TRUE, FALSE, 0);
	gtk_button_set_relief (GTK_BUTTON (HistoryData->btn_next), GTK_RELIEF_NONE);
	gtk_widget_set_sensitive (HistoryData->btn_next, FALSE);

	e_paned_set_position (E_PANED (HistoryData->e_paned), HistoryConfig.paned_pos);

	HistoryData->clist_inbox = gtk_clist_new (6);
	gtk_widget_show (HistoryData->clist_inbox);
	gtk_clist_set_shadow_type(GTK_CLIST (HistoryData->clist_inbox),GTK_SHADOW_IN);
	gtk_container_add (GTK_CONTAINER (HistoryData->sw_hist), HistoryData->clist_inbox);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (HistoryData->sw_hist), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	HistoryData->sort_h1 = gtk_sort_heading_new(ab_group,GTK_CLIST(HistoryData->clist_inbox),2,_("Time Received"));
	gtk_sort_heading_set_label_justify(GTK_SORT_HEADING(HistoryData->sort_h1), GTK_JUSTIFY_LEFT);
  	ab_group = gtk_sort_heading_group(GTK_SORT_HEADING(HistoryData->sort_h1));
  	gtk_sort_heading_set_sort_column(GTK_SORT_HEADING(HistoryData->sort_h1), 0);
  	gtk_widget_show(HistoryData->sort_h1);

	HistoryData->sort_h2 = gtk_sort_heading_new(ab_group,GTK_CLIST(HistoryData->clist_inbox),3,_("Pages"));
	gtk_sort_heading_set_label_justify(GTK_SORT_HEADING(HistoryData->sort_h2), GTK_JUSTIFY_LEFT);
  	ab_group = gtk_sort_heading_group(GTK_SORT_HEADING(HistoryData->sort_h2));
  	gtk_widget_show(HistoryData->sort_h2);
	
	HistoryData->sort_h3 = gtk_sort_heading_new(ab_group,GTK_CLIST(HistoryData->clist_inbox),4,_("Last Printed"));
	gtk_sort_heading_set_label_justify(GTK_SORT_HEADING(HistoryData->sort_h3), GTK_JUSTIFY_LEFT);
	gtk_sort_heading_set_sort_column(GTK_SORT_HEADING(HistoryData->sort_h3), 1);
  	ab_group = gtk_sort_heading_group(GTK_SORT_HEADING(HistoryData->sort_h3));
  	gtk_widget_show(HistoryData->sort_h3);
	
	HistoryData->sort_h4 = gtk_sort_heading_new(ab_group,GTK_CLIST(HistoryData->clist_inbox),5,_("Polled"));
	gtk_sort_heading_set_label_justify(GTK_SORT_HEADING(HistoryData->sort_h4), GTK_JUSTIFY_LEFT);
  	ab_group = gtk_sort_heading_group(GTK_SORT_HEADING(HistoryData->sort_h4));
  	gtk_widget_show(HistoryData->sort_h4);

	gtk_clist_set_column_widget (GTK_CLIST (HistoryData->clist_inbox),2,HistoryData->sort_h1);
	gtk_clist_set_column_widget (GTK_CLIST (HistoryData->clist_inbox),3,HistoryData->sort_h2);
	gtk_clist_set_column_widget (GTK_CLIST (HistoryData->clist_inbox),4,HistoryData->sort_h3);
	gtk_clist_set_column_widget (GTK_CLIST (HistoryData->clist_inbox),5,HistoryData->sort_h4);
	
	gtk_clist_set_column_width (GTK_CLIST (HistoryData->clist_inbox), 0, 0);
	gtk_clist_set_column_width (GTK_CLIST (HistoryData->clist_inbox), 1, 0);
	gtk_clist_set_column_width (GTK_CLIST (HistoryData->clist_inbox), 2, HistoryConfig.clist_pos[0]);
	gtk_clist_set_column_width (GTK_CLIST (HistoryData->clist_inbox), 3, HistoryConfig.clist_pos[1]);
	gtk_clist_set_column_width (GTK_CLIST (HistoryData->clist_inbox), 4, HistoryConfig.clist_pos[2]);
	gtk_clist_set_column_width (GTK_CLIST (HistoryData->clist_inbox), 5, HistoryConfig.clist_pos[3]);
	gtk_clist_column_titles_show (GTK_CLIST (HistoryData->clist_inbox));
	
	gtk_clist_column_titles_passive(GTK_CLIST (HistoryData->clist_inbox));
  
	/*column 0,1 is only for identifying the row as it is unique*/
	gtk_clist_set_column_visibility(GTK_CLIST (HistoryData->clist_inbox),0,FALSE);
	gtk_clist_set_column_visibility(GTK_CLIST (HistoryData->clist_inbox),1,FALSE);
  
	gtk_signal_connect (GTK_OBJECT (HistoryData->clist_inbox), "select_row",
					GTK_SIGNAL_FUNC (on_clist_inbox_select_row), HistoryData);
	gtk_signal_connect (GTK_OBJECT (HistoryData->clist_inbox), "unselect_row",
					GTK_SIGNAL_FUNC (on_clist_inbox_unselect_row), HistoryData);
	gtk_signal_connect (GTK_OBJECT (HistoryData->clist_inbox), "resize_column",
					GTK_SIGNAL_FUNC (on_clist_history_resize_column), NULL);
	gtk_signal_connect (GTK_OBJECT (HistoryData->btn_next), "clicked",
					GTK_SIGNAL_FUNC (on_viewer_btn_next_clicked), HistoryData);
	gtk_signal_connect (GTK_OBJECT (HistoryData->btn_prev), "clicked",
					GTK_SIGNAL_FUNC (on_viewer_btn_prev_clicked), HistoryData);
  
	/*right click menu callback*/
	gtk_signal_connect(GTK_OBJECT(HistoryData->clist_inbox), "event",
					GTK_SIGNAL_FUNC (on_history_click_event), HistoryData);
	gtk_widget_show (HistoryData->sw_hist);

	gtk_sort_heading_set_mode(GTK_SORT_HEADING(HistoryData->sort_h1), GTK_SORT_HEADING_ASCENDING);

	/*setup menu*/
	while (gtk_events_pending ()) gtk_main_iteration ();
	create_history_menus (HistoryData);

	if (HistList) {
		load_clist_in_order (HistoryData);
	}
	return;
}

void
history_destroy_yourself				(void)
{
	destroy_hist_viewer(NULL,NULL);
}

static void
on_clist_history_resize_column			(GtkCList			*clist,
						gint				col,
						gint				width,
						gpointer			data)
{
	if (col > 0) {
		HistoryConfig.clist_pos[col - 2] = width;
	}
}

void
history_load_config					(void)
{
	HistList = lib_gefax_history_data_read_xml_file (recvd_dir);
	if (!HistList) {
		/*Try Loading from ver < 0.4-3*/
		HistList = lib_gefax_history_data_read_old_history (recvd_dir);
		/*Save if it worked*/
		if (HistList) {
			lib_gefax_history_data_save_xml_file (recvd_dir, HistList);
		}
	}
	gnome_config_push_prefix ("Gefax-History/");

	HistoryConfig.paned_pos = gnome_config_get_int ("UserConfig/PanedPos=350");
	HistoryConfig.clist_pos[0] = gnome_config_get_int ("UserConfig/ClistPos[0]=130");
	HistoryConfig.clist_pos[1] = gnome_config_get_int ("UserConfig/ClistPos[1]=50");
	HistoryConfig.clist_pos[2] = gnome_config_get_int ("UserConfig/ClistPos[2]=110");
	HistoryConfig.clist_pos[3] = gnome_config_get_int ("UserConfig/ClistPos[3]=60");
	
	gnome_config_sync ();
	gnome_config_pop_prefix ();
}

void
history_save_config					(void)
{
	if (HistList) {
		lib_gefax_history_data_save_xml_file (recvd_dir, HistList);
		lib_gefax_history_data_free_data (HistList);
		HistList = NULL;
	}
	gnome_config_push_prefix ("Gefax-History/");
	gnome_config_set_int ("UserConfig/PanedPos", HistoryConfig.paned_pos);
	gnome_config_set_int ("UserConfig/ClistPos[0]", HistoryConfig.clist_pos[0]);
	gnome_config_set_int ("UserConfig/ClistPos[1]", HistoryConfig.clist_pos[1]);
	gnome_config_set_int ("UserConfig/ClistPos[2]", HistoryConfig.clist_pos[2]);
	gnome_config_set_int ("UserConfig/ClistPos[3]", HistoryConfig.clist_pos[3]);

	gnome_config_sync ();
	gnome_config_pop_prefix ();
}

gchar
*get_receive_directory					(void)
{
	const gint  SIZE = 256;
	gchar buff[SIZE];
	time_t curtime;
	struct tm *loctime;
	gchar *dir;
	
	curtime = time (NULL);
	loctime = localtime (&curtime);
	strftime (buff, SIZE, "%j_%H_%M_%S", loctime);
	dir = g_strdup_printf ("%s%s", recvd_dir , buff);
	if (!g_file_test (dir, G_FILE_TEST_ISDIR)) {
		if (mkdir ( dir, S_IRWXU) == -1) {
			g_error("ERROR making DIR");
		}
	}
	return (dir);

}

void
remove_receive_directory				(gchar		*dir)
{
	gint no_files;
	gint fcount;
	gchar *fname;
	struct dirent **filelist;
	
	if (!g_file_test (dir, G_FILE_TEST_ISDIR)) {
		return;
	}

	/*delete any files*/
	no_files = scandir (dir, &filelist, 0, alphasort ); /*count num of files*/
	for (fcount = 0; fcount < no_files; ++fcount){
		fname = g_strdup_printf ("%s/%s", dir, filelist[fcount]->d_name);
		if (g_file_test (fname, G_FILE_TEST_ISFILE)) {
			if ((remove (fname)) != 0) {
				break;
			}
		}
		g_free (fname);
	}
	if (rmdir (dir) == -1) {
		gnome_error_dialog (_("Can't Remove Directory\nPlease remove the directory manually"));		
	}
	return;
}

/*_________________________________________MAKE DATA FILE_________________________________________*/


void
save_fax_data_to_directory			(gchar		*dir,
						gint		autoprint,
						gint		polled)
{
	time_t curtime;
	Fax_Data *Fd;
	gint no_files;
	gint fcount;
	gchar *fname;
	gchar *ftype;
	struct dirent **filelist;
	
	curtime = time (NULL);
	Fd = g_malloc0 (sizeof (Fax_Data));
	
	Fd->directory = g_strdup (dir);
	Fd->save_time = curtime;
	Fd->pages = 0;
	if (polled == TRUE) {
		Fd->polled = g_strdup ("yes");
	} else {
		Fd->polled = g_strdup ("no");
	}
	
	/*get file list*/
	no_files = scandir(dir, &filelist, 0, alphasort );

	/*count the tiff files*/
	for (fcount = 0; fcount < no_files; ++fcount) {
		fname = g_strdup_printf("%s/%s", dir, filelist[fcount]->d_name);
		if (g_file_test (fname, G_FILE_TEST_ISFILE)) {
			ftype = g_strdup (gnome_mime_type_or_default_of_file (fname, "UNKNOWN"));
			if (strcmp (ftype, "image/tiff") == 0) {
				gchar *command, *tmp;
				Fax_Page *Fp = g_malloc0 (sizeof (Fax_Page));
				
				++Fd->pages;
				Fp->index = Fd->pages;
				Fp->page_file_name = g_strdup_printf ("Page%d.tiff", Fp->index);
				Fp->page_thumb_file_name = g_strdup_printf ("Page%d.tiff.thumb", Fp->index);
				tmp = g_strdup_printf ("%s%s", Fd->directory, Fp->page_file_name);
				if (rename (fname, tmp) == -1){
					g_warning ("Error moving file");
				}
				g_free (tmp);
				command = g_strdup_printf ("thumbnail %s%s %s%s", Fd->directory,
						Fp->page_file_name, Fd->directory, Fp->page_thumb_file_name);
				if (gnome_execute_shell (Fd->directory, command) == -1) {
					g_warning ("Unable to generate thumbnails");
				}
				Fd->Pages = g_slist_append (Fd->Pages, Fp);			
			}
			g_free (ftype);
		}
		g_free (fname);
	}
	if (autoprint == TRUE) {
		Fd->print_time = curtime;
		print_fax_pages (Fd->directory, TRUE);
	} else {
		Fd->print_time = -1;
	}
	HistList = lib_gefax_history_data_append_fax_data (HistList, Fd);
	lib_gefax_history_data_save_xml_file (recvd_dir, HistList);
	printf ("%d\n", Fd->key);
	
	return;
}

