/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gnome.h>

#include "control.h"
#include "menu.h"
#include "support.h"
#include "exec_cmd.h"
#include "gefax.h"
#include "toolbars.h"
#include "browser.h"

 
static GnomeUIInfo file_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Open Directory"), N_("Open alternate directory"),
		on_btn_open_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Preview Fax"), N_("Create and preview fax"),
	on_btn_view_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH, 0, 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	{
		GNOME_APP_UI_ITEM, N_("_Send Fax"), N_("Fax selected files"),
		on_btn_send_fax_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_MAIL_SND, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Receive Fax"), N_("Answer fax call immediately or wait for fax"),
		on_btn_receive_fax_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_MAIL_RCV, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Poll Fax"), N_("Dial and receive a fax"),
		on_btn_poll_fax_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_MAIL_FWD, 0, 0, NULL
	},
	GNOMEUIINFO_END
};


static GnomeUIInfo stop_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Cancel Fax"), N_("Stop sending receiving or polling fax"),
		kill_fax_operation, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 0, 0, NULL
	},
	GNOMEUIINFO_END
};
static GnomeUIInfo exit_menu[] =
{
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM (on_btn_exit_clicked, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo select_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Select All Files"), N_("Select first 9 files for faxing"),
		on_btn_select_all_clicked, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, GEFAX_ICON_BROWSER_MENU_SELECT, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Refresh File List"), N_("Clear file list and reload files"),
		on_btn_clear_all_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REFRESH, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Add Fax Numbers"), N_("Add fax numbers"),
		on_btn_set_nums_clicked, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, GEFAX_ICON_BROWSER_MENU_NUMBERS, 0, 0, NULL
	},
	GNOMEUIINFO_END
};
static GnomeUIInfo trash_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Move To Trash"), N_("Move selected files to trash"),
		on_btn_move_to_trash_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Retrieve From Trash"), N_("Retrieve files from trash"),
		on_btn_get_from_trash_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDELETE, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Clear Trash"), N_("Remove all files from trash"),
		on_btn_delete_trash_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH_FULL, 0, 0, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo view_menu[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_View History"), N_("Open received fax browser"),
		change_to_history, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_INDEX, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_View Contacts"), N_("Open address book"),
		change_to_contacts, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_INDEX, 0, 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_PREFERENCES_ITEM (on_btn_options_clicked, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo browser_menu[] =
{
	GNOMEUIINFO_SUBTREE (N_("_Fax"),select_menu),
	GNOMEUIINFO_SUBTREE (N_("_Trash"),trash_menu),
	GNOMEUIINFO_SUBTREE (N_("_View"),view_menu),
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu[] =
{
	GNOMEUIINFO_HELP ("gefax"),
	GNOMEUIINFO_MENU_ABOUT_ITEM (on_mnu_about_activate, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menubar[] =
{
	GNOMEUIINFO_SUBTREE (N_("_File"),exit_menu),
	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
	GNOMEUIINFO_END
};

void create_main_menu(GtkWidget *gefax)
{
	gnome_app_create_menus (GNOME_APP (gefax), menubar);
	/*gtk_menu_item_right_justify(GTK_MENU_ITEM(menubar[1].widget));*/
	gnome_app_install_menu_hints (GNOME_APP (gefax), menubar);
	return;
}

void adjust_menu(gint action)
{
		gnome_app_remove_menus(GNOME_APP (gefax),"File/Open Directory",6);
	switch(action){
		case SEND_A:
				gnome_app_insert_menus (GNOME_APP (gefax),"File/",stop_menu);
				gtk_window_set_title(GTK_WINDOW(gefax),"gefax - Sending Fax");
				break;
			case RECEIVE_A:
				gnome_app_insert_menus (GNOME_APP (gefax),"File/",stop_menu);
				gtk_window_set_title(GTK_WINDOW(gefax),"gefax - Receive Mode");
				break;
			case POLL_A:
				gnome_app_insert_menus (GNOME_APP (gefax),"File/",stop_menu);
				gtk_window_set_title(GTK_WINDOW(gefax),"gefax - Polling Fax");
				break;
			default:
				break;
	}
	return;
}

void remove_browser_menu(void)
{
	gnome_app_remove_menus(GNOME_APP (gefax),"File/Open Directory",6);
	gnome_app_remove_menus(GNOME_APP (gefax),"Fax",3);
}

void restore_browser_menu(void)
{
	gchar *title;
	gnome_app_insert_menus (GNOME_APP (gefax),"File/",file_menu);
	gnome_app_insert_menus (GNOME_APP (gefax),"File",browser_menu);
	gnome_app_install_menu_hints (GNOME_APP (gefax), file_menu);
	gnome_app_install_menu_hints (GNOME_APP (gefax), browser_menu);
	title = g_strdup_printf("gefax - Directory: %s", fax_dir);
	gtk_window_set_title(GTK_WINDOW(gefax), title);
	g_free(title);

	return;
}

GtkWidget* create_browser_pop_menu(void)
{
  	GtkWidget *browser_pop_menu;

	browser_pop_menu = gtk_menu_new ();
	gtk_object_set_data (GTK_OBJECT (browser_pop_menu), "browser_pop_menu", browser_pop_menu);
	gnome_app_fill_menu (GTK_MENU_SHELL (browser_pop_menu), select_menu,
	                     NULL, FALSE, 0);
	gnome_app_install_menu_hints(GNOME_APP (gefax),select_menu);
	return browser_pop_menu;
}

void disable_kill_menu(void)
{

	gtk_widget_set_sensitive(GTK_WIDGET(stop_menu[0].widget),FALSE);
}

void remove_kill_menu(void)
{
	gchar *title;
	gnome_app_remove_menus(GNOME_APP (gefax),"File/Cancel Fax",1);
	gnome_app_insert_menus (GNOME_APP (gefax),"File/",file_menu);
	title = g_strdup_printf("gefax - Directory: %s", fax_dir);
	gtk_window_set_title(GTK_WINDOW(gefax), title);
	g_free(title);

}

/*end*/





