/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <stdio.h>  
#include <gnome.h>
#include <fnmatch.h>

#include "print.h"
#include "gefax.h"
#include "appbar.h"




struct p_pages{
	gchar	*page;				/*full path to filename*/
	gint		*no;
	struct	p_pages *next_ptr;		/*pointer to the next element*/
};

typedef struct _print_d{
	GtkWidget *spin_first;
	GtkWidget *lbl_to;
	GtkWidget *spin_last;
	GtkWidget *dialog_print;
	GSList *location_group;
	GtkWidget *dialog_vbox1;
	GtkWidget *dialog_hbox1;
	GtkWidget *lbl_print_to;
	GtkWidget *rad_printer;
	GtkWidget *rad_file;
	GtkWidget *hseperator1;
	GtkWidget *lbl_range;
	GSList *range_group;
	GtkWidget *rad_all;
	GtkWidget *hbox2;
	GtkWidget *rad_from;
	GtkObject *spin_last_adj;
	GtkObject *spin_first_adj;
	GtkWidget *hseparator2;
	GtkWidget *hbox3;
	GtkWidget *lbl_copies;
	GtkObject *spin_copies_adj;
	GtkWidget *spin_copies;
	GtkWidget *dialog_action_area;
	GtkWidget *btn_print;
	GtkWidget *btn_cancel;
	GtkWidget *fs;
	guint destroy_signal;
	gchar *dir;
	gchar *pfile;
	gboolean print_all;
	gboolean print_file;
	gint first;
	gint last;
	gint files;
	gint copies;
	struct p_pages *filelist;
} print_d;

static gboolean print_active = FALSE;

static void create_dialog_print (gint files, struct p_pages *filelist, gchar *dir);
static void on_rad_all_toggled(GtkToggleButton *tb,  print_d *PrintDialog);
static void on_rad_file_toggled(GtkToggleButton *tb, print_d *PrintDialog);
static void on_spin_first_changed(GtkSpinButton *first,  print_d *PrintDialog);
static void on_spin_last_changed(GtkSpinButton *slast,  print_d *PrintDialog);
static void on_spin_copies_changed(GtkSpinButton *scopies,  print_d *PrintDialog);
static void on_btn_cancel_clicked(GtkButton *btn,  print_d *PrintDialog);
static void on_btn_print_clicked(GtkButton *btn,  print_d *PrintDialog);
static void on_fs_ok_clicked(GtkButton *btn,  print_d *PrintDialog);
static void on_fs_cancel_clicked(GtkButton *btn,  print_d *PrintDialog);
static void print_gui_file(gchar *cmd, print_d *PrintDialog);

void print_fax_pages(gchar *dir, gint all)
{
	gint no_files;
	gint no_pages;
	gint fcount;
	gchar *fname;
	gchar *tmp;
	gchar *cmd;
	struct dirent **filelist;
	struct p_pages *first_p_page_ptr = NULL;
	struct p_pages *current_p_page_ptr = NULL;
	
	if(print_active) return;
	print_active = TRUE;	
	if (g_file_test(dir, G_FILE_TEST_ISDIR)){
		/*get a list of all tiff files*/
		no_files = scandir(dir, &filelist, 0,alphasort );
		
		for (fcount = no_files - 1; fcount > 0; --fcount){
			fname = g_strdup_printf("%s%s", dir, filelist[fcount]->d_name);
			if ((g_file_test(fname, G_FILE_TEST_ISFILE))
				&& (strcmp(gnome_mime_type_or_default_of_file(fname, "UNKNOWN"),"image/tiff") == 0)
				&& (fnmatch ("*.thumb", fname, 0) != 0)){
				struct p_pages *new_ptr = g_malloc(sizeof(struct p_pages));
				new_ptr->page = g_strdup(fname);
				new_ptr->next_ptr = first_p_page_ptr;
				first_p_page_ptr = new_ptr;
			}
			g_free(fname);
		}
		current_p_page_ptr = first_p_page_ptr;
		no_pages = 0;
		while (current_p_page_ptr){
			++no_pages;
			current_p_page_ptr->no = g_malloc(sizeof(gint));
			*current_p_page_ptr->no = no_pages;
			current_p_page_ptr = current_p_page_ptr->next_ptr;
		}
		if((all) && (no_pages > 0)){
			/*auto print - no need to do the gui print dialog
			Free the list as we get a string of file names*/
			struct p_pages *current_ptr = first_p_page_ptr;
			cmd = NULL;
			while(current_ptr){
				struct p_pages *next = current_ptr->next_ptr;
				if(cmd){
					tmp = g_strdup_printf("%s %s", cmd, current_ptr->page);
					g_free(cmd);
					cmd = g_strdup(tmp);
					g_free(tmp);
				}else{
					cmd = g_strdup(current_ptr->page);
				}
				g_free(current_ptr->page);
				g_free(current_ptr->no);
				g_free(current_ptr);
				current_ptr = next;
			}
			if (Print.convert){
				/*make a command with fax2ps and use the shell to print*/
				tmp = g_strdup_printf("fax2ps -S -x%d -y%d -W%.2f -H%.2f %s | %s",Print.x_res,Print.y_res,Print.page_width,
																Print.page_height,cmd,Print.command);
				if (gnome_execute_shell(home,tmp) == -1){
					gnome_error_dialog("Error Printing\n\nCheck your Print command");
				}
			}else{
				/*make a command to cat files to the printer and use the shell to print*/
				tmp = g_strdup_printf("cat %s | %s",cmd,Print.command);
				if (gnome_execute_shell(home,tmp) == -1){
					gnome_error_dialog("Error Printing\n\nCheck your print command");
				}
			}
			g_free(cmd);
			g_free(tmp);
			print_active = FALSE;
		}else if(no_pages > 0){
			/*display the print dialog for more options*/
			create_dialog_print (no_pages, first_p_page_ptr, dir);
			print_active = FALSE;		
		}else{
			static_status_msg(_("No fax pages"));
			print_active = FALSE;
		}
	}

	return;
}

void create_dialog_print (gint files, struct p_pages *filelist, gchar *dir)
{
	print_d *PrintDialog = NULL; /*passed around for all data*/
	
	PrintDialog = g_malloc(sizeof(print_d));
	PrintDialog->filelist = filelist;
	PrintDialog->files = files;
	PrintDialog->dir = g_strdup(dir);
	PrintDialog->fs = NULL;
	PrintDialog->dialog_print = gnome_dialog_new (_("Print"), NULL);
	GTK_WINDOW (PrintDialog->dialog_print)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_transient_for(GTK_WINDOW (PrintDialog->dialog_print),GTK_WINDOW (gefax));
	gtk_window_set_default_size (GTK_WINDOW (PrintDialog->dialog_print), 269, 258);
	gtk_window_set_policy (GTK_WINDOW (PrintDialog->dialog_print), TRUE, TRUE, FALSE);

	PrintDialog->dialog_vbox1 = GNOME_DIALOG (PrintDialog->dialog_print)->vbox;
	gtk_widget_show (PrintDialog->dialog_vbox1);

	PrintDialog->dialog_hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (PrintDialog->dialog_hbox1);
	gtk_box_pack_start (GTK_BOX (PrintDialog->dialog_vbox1), PrintDialog->dialog_hbox1, TRUE, TRUE, 0);

	PrintDialog->lbl_print_to = gtk_label_new (_("Print to:"));
	gtk_widget_show (PrintDialog->lbl_print_to);
	gtk_box_pack_start (GTK_BOX (PrintDialog->dialog_hbox1), PrintDialog->lbl_print_to, FALSE, FALSE, 0);
	gtk_misc_set_padding (GTK_MISC (PrintDialog->lbl_print_to), 8, 0);

	PrintDialog->location_group = NULL;
	PrintDialog->rad_printer = gtk_radio_button_new_with_label (PrintDialog->location_group, _("Printer"));
  
	PrintDialog->location_group = gtk_radio_button_group (GTK_RADIO_BUTTON (PrintDialog->rad_printer));
	gtk_widget_show (PrintDialog->rad_printer);
	gtk_box_pack_start (GTK_BOX (PrintDialog->dialog_hbox1), PrintDialog->rad_printer, FALSE, FALSE, 0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (PrintDialog->rad_printer), TRUE);

	PrintDialog->rad_file = gtk_radio_button_new_with_label (PrintDialog->location_group, _("File"));
	PrintDialog->location_group = gtk_radio_button_group (GTK_RADIO_BUTTON (PrintDialog->rad_file));
	gtk_widget_show (PrintDialog->rad_file);
	gtk_box_pack_start (GTK_BOX (PrintDialog->dialog_hbox1), PrintDialog->rad_file, FALSE, FALSE, 0);

	PrintDialog->hseperator1 = gtk_hseparator_new ();
	gtk_widget_show (PrintDialog->hseperator1);
	gtk_box_pack_start (GTK_BOX (PrintDialog->dialog_vbox1), PrintDialog->hseperator1, TRUE, TRUE, 0);

	PrintDialog->lbl_range = gtk_label_new (_("Page Ranges:"));
	gtk_widget_show (PrintDialog->lbl_range);
	gtk_box_pack_start (GTK_BOX (PrintDialog->dialog_vbox1), PrintDialog->lbl_range, FALSE, FALSE, 0);
  
	PrintDialog->range_group = NULL;
	PrintDialog->rad_all = gtk_radio_button_new_with_label (PrintDialog->range_group, _("All"));
	PrintDialog->range_group = gtk_radio_button_group (GTK_RADIO_BUTTON (PrintDialog->rad_all));
	gtk_widget_show (PrintDialog->rad_all);
	gtk_box_pack_start (GTK_BOX (PrintDialog->dialog_vbox1), PrintDialog->rad_all, FALSE, FALSE, 0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (PrintDialog->rad_all), TRUE);

	PrintDialog->hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (PrintDialog->hbox2);
	gtk_box_pack_start (GTK_BOX (PrintDialog->dialog_vbox1), PrintDialog->hbox2, TRUE, TRUE, 0);

	PrintDialog->rad_from = gtk_radio_button_new_with_label (PrintDialog->range_group, _("From:"));
	PrintDialog->range_group = gtk_radio_button_group (GTK_RADIO_BUTTON (PrintDialog->rad_from));
	gtk_widget_show (PrintDialog->rad_from);
	gtk_box_pack_start (GTK_BOX (PrintDialog->hbox2), PrintDialog->rad_from, FALSE, FALSE, 0);

	PrintDialog->spin_first_adj = gtk_adjustment_new (1, 1, files, 1, 10, 10);
	PrintDialog->spin_first = gtk_spin_button_new (GTK_ADJUSTMENT (PrintDialog->spin_first_adj), 1, 0);
	gtk_widget_show (PrintDialog->spin_first);
	gtk_box_pack_start (GTK_BOX (PrintDialog->hbox2), PrintDialog->spin_first, TRUE, TRUE, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (PrintDialog->spin_first), TRUE);
	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (PrintDialog->spin_first), GTK_UPDATE_IF_VALID);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (PrintDialog->spin_first), TRUE);

	PrintDialog->lbl_to = gtk_label_new (_("to"));
	gtk_widget_show (PrintDialog->lbl_to);
	gtk_box_pack_start (GTK_BOX (PrintDialog->hbox2), PrintDialog->lbl_to, FALSE, FALSE, 0);
	gtk_misc_set_padding (GTK_MISC (PrintDialog->lbl_to), 4, 0);

	PrintDialog->spin_last_adj = gtk_adjustment_new (1, 1, files, 1, 10, 10);
	PrintDialog->spin_last = gtk_spin_button_new (GTK_ADJUSTMENT (PrintDialog->spin_last_adj), 1, 0);
	gtk_widget_show (PrintDialog->spin_last);
	gtk_box_pack_start (GTK_BOX (PrintDialog->hbox2), PrintDialog->spin_last, TRUE, TRUE, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (PrintDialog->spin_last), TRUE);
	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (PrintDialog->spin_last), GTK_UPDATE_IF_VALID);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (PrintDialog->spin_last), TRUE);

	PrintDialog->hseparator2 = gtk_hseparator_new ();
	gtk_widget_show (PrintDialog->hseparator2);
	gtk_box_pack_start (GTK_BOX (PrintDialog->dialog_vbox1), PrintDialog->hseparator2, TRUE, TRUE, 0);

	PrintDialog->hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (PrintDialog->hbox3);
	gtk_box_pack_start (GTK_BOX (PrintDialog->dialog_vbox1), PrintDialog->hbox3, TRUE, TRUE, 0);

	PrintDialog->lbl_copies = gtk_label_new (_("Copies:"));
	gtk_widget_show (PrintDialog->lbl_copies);
	gtk_box_pack_start (GTK_BOX (PrintDialog->hbox3), PrintDialog->lbl_copies, TRUE, TRUE, 0);
	gtk_label_set_justify (GTK_LABEL (PrintDialog->lbl_copies), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (PrintDialog->lbl_copies), 0.96, 0.5);

	PrintDialog->spin_copies_adj = gtk_adjustment_new (1, 1, 10, 1, 10, 10);
	PrintDialog->spin_copies = gtk_spin_button_new (GTK_ADJUSTMENT (PrintDialog->spin_copies_adj), 1, 0);
	gtk_widget_show (PrintDialog->spin_copies);
	gtk_box_pack_start (GTK_BOX (PrintDialog->hbox3), PrintDialog->spin_copies, FALSE, TRUE, 0);
	gtk_widget_set_usize (PrintDialog->spin_copies, 45, -2);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (PrintDialog->spin_copies), TRUE);
	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (PrintDialog->spin_copies), GTK_UPDATE_IF_VALID);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (PrintDialog->spin_copies), TRUE);

	PrintDialog->dialog_action_area = GNOME_DIALOG (PrintDialog->dialog_print)->action_area;
	gtk_widget_show (PrintDialog->dialog_action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (PrintDialog->dialog_action_area), GTK_BUTTONBOX_SPREAD);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (PrintDialog->dialog_action_area), 8);

	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (PrintDialog->dialog_print),
                                          _("Print"), GNOME_STOCK_PIXMAP_PRINT);
	PrintDialog->btn_print = g_list_last (GNOME_DIALOG (PrintDialog->dialog_print)->buttons)->data;
	gtk_widget_show (PrintDialog->btn_print);
	GTK_WIDGET_SET_FLAGS (PrintDialog->btn_print, GTK_CAN_DEFAULT);

	gnome_dialog_append_button (GNOME_DIALOG (PrintDialog->dialog_print), GNOME_STOCK_BUTTON_CANCEL);
	PrintDialog->btn_cancel = g_list_last (GNOME_DIALOG (PrintDialog->dialog_print)->buttons)->data;
	gtk_widget_show (PrintDialog->btn_cancel);
	GTK_WIDGET_SET_FLAGS (PrintDialog->btn_cancel, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (PrintDialog->rad_all), "toggled",
					GTK_SIGNAL_FUNC (on_rad_all_toggled), (print_d *)PrintDialog);
	gtk_signal_connect (GTK_OBJECT (PrintDialog->rad_file), "toggled",
					GTK_SIGNAL_FUNC (on_rad_file_toggled), (print_d *)PrintDialog);
	gtk_signal_connect (GTK_OBJECT (PrintDialog->spin_first), "changed",
					GTK_SIGNAL_FUNC (on_spin_first_changed), (print_d *)PrintDialog);
	gtk_signal_connect (GTK_OBJECT (PrintDialog->spin_last), "changed",
					GTK_SIGNAL_FUNC (on_spin_last_changed), (print_d *)PrintDialog);
	gtk_signal_connect (GTK_OBJECT (PrintDialog->spin_copies), "changed",
					GTK_SIGNAL_FUNC (on_spin_copies_changed), (print_d *)PrintDialog);
	gtk_signal_connect (GTK_OBJECT (PrintDialog->btn_cancel), "clicked",
					GTK_SIGNAL_FUNC (on_btn_cancel_clicked), (print_d *)PrintDialog);
	PrintDialog->destroy_signal = gtk_signal_connect(GTK_OBJECT (PrintDialog->dialog_print), "destroy",
					GTK_SIGNAL_FUNC (on_btn_cancel_clicked), (print_d *)PrintDialog);
	gtk_signal_connect (GTK_OBJECT (PrintDialog->btn_print), "clicked",
					GTK_SIGNAL_FUNC (on_btn_print_clicked), (print_d *)PrintDialog);
                      
	gtk_widget_set_sensitive(PrintDialog->spin_first ,FALSE);
	gtk_widget_set_sensitive(PrintDialog->spin_last ,FALSE);
	gtk_widget_set_sensitive(PrintDialog->lbl_to ,FALSE);
	if (files == 1){
		gtk_widget_set_sensitive(PrintDialog->rad_from ,FALSE);
	}
	PrintDialog->print_all = TRUE;
	PrintDialog->print_file = FALSE;
	PrintDialog->first = 1;
	PrintDialog->last = 1;
	PrintDialog->copies = 1;
	PrintDialog->pfile = NULL;
	gtk_widget_show (PrintDialog->dialog_print);

  return;
}

void on_rad_all_toggled(GtkToggleButton *tb, print_d *PrintDialog)
{
	if (GTK_TOGGLE_BUTTON(tb)->active){
		gtk_widget_set_sensitive(PrintDialog->spin_first ,FALSE);
		gtk_widget_set_sensitive(PrintDialog->spin_last ,FALSE);
		gtk_widget_set_sensitive(PrintDialog->lbl_to ,FALSE);
		PrintDialog->print_all = TRUE;
	}else{
		gtk_widget_set_sensitive(PrintDialog->spin_first ,TRUE);
		gtk_widget_set_sensitive(PrintDialog->spin_last ,TRUE);
		gtk_widget_set_sensitive(PrintDialog->lbl_to ,TRUE);
		PrintDialog->print_all = FALSE;
	}
	return;
}

void on_rad_file_toggled(GtkToggleButton *tb, print_d *PrintDialog)
{
	if (PrintDialog->pfile){
		g_free(PrintDialog->pfile);
		PrintDialog->pfile = NULL;
	}
	
	if (GTK_TOGGLE_BUTTON(tb)->active){
		gchar *file_name;
		PrintDialog->fs = gtk_file_selection_new("Print To File...");
		gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(PrintDialog->fs)->ok_button), "clicked",
                      GTK_SIGNAL_FUNC (on_fs_ok_clicked),
                      (print_d *)PrintDialog);
          gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(PrintDialog->fs)->cancel_button), "clicked",
                      GTK_SIGNAL_FUNC (on_fs_cancel_clicked),
                      (print_d *)PrintDialog);
          file_name = g_strdup_printf("%s/output.ps", home);
          gtk_file_selection_set_filename(GTK_FILE_SELECTION(PrintDialog->fs), file_name);
          g_free(file_name);
          /*Make the window show above the print dialog*/
          gtk_window_set_transient_for (GTK_WINDOW (PrintDialog->fs), GTK_WINDOW (PrintDialog->dialog_print));
          gtk_widget_show(PrintDialog->fs);
          PrintDialog->print_file = TRUE;
          gtk_widget_set_sensitive(PrintDialog->spin_copies ,FALSE);  
	}else{
		PrintDialog->print_file = FALSE;
		gtk_widget_set_sensitive(PrintDialog->spin_copies ,TRUE);
	}
}
void on_fs_ok_clicked(GtkButton *btn,  print_d *PrintDialog)
{
	GtkWidget *fwarn;
	
	PrintDialog->pfile = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(PrintDialog->fs)));
	if (g_file_test(PrintDialog->pfile, G_FILE_TEST_ISFILE)){
		fwarn = gnome_message_box_new(_("File Already Exists\n\nOverwrite File?"),
							GNOME_MESSAGE_BOX_WARNING,
							GNOME_STOCK_BUTTON_OK,
							GNOME_STOCK_BUTTON_CANCEL,
							NULL);
		gtk_window_set_position (GTK_WINDOW (fwarn), GTK_WIN_POS_MOUSE);
		if ((gnome_dialog_run_and_close(GNOME_DIALOG(fwarn))) == 0){
			gtk_widget_destroy(PrintDialog->fs);
			PrintDialog->fs = NULL;
		}else{
			g_free(PrintDialog->pfile);
			PrintDialog->pfile = NULL;
		}
	}else{
		gtk_widget_destroy(PrintDialog->fs);
		PrintDialog->fs = NULL;
	}

}
void on_fs_cancel_clicked(GtkButton *btn,  print_d *PrintDialog)
{
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (PrintDialog->rad_printer), TRUE);
	PrintDialog->print_file = FALSE;
	gtk_widget_destroy(PrintDialog->fs);
	PrintDialog->fs = NULL;
}
void on_spin_first_changed(GtkSpinButton *sfirst,  print_d *PrintDialog)
{
	
	PrintDialog->first = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(PrintDialog->spin_first));
	if (PrintDialog->first > PrintDialog->last){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(PrintDialog->spin_last),PrintDialog->first);
	}
#	ifdef DEBUG
		printf("DEBUG: First=%d Last=%d\n",PrintDialog->first,PrintDialog->last);
#	endif /*DEBUG*/
}

void on_spin_last_changed(GtkSpinButton *slast,  print_d *PrintDialog)
{
	
	PrintDialog->last = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(PrintDialog->spin_last));
	if (PrintDialog->first > PrintDialog->last){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(PrintDialog->spin_first),PrintDialog->last);
	}
#	ifdef DEBUG
		printf("DEBUG: First=%d Last=%d\n",PrintDialog->first,PrintDialog->last);
#	endif DEBUG
}

void on_spin_copies_changed(GtkSpinButton *scopies,  print_d *PrintDialog)
{
	PrintDialog->copies = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(PrintDialog->spin_copies));
}

void on_btn_cancel_clicked(GtkButton *btn,  print_d *PrintDialog)
{

	/*if the user hit the cancel or print button, remove the 
	"destroy" handler to prevent executing this twice*/
	gtk_signal_disconnect (GTK_OBJECT (PrintDialog->dialog_print), PrintDialog->destroy_signal);
#	ifdef DEBUG
		puts("DEBUG:destroying print dialog");
#	endif DEBUG

	if (PrintDialog->fs) gtk_widget_destroy(PrintDialog->fs);
	if (PrintDialog->dialog_print) gtk_widget_destroy(PrintDialog->dialog_print);
	if (PrintDialog->dir) g_free(PrintDialog->dir);
	if (PrintDialog->pfile) g_free(PrintDialog->pfile);
	if (PrintDialog->filelist){	/*free the file list*/
		while (PrintDialog->filelist){
			struct p_pages *next = PrintDialog->filelist->next_ptr;
			g_free(PrintDialog->filelist->no);
			g_free(PrintDialog->filelist->page);
			g_free(PrintDialog->filelist);
			PrintDialog->filelist = next;
		}
	}
	g_free(PrintDialog);
	PrintDialog = NULL;
	print_active = FALSE;
	return;
}

void on_btn_print_clicked(GtkButton *btn,  print_d *PrintDialog)
{
	gint copy;
	gint fcount;
	gchar *cmd = NULL;
	gchar *tmp = NULL;
	gint pages = 0;
	
	for (copy = 0; copy < PrintDialog->copies; copy++){
		if (PrintDialog->print_all){
			struct p_pages *current_ptr = PrintDialog->filelist;
			cmd = NULL;
			while(current_ptr){
				if(cmd){
					tmp = g_strdup_printf("%s %s", cmd, current_ptr->page);
					g_free(cmd);
					cmd = g_strdup(tmp);
					g_free(tmp);
				}else{
					cmd = g_strdup(current_ptr->page);
				}
				++pages;
				current_ptr = current_ptr->next_ptr;
			}
		}else{
			cmd = NULL;
			for (fcount = PrintDialog->first; fcount < (PrintDialog->last + 1); fcount++){
				struct p_pages *current_ptr = PrintDialog->filelist;
				while(current_ptr){
					if(*current_ptr->no == fcount){
						if(cmd){
							tmp = g_strdup_printf("%s %s", cmd, current_ptr->page);
							g_free(cmd);
							cmd = g_strdup(tmp);
							g_free(tmp);
						}else{
							cmd = g_strdup(current_ptr->page);
						}
						++pages;
					}
					current_ptr = current_ptr->next_ptr;
				}
			}
		}
		/*show some info on the status bar*/
		tmp = g_strdup_printf("Printed %d Pages", pages);
		static_status_msg(tmp);
		g_free(tmp);
		pages = 0;
		print_gui_file(cmd, (print_d *)PrintDialog);
		/*no point printing to a file more than once*/
		if(PrintDialog->print_file){
			break;
		}
	}
	on_btn_cancel_clicked(NULL,(print_d *)PrintDialog);
}

void print_gui_file(gchar *cmd, print_d *PrintDialog)
{
	gchar *tmp;
	
	while (1){
		if ((Print.convert) && (cmd) && (!PrintDialog->print_file)){
			tmp = g_strdup_printf("fax2ps -S -x%d -y%d -W%.2f -H%.2f %s | %s",Print.x_res,Print.y_res,Print.page_width,
														Print.page_height,cmd,Print.command);
			
			if (gnome_execute_shell(home,tmp) == -1){
				gnome_error_dialog(_("Error Printing\n\nCheck your print command"));
			}
			g_free(tmp);
			g_free(cmd);
			break;
		}else if ((cmd) && (!PrintDialog->print_file)){
			tmp = g_strdup_printf("cat %s | %s",cmd,Print.command);
						
			if (gnome_execute_shell(home,tmp) == -1){
				gnome_error_dialog(_("Error Printing\n\nCheck your print command"));
			}
			g_free(tmp);
			g_free(cmd);
			break;
		}else if ((cmd) && (PrintDialog->pfile)){
			tmp = g_strdup_printf("fax2ps -S -x%d -y%d -W%.2f -H%.2f %s > %s",Print.x_res,Print.y_res,Print.page_width,
														Print.page_height,cmd,PrintDialog->pfile);
			g_free(PrintDialog->pfile);
			PrintDialog->pfile = NULL;
			if (gnome_execute_shell(home,tmp) == -1){
				gnome_error_dialog(_("Error Printing to File\n\nCheck fax2ps is in your Path"));
			}
			g_free(tmp);
			g_free(cmd);
			break;
		}else{
			gnome_error_dialog(_("Error Printing\n\nAn Unknown Error Occured"));
			break;
		}
	}
}
/*end*/





