/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <gnome.h>

#include "gefax.h"
#include "setup.h"
#include "hist.h"

extern void load_dir(void);
extern void load_clist_fax(void);

typedef struct _S_G_Tab{
	GtkWidget *chk_auto_print;
	GtkWidget *entry_fax_dir;
	GtkWidget *entry_bin_dir;
	GtkWidget *chk_view_all;
	GtkWidget *chk_sort;
	GtkWidget *chk_number_warn;
} S_G_Tab;
S_G_Tab SetupGeneralTab;

typedef struct _S_M_Tab{
	GtkWidget *cbo_entry_dev;
	GtkWidget *cbo_entry_lock;
	GtkWidget *cbo_entry_bitrate;
	GtkObject *adj_vol;
	GtkWidget *rad_tone;
	GtkWidget *cbo_entry_init;
	GtkWidget *cbo_entry_init_after;
	GtkWidget *cbo_entry_class;
	GtkWidget *cbo_entry_xon;
	GtkWidget *cbo_entry_scan;
} S_M_Tab;
S_M_Tab SetupModemTab;

typedef struct _S_F_Tab{
	GtkWidget *entry_header;
	GtkWidget *entry_id_num;
	GtkWidget *cbo_entry_cover;
	GtkWidget *chk_cover;
	GtkWidget *lbl_cover;
	GtkWidget *spin_line_len;
	GtkWidget *spin_tab_len;
	GtkWidget *chk_wrap_text;
	GtkWidget *vbox_line_len;
	GtkWidget *vbox_tab_len;
} S_F_Tab;
S_F_Tab SetupFaxTab;

typedef struct _S_P_Tab{
	GtkWidget *rad_ps;
	GtkWidget *spin_x_res;
	GtkWidget *spin_y_res;
	GtkWidget *spin_page_width;
	GtkWidget *spin_page_height;
	GtkWidget *entry_printer;
} S_P_Tab;
S_P_Tab SetupPrintTab;

typedef struct _S_A_Tab{
	GtkWidget *chk_debug_win;
	GtkWidget *chk_auto_save_cardfile;
} S_A_Tab;
S_A_Tab SetupAdvancedTab;

static GtkWidget *dialog_prop = NULL;

static GnomeHelpMenuEntry help_ref_G =
{
	"gefax",
	"setup.html#general"
};
static GnomeHelpMenuEntry help_ref_M =
{
	"gefax",
	"setup.html#modem"
};
static GnomeHelpMenuEntry help_ref_F =
{
	"gefax",
	"setup.html#fax"
};
static GnomeHelpMenuEntry help_ref_P =
{
	"gefax",
	"setup.html#Print"
};

static void on_rad_ps_toggled(GtkToggleButton *togglebutton, gpointer user_data);
static void on_chk_cover_toggled(GtkButton *button, gpointer FileCover);
static void on_chk_wrap_text_toggled(GtkButton *button, gpointer data);
static void destroy_dialog_prop (GnomePropertyBox *gnomepropertybox, gpointer data);
static void on_btn_prop_ok_clicked (GnomePropertyBox *gnomepropertybox, gint arg1, gpointer data);
static void on_btn_prop_help_clicked (GnomePropertyBox *gnomepropertybox, gint arg1, gpointer data);
static void property_box_changed(GtkWidget w, gpointer data);
/*This is all pretty clunky I think - any idears? Return must be freed*/
static gchar *fix_up_dir(gchar *dir);
void property_box_changed(GtkWidget w, gpointer data)
{
  	gnome_property_box_changed((GnomePropertyBox *)dialog_prop);
}

gchar *fix_up_dir(gchar *dir)
{
	gchar *n_dir;
	
	if (dir[strlen(dir)] == '\0'){	/*User typed dir in*/
		if (dir[strlen(dir) - 1] != '/'){
			n_dir = g_strdup_printf("%s/",dir);
		}else{
			n_dir = g_strdup(dir);
		}
	}else{
		if (dir[strlen(dir)] != '/'){
			n_dir = g_strdup_printf("%s/",dir);
		}else{
			n_dir = g_strdup(dir);
		}
	}
	return (n_dir);
}
                      
void on_btn_prop_ok_clicked (GnomePropertyBox *gnomepropertybox, gint arg1, gpointer data)
{
	gchar *tmp;
	gchar *tmp2;
	gint count;

	/*only save if once as we only wish to save all.
	 Probably split this up into at lease a function for each page*/
	if(arg1 != -1) return;

#	ifdef DEBUG
		printf("\nDEBUG:Saving Prefs\n");
#	endif /*DEBUG*/
	
  while (1){	/*Get out on an error*/  

/*
 *General prefs
*/
	/*auto prinf fax after received*/
	if (GTK_TOGGLE_BUTTON(SetupGeneralTab.chk_auto_print)->active){
		Print.auto_print = TRUE;
	}else{
		Print.auto_print = FALSE;
	}
	
	/*load selected files in order*/
	if (!load_ordered == GTK_TOGGLE_BUTTON(SetupGeneralTab.chk_sort)->active){
		load_ordered = GTK_TOGGLE_BUTTON(SetupGeneralTab.chk_sort)->active;
		while (gtk_events_pending())
		gtk_main_iteration();
		load_clist_fax();
	}
	
	/*Warn before removing fax numbers?*/
	if (GTK_TOGGLE_BUTTON(SetupGeneralTab.chk_number_warn)->active){
		remove_number_warn = TRUE;
	}else{
		remove_number_warn = FALSE;
	}
	
	/*Where we put received files*/
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupGeneralTab.entry_fax_dir)));
	
	if (strcmp (tmp, recvd_dir) != 0 ) {

		if ((g_file_test(tmp, G_FILE_TEST_ISDIR)) && (access(tmp , W_OK) == 0)){
			history_save_config ();
			tmp2 = fix_up_dir(tmp);
			g_free(recvd_dir);
			recvd_dir = g_strdup(tmp2);
			
			g_free(tmp2);
			history_load_config ();
		}else{
			gnome_error_dialog(_("Received Files Directory Doesn't Exist or,\n"
				"You don't have permission to write files"));
			g_free(tmp);
			break;
		}
	}
	g_free(tmp);

	/*rubish bin*/
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupGeneralTab.entry_bin_dir)));
	
#	ifdef DEBUG
		printf("	DEBUG:SetupGeneralTab.entry_bin_dir=%s\n",tmp);
#	endif /*DEBUG*/
	
	if ((g_file_test(tmp, G_FILE_TEST_ISDIR)) && (access(tmp , W_OK) == 0)){
		tmp2 = fix_up_dir(tmp);
		g_free(bin_dir);
		bin_dir = g_strdup(tmp2);
			
#		ifdef DEBUG
			printf("	DEBUG:Fixed bin dir=%s\n",bin_dir);
#		endif /*DEBUG*/
		
		g_free(tmp2);
	}else{
		gnome_error_dialog("Bin Directory Doesn't Exist or\n"
							"You don't have permission to write files");
		g_free(tmp);
		break;
	}
	g_free(tmp);
	

/*
 *Modem prefs
*/	
	/*device*/
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupModemTab.cbo_entry_dev)));
	if (access(tmp , F_OK) == -1){
		gnome_error_dialog(_("Modem Device Doesn't Exist\n"));
		break;
	}
	tmp2 = g_strdup_printf("-d%s", tmp);
	g_free(EfaxArgs.dev);
	EfaxArgs.dev = g_strdup(tmp2);
	g_free(tmp2);
		
	/*lock type*/
	tmp2 = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupModemTab.cbo_entry_lock)));
	g_free(EfaxArgs.locktype);
	EfaxArgs.locktype = g_strdup(tmp2);
	g_free(tmp2);
	
	/*lock*/
	g_free(tmp);	/*need device without -d dev */
	
#	ifdef DEBUG
		printf("	DEBUG:Device=%s\n", EfaxArgs.dev);
#	endif /*DEBUG*/

	tmp = (char *) g_malloc (strlen(EfaxArgs.dev) - 6);
	for (count = 0; count < (strlen(EfaxArgs.dev) - 6); ++count){
		tmp[count] = EfaxArgs.dev[(count + 7)];
	}
	
#	ifdef DEBUG
		printf("	DEBUG:Device Name=%s\n", tmp);
#	endif /*DEBUG*/

	switch(EfaxArgs.locktype[0]){
		case 'm': /*modern*/
			tmp2 = g_strdup_printf("-x/var/lock/LCK..%s", tmp);
			break;
		case 'o': /*older uucp*/
			tmp2 = g_strdup_printf("-x/usr/spool/uucp/LCK..%s", tmp);
			break;
		case 'b': /*binary*/
			tmp2 = g_strdup_printf("-x#/usr/spool/uucp/LCK..%s", tmp);
			break;
		default : /*none*/
			tmp2 = g_strdup("none");
			break;
	}
	g_free(EfaxArgs.lock);
	EfaxArgs.lock = g_strdup(tmp2);
	
#	ifdef DEBUG
		printf("	DEBUG:Lock=%s\n", EfaxArgs.lock);
#	endif /*DEBUG*/
	g_free(tmp2);
	g_free(tmp);
	
	/*modem class*/
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupModemTab.cbo_entry_class)));
	if (strcmp(tmp, "CLASS 2") == 0){
		EfaxArgs.mclass[2] = '2';
	}else if (strcmp(tmp, "CLASS 1") == 0){
		EfaxArgs.mclass[2] = '1';
	}else{
		EfaxArgs.mclass[2] = '0';
	}
	g_free(tmp);
	
#	ifdef DEBUG
		printf("	DEBUG:Modem Class=%s\n", EfaxArgs.mclass);
#	endif /*DEBUG*/

	/*bitrate*/
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupModemTab.cbo_entry_bitrate)));
	if (strcmp(tmp, "14400 (v:17)") == 0){
		EfaxArgs.caps[4] = '5';
	}else if (strcmp(tmp, "12000 (v:17)") == 0){
		EfaxArgs.caps[4] = '4';
	}else if (strcmp(tmp, "9600 bps") == 0){
		EfaxArgs.caps[4] = '3';
	}else if (strcmp(tmp, "7200 bps") == 0){
		EfaxArgs.caps[4] = '2';
	}else if (strcmp(tmp, "4800 bps") == 0){
		EfaxArgs.caps[4] = '1';
	}else{
		EfaxArgs.caps[4] = '0';
	}
	g_free(tmp);
	
#	ifdef DEBUG
		printf("	DEBUG:Modem Caps=%s\n",EfaxArgs.caps);
#	endif /*DEBUG*/
	
	/*modem volume*/
	g_free(EfaxArgs.vol);
	switch ((gint)GTK_ADJUSTMENT(SetupModemTab.adj_vol)->value){
		case 0:
			EfaxArgs.vol = g_strdup("-iM0L0");
			break;
		case 1:
			EfaxArgs.vol = g_strdup("-iM2L0");
			break;
		default:
			EfaxArgs.vol = g_strdup("-iM2L3");
			break;
	}
	
#	ifdef DEBUG
		printf("	DEBUG:Modem Vol=%s\n",EfaxArgs.vol);
#	endif /*DEBUG*/
	
	/*Dial prefix*/
	if (GTK_TOGGLE_BUTTON(SetupModemTab.rad_tone)->active){
		EfaxArgs.prefix[2] = 'T';
	}else{
		EfaxArgs.prefix[2] = 'P';
	}
	
#	ifdef DEBUG
		printf("	DEBUG:Dial Prefix=%s\n",EfaxArgs.prefix);
#	endif /*DEBUG*/
	
	/*modem init string*/
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupModemTab.cbo_entry_init)));
	if (strcmp(EfaxArgs.init , tmp) != 0){	/*has changed*/
		g_free(EfaxArgs.init);
		EfaxArgs.init = g_strdup(tmp);
	}
	g_free(tmp);
	
#	ifdef DEBUG
		printf("	DEBUG:Modem Init=%s\n",EfaxArgs.init);
#	endif /*DEBUG*/

	/*modem reset string*/
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupModemTab.cbo_entry_init_after)));
	if (strcmp(EfaxArgs.reset , tmp) != 0){	/*has changed*/
		g_free(EfaxArgs.reset);
		EfaxArgs.reset = g_strdup(tmp);
	}
	g_free(tmp);
	
#	ifdef DEBUG
		printf("	DEBUG:Modem Reset=%s\n",EfaxArgs.reset);
#	endif /*DEBUG*/

	/*force_xonxoff?*/
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupModemTab.cbo_entry_xon)));
	g_free(EfaxArgs.xonxoff);
	if (strcmp(tmp, "none") == 0){
		EfaxArgs.xonxoff = g_strdup("none");
	}else if (strcmp(tmp, "USR (Courier, Sportster)") == 0){
		EfaxArgs.xonxoff = g_strdup("-j&H2");
	}else if (strcmp(tmp, "QuickCom (Spirit II)") == 0){
		EfaxArgs.xonxoff = g_strdup("-j*F1");
	}else if (strcmp(tmp, "Motarola (Power Modem, 3400 Pro, ...)") == 0){
		EfaxArgs.xonxoff = g_strdup("-j\\Q1");
	}else if (strcmp(tmp, "AT&T (Dataport, Paradyne)") == 0){
		EfaxArgs.xonxoff = g_strdup("-j\\Q4");
	}else{	/*Bit Reversal (Multi-Teck)*/
		EfaxArgs.xonxoff = g_strdup("-or");
	}
	g_free(tmp);
	
#	ifdef DEBUG
		printf("	DEBUG:Modem XonXoff=%s\n",EfaxArgs.xonxoff);
#	endif /*DEBUG*/

		
	/*cmddelay = */
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupModemTab.cbo_entry_scan)));
	if (tmp[0] == '1'){
		EfaxArgs.cmddelay = g_strdup("default");
	}else if (tmp[0] == '2'){
		EfaxArgs.cmddelay = g_strdup("-oz");
	}else if (tmp[0] == '3'){
		EfaxArgs.cmddelay = g_strdup("-ozz");
	}else if (tmp[0] == '4'){
		EfaxArgs.cmddelay = g_strdup("-ozzz");
	}else{
		EfaxArgs.cmddelay = g_strdup("-ozzzz");
	}
	g_free(tmp);
	
#	ifdef DEBUG
		printf("	DEBUG:Modem Command Delay=%s\n",EfaxArgs.cmddelay);
#	endif /*DEBUG*/
	
/*
 *Fax prefs
*/
	/*Page header*/
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupFaxTab.entry_header)));
	if((tmp[0] == '-') && (tmp[1] == 'h')){
		gnome_error_dialog("Please remove the -h from the beginning of\n"
						"Page Header\nNot required any longer");
		break;
	}else{
		tmp2 = g_strdup_printf("-h%s",tmp);
		g_free(EfaxArgs.header);
		EfaxArgs.header = g_strdup(tmp2);
		g_free(tmp2);
	}
	g_free(tmp);
	
#	ifdef DEBUG
		printf("	DEBUG:Header=%s\n",EfaxArgs.header);
#	endif /*DEBUG*/
	
	/*Fax station Ident*/
	tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupFaxTab.entry_id_num)));
	if((tmp[0] == '-') && (tmp[1] == 'l')){
		gnome_error_dialog("Please remove the -l from the beginning of\n"
						"Fax Station Ident Number\nNot required any longer");
		break;
	}else{
		tmp2 = g_strdup_printf("-l%s",tmp);
		g_free(EfaxArgs.ident);
		EfaxArgs.ident = g_strdup(tmp2);
		g_free(tmp2);
	}
	g_free(tmp);

#	ifdef DEBUG
		printf("	DEBUG:Fax Ident=%s\n",EfaxArgs.ident);
#	endif /*DEBUG*/
	
	/*need a cover page?*/
	if (GTK_TOGGLE_BUTTON(SetupFaxTab.chk_cover)->active){
		EfaxArgs.usecover = 1;
	}else{
		EfaxArgs.usecover = 0;
	}
	
	/*where is the cover page?*/
	if (EfaxArgs.usecover == 1){
		tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupFaxTab.cbo_entry_cover)));
		if (strcmp(EfaxArgs.cover, tmp) != 0){	/*has changed*/
			if(g_file_test(tmp, G_FILE_TEST_ISFILE)){ /*is it a file?*/
				if (strcmp(gnome_mime_type_or_default_of_file(tmp, "UNKNOWN"),
						"image/tiff") == 0){	/*is it a tiff file at least?*/
					g_free(EfaxArgs.cover);
					EfaxArgs.cover = g_strdup(tmp);
				}else{
					gnome_error_dialog("Cover page is not a tiff file");
					break;
				}
			}else{
				gnome_error_dialog("Cover page doesn't exist");
				break;
			}
			
#			ifdef DEBUG
				printf("	DEBUG:Cover File=%s\n",EfaxArgs.cover);
#			endif /*DEBUG*/
		}
		g_free(tmp);
	}
	TextWrapArgs.do_wrap = (GTK_TOGGLE_BUTTON(SetupFaxTab.chk_wrap_text)->active);
	TextWrapArgs.line_len = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(SetupFaxTab.spin_line_len));
	TextWrapArgs.tab_len = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(SetupFaxTab.spin_tab_len));
/*
 *Printing
*/
	g_free(Print.command);
	Print.command = g_strdup(gtk_entry_get_text(GTK_ENTRY(SetupPrintTab.entry_printer)));
	Print.convert = (GTK_TOGGLE_BUTTON(SetupPrintTab.rad_ps)->active);
	Print.page_width = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(SetupPrintTab.spin_page_width));
	Print.page_height = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(SetupPrintTab.spin_page_height));
	Print.x_res = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(SetupPrintTab.spin_x_res));
	Print.y_res = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(SetupPrintTab.spin_y_res));
#	ifdef DEBUG
		printf("DEBUG:Finished Saving Prefs\n\n");
#	endif /*DEBUG*/
	
/*
 *Advanced
*/
	debug_window = (GTK_TOGGLE_BUTTON(SetupAdvancedTab.chk_debug_win)->active);
	auto_save_cardfile = (GTK_TOGGLE_BUTTON(SetupAdvancedTab.chk_auto_save_cardfile)->active);
	/*veiw all files in main window?*/
	if(load_all != GTK_TOGGLE_BUTTON(SetupGeneralTab.chk_view_all)->active){
		load_all = GTK_TOGGLE_BUTTON(SetupGeneralTab.chk_view_all)->active;
		while (gtk_events_pending())
		gtk_main_iteration();
		load_dir();
	}
	break;
  }	/*End While 1*/

}


void on_rad_ps_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	if (GTK_TOGGLE_BUTTON(togglebutton)->active){
		gtk_widget_set_sensitive (GTK_WIDGET(user_data), TRUE);
		Print.convert = TRUE;
	}else{
		gtk_widget_set_sensitive (GTK_WIDGET(user_data), FALSE);
		Print.convert = FALSE;
	}
}
                                        
void on_chk_cover_toggled(GtkButton *button, gpointer FileCover)
{

	if (GTK_TOGGLE_BUTTON(button)->active){
		gtk_widget_set_sensitive (SetupFaxTab.lbl_cover, TRUE);
		gtk_widget_set_sensitive (FileCover, TRUE);
	}else{
		gtk_widget_set_sensitive (SetupFaxTab.lbl_cover, FALSE);
		gtk_widget_set_sensitive (FileCover, FALSE);
	}
}

void on_chk_wrap_text_toggled(GtkButton *button, gpointer data)
{
	if (GTK_TOGGLE_BUTTON(button)->active){
		gtk_widget_set_sensitive (SetupFaxTab.vbox_line_len, TRUE);
		gtk_widget_set_sensitive (SetupFaxTab.vbox_tab_len, TRUE);
	}else{
		gtk_widget_set_sensitive (SetupFaxTab.vbox_line_len, FALSE);
		gtk_widget_set_sensitive (SetupFaxTab.vbox_tab_len, FALSE);
	}
}

void destroy_dialog_prop (GnomePropertyBox *gnomepropertybox, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(dialog_prop));
	dialog_prop = NULL;
}

void on_btn_prop_help_clicked (GnomePropertyBox *gnomepropertybox, gint arg1, gpointer data)
{
	switch (arg1){
		case 0:
			gnome_help_display("ignored", &help_ref_G);
			break;
		case 1:
			gnome_help_display("ignored", &help_ref_M);
			break;
		case 2:
			gnome_help_display("ignored", &help_ref_F);
			break;
		case 3:
			gnome_help_display("ignored", &help_ref_P);
			break;
	}
}

void create_dialog_prop(void)
{
	GtkWidget *note_prop;
	GtkWidget *hbox3;
	GtkWidget *table1;
	GtkWidget *lbl_fax_dir;
	GtkWidget *lbl_bin_dir;
	GtkWidget *hbox5;
	GtkWidget *file_fax;
	GtkWidget *file_bin;
	GtkWidget *lbl_General;
	GtkWidget *table3;
	GtkWidget *label20;
	GtkWidget *label19;
	GtkWidget *label18;
	GtkWidget *label17;
	GtkWidget *label16;
	GtkWidget *label11;
	GtkWidget *label12;
	GtkWidget *label13;
	GtkWidget *label14;
	GtkWidget *label15;
	GtkWidget *cbo_dev;
	GList *cbo_dev_items = NULL;
	GtkWidget *cbo_class;
	GList *cbo_class_items = NULL;
	GtkWidget *cbo_bitrate;
	GList *cbo_bitrate_items = NULL;
	GtkWidget *hbox1;
	GtkWidget *vbox2;
	GSList *dt_group = NULL;
	GtkWidget *rad_pulse;
	GtkWidget *cbo_scan;
	GList *cbo_scan_items = NULL;
	GtkWidget *cbo_init;
	GList *cbo_init_items = NULL;
	GtkWidget *cbo_init_after;
	GList *cbo_init_after_items = NULL;
	GtkWidget *cbo_lock;
	GList *cbo_lock_items = NULL;
	GtkWidget *hscale_vol;
	GtkWidget *cbo_xon;
	GList *cbo_xon_items = NULL;
	GtkWidget *lbl_modem;
	GtkWidget *table7;
	GtkWidget *label26;
	GtkWidget *label28;
	GtkWidget *file_cover;
	GtkWidget *lbl_fax;
	GtkWidget *vbox7;
	GtkWidget *table18;
	GtkWidget *hseparator3;
	GtkWidget *label39;
	GtkWidget *label36;
	GtkWidget *hseparator5;
	GSList *lan_group = NULL;
	GtkWidget *rad_pcl;
	GtkWidget *table19;
	GtkWidget *label41;
	GtkWidget *label40;
	GtkWidget *label35;
	GtkWidget *label34;
	GtkWidget *label31;
	GtkWidget *label32;
	GtkObject *spin_x_res_adj;
	GtkObject *spin_page_width_adj;
	GtkObject *spin_page_height_adj;
	GtkObject *spin_y_res_adj;
	GtkWidget *vseparator1;
	GtkWidget *label29;
	
	GtkWidget *fra_wrap;
	GtkWidget *hbox_w;
	GtkWidget *lbl_line_length;
	GtkObject *spin_line_len_adj;
	GtkWidget *lbl_tab_expand;
	GtkObject *spin_tab_len_adj;
  	GtkWidget *label_adv;
  	GtkWidget *table_advanced;
  
	gchar *temp;
	gchar *temp2;
  
	if(dialog_prop){
		gdk_window_raise(dialog_prop->window);
		return;
	}

	dialog_prop =  gnome_property_box_new ();
	gtk_window_set_title (GTK_WINDOW (dialog_prop), _("gefax properties"));
	gtk_window_set_transient_for(GTK_WINDOW (dialog_prop),GTK_WINDOW (gefax));
	gtk_window_set_default_size (GTK_WINDOW (dialog_prop), 400, 350);
	gtk_window_set_policy (GTK_WINDOW (dialog_prop), TRUE, TRUE, FALSE);

	note_prop = GNOME_PROPERTY_BOX (dialog_prop)->notebook;
	gtk_widget_show (note_prop);
	gtk_notebook_set_tab_hborder (GTK_NOTEBOOK (note_prop), 10);
	gtk_notebook_popup_enable (GTK_NOTEBOOK (note_prop));
  
	hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox3);
	gtk_container_add (GTK_CONTAINER (note_prop), hbox3);
	
	table1 = gtk_table_new (10, 1, TRUE);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (hbox3), table1, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (table1), 10);
	gtk_table_set_col_spacings (GTK_TABLE (table1), 5);

	lbl_fax_dir = gtk_label_new (_("Received files directory"));
	gtk_widget_show (lbl_fax_dir);
	gtk_table_attach (GTK_TABLE (table1), lbl_fax_dir, 0, 1, 5, 6,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (lbl_fax_dir), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (lbl_fax_dir), 7.45058e-09, 0.5);
	lbl_bin_dir = gtk_label_new (_("Bin directory"));
	gtk_widget_show (lbl_bin_dir);
	gtk_table_attach (GTK_TABLE (table1), lbl_bin_dir, 0, 1, 7, 8,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (lbl_bin_dir), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (lbl_bin_dir), 7.45058e-09, 0.5);

	SetupGeneralTab.chk_view_all = gtk_check_button_new_with_label (_("View all files"));
	gtk_widget_show (SetupGeneralTab.chk_view_all);
	gtk_table_attach (GTK_TABLE (table1), SetupGeneralTab.chk_view_all, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	SetupGeneralTab.chk_sort = gtk_check_button_new_with_label (_("View selected files in order"));
	gtk_widget_show (SetupGeneralTab.chk_sort);
	gtk_table_attach (GTK_TABLE (table1), SetupGeneralTab.chk_sort, 0, 1, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	SetupGeneralTab.chk_auto_print = gtk_check_button_new_with_label (_("Automatically print received fax"));
	gtk_widget_show (SetupGeneralTab.chk_auto_print);
	gtk_table_attach (GTK_TABLE (table1), SetupGeneralTab.chk_auto_print, 0, 1, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	                
	SetupGeneralTab.chk_number_warn = gtk_check_button_new_with_label (_("Warn before removing fax numbers"));
	gtk_widget_show (SetupGeneralTab.chk_number_warn);
	gtk_table_attach (GTK_TABLE (table1), SetupGeneralTab.chk_number_warn, 0, 1, 3, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	                  
	hbox5 = gtk_hbox_new (FALSE, 10);
	gtk_widget_show (hbox5);
	gtk_table_attach (GTK_TABLE (table1), hbox5, 0, 1, 3, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	
	file_fax = gnome_file_entry_new ("faxdir", _("Select directory to save received faxes"));
	gtk_widget_show (file_fax);
	gtk_table_attach (GTK_TABLE (table1), file_fax, 0, 1, 6, 7,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	gnome_file_entry_set_directory (GNOME_FILE_ENTRY (file_fax), TRUE);
	SetupGeneralTab.entry_fax_dir = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_fax));
	gtk_widget_show (SetupGeneralTab.entry_fax_dir);
	file_bin = gnome_file_entry_new ("bindir", _("Select bin directory"));
	gtk_widget_show (file_bin);
	gtk_table_attach (GTK_TABLE (table1), file_bin, 0, 1, 8, 9,
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
	                 (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	gnome_file_entry_set_directory (GNOME_FILE_ENTRY (file_bin), TRUE);
	SetupGeneralTab.entry_bin_dir = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_bin));
	gtk_widget_show (SetupGeneralTab.entry_bin_dir);
	

	lbl_General = gtk_label_new (_("General"));
	gtk_widget_show (lbl_General);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (note_prop), gtk_notebook_get_nth_page (GTK_NOTEBOOK (note_prop), 0), lbl_General);
	table3 = gtk_table_new (10, 2, TRUE);
	gtk_widget_show (table3);
	gtk_container_add (GTK_CONTAINER (note_prop), table3);
	gtk_container_set_border_width (GTK_CONTAINER (table3), 10);
	gtk_table_set_col_spacings (GTK_TABLE (table3), 5);
	label20 = gtk_label_new (_("Additional settings"));
	gtk_widget_show (label20);
	gtk_table_attach (GTK_TABLE (table3), label20, 1, 2, 8, 9,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label20), 1.49012e-08, 0.5);
	label19 = gtk_label_new (_("Lock type"));
	gtk_widget_show (label19);
	gtk_table_attach (GTK_TABLE (table3), label19, 1, 2, 6, 7,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label19), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label19), 1.49012e-08, 0.5);
	label18 = gtk_label_new (_("Reset after string"));
	gtk_widget_show (label18);
	gtk_table_attach (GTK_TABLE (table3), label18, 1, 2, 4, 5,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label18), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label18), 0, 0.5);
	label17 = gtk_label_new (_("Init string"));
	gtk_widget_show (label17);
	gtk_table_attach (GTK_TABLE (table3), label17, 1, 2, 2, 3,
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label17), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label17), 0, 0.5);
	label16 = gtk_label_new (_("Command delay"));
	gtk_widget_show (label16);
	gtk_table_attach (GTK_TABLE (table3), label16, 1, 2, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label16), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label16), 2.98023e-08, 0.5);
	label11 = gtk_label_new (_("Device"));
	gtk_widget_show (label11);
	gtk_table_attach (GTK_TABLE (table3), label11, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label11), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label11), 0, 0.5);
	label12 = gtk_label_new (_("Class"));
	gtk_widget_show (label12);
	gtk_table_attach (GTK_TABLE (table3), label12, 0, 1, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label12), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label12), 1.49012e-08, 0.5);
	label13 = gtk_label_new (_("Volume"));
	gtk_widget_show (label13);
	gtk_table_attach (GTK_TABLE (table3), label13, 0, 1, 4, 5,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label13), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label13), 2.23517e-08, 0.5);
	label14 = gtk_label_new (_("Bitrate"));
	gtk_widget_show (label14);
	gtk_table_attach (GTK_TABLE (table3), label14, 0, 1, 6, 7,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label14), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label14), 0, 0.5);
	label15 = gtk_label_new (_("Dial type"));
	gtk_widget_show (label15);
	gtk_table_attach (GTK_TABLE (table3), label15, 0, 1, 8, 9,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label15), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label15), 0, 0.5);
	cbo_dev = gtk_combo_new ();
	gtk_widget_show (cbo_dev);
	gtk_table_attach (GTK_TABLE (table3), cbo_dev, 0, 1, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	cbo_dev_items = g_list_append (cbo_dev_items, "/dev/modem");
	cbo_dev_items = g_list_append (cbo_dev_items, "/dev/ttyS0");
	cbo_dev_items = g_list_append (cbo_dev_items, "/dev/ttyS1");
	cbo_dev_items = g_list_append (cbo_dev_items, "/dev/ttyS2");
	cbo_dev_items = g_list_append (cbo_dev_items, "/dev/ttyS3");
	gtk_combo_set_popdown_strings (GTK_COMBO (cbo_dev), cbo_dev_items);
	g_list_free (cbo_dev_items);
	SetupModemTab.cbo_entry_dev = GTK_COMBO (cbo_dev)->entry;
	gtk_widget_show (SetupModemTab.cbo_entry_dev);
	cbo_class = gtk_combo_new ();
	gtk_widget_show (cbo_class);
	gtk_table_attach (GTK_TABLE (table3), cbo_class, 0, 1, 3, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_combo_set_value_in_list (GTK_COMBO (cbo_class), TRUE, FALSE);
	gtk_combo_set_case_sensitive (GTK_COMBO (cbo_class), TRUE);
	cbo_class_items = g_list_append (cbo_class_items, ("CLASS 1"));
	cbo_class_items = g_list_append (cbo_class_items, ("CLASS 2"));
	cbo_class_items = g_list_append (cbo_class_items, ("CLASS 2.0"));
	gtk_combo_set_popdown_strings (GTK_COMBO (cbo_class), cbo_class_items);
	g_list_free (cbo_class_items);
	SetupModemTab.cbo_entry_class = GTK_COMBO (cbo_class)->entry;
	gtk_widget_show (SetupModemTab.cbo_entry_class);
	gtk_entry_set_editable (GTK_ENTRY (SetupModemTab.cbo_entry_class), FALSE);
	SetupModemTab.adj_vol = gtk_adjustment_new (1, 0, 3, 1, 1, 1);
	hscale_vol = gtk_hscale_new (GTK_ADJUSTMENT (SetupModemTab.adj_vol));
	gtk_widget_show (hscale_vol);
	gtk_table_attach (GTK_TABLE (table3), hscale_vol, 0, 1, 5, 6,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_scale_set_draw_value (GTK_SCALE (hscale_vol), FALSE);
	gtk_scale_set_value_pos (GTK_SCALE (hscale_vol), GTK_POS_LEFT);
	gtk_scale_set_digits (GTK_SCALE (hscale_vol), 0);
	gtk_range_set_update_policy (GTK_RANGE (hscale_vol), GTK_UPDATE_DELAYED);
	cbo_bitrate = gtk_combo_new ();
	gtk_widget_show (cbo_bitrate);
	gtk_table_attach (GTK_TABLE (table3), cbo_bitrate, 0, 1, 7, 8,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_combo_set_value_in_list (GTK_COMBO (cbo_bitrate), TRUE, FALSE);
	gtk_combo_set_case_sensitive (GTK_COMBO (cbo_bitrate), TRUE);
	cbo_bitrate_items = g_list_append (cbo_bitrate_items, "2400 bps");
	cbo_bitrate_items = g_list_append (cbo_bitrate_items, "4800 bps");
	cbo_bitrate_items = g_list_append (cbo_bitrate_items, "7200 bps");
	cbo_bitrate_items = g_list_append (cbo_bitrate_items, "9600 bps");
	cbo_bitrate_items = g_list_append (cbo_bitrate_items, "12000 (v:17)");
	cbo_bitrate_items = g_list_append (cbo_bitrate_items, "14400 (v:17)");
	gtk_combo_set_popdown_strings (GTK_COMBO (cbo_bitrate), cbo_bitrate_items);
	g_list_free (cbo_bitrate_items);
	SetupModemTab.cbo_entry_bitrate = GTK_COMBO (cbo_bitrate)->entry;
	gtk_widget_show (SetupModemTab.cbo_entry_bitrate);
	gtk_entry_set_editable (GTK_ENTRY (SetupModemTab.cbo_entry_bitrate), FALSE);
	
	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	gtk_table_attach (GTK_TABLE (table3), hbox1, 0, 1, 9, 10,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);
	SetupModemTab.rad_tone = gtk_radio_button_new_with_label (dt_group, _("Tone"));
	dt_group = gtk_radio_button_group (GTK_RADIO_BUTTON (SetupModemTab.rad_tone));
	gtk_widget_show (SetupModemTab.rad_tone);
	gtk_box_pack_start (GTK_BOX (vbox2), SetupModemTab.rad_tone, TRUE, TRUE, 0);
	rad_pulse = gtk_radio_button_new_with_label (dt_group, _("Pulse"));
	dt_group = gtk_radio_button_group (GTK_RADIO_BUTTON (rad_pulse));
	gtk_widget_show (rad_pulse);
	gtk_box_pack_start (GTK_BOX (hbox1), rad_pulse, TRUE, TRUE, 0);
	cbo_scan = gtk_combo_new ();
	gtk_widget_show (cbo_scan);
	gtk_table_attach (GTK_TABLE (table3), cbo_scan, 1, 2, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_combo_set_value_in_list (GTK_COMBO (cbo_scan), TRUE, FALSE);
	gtk_combo_set_case_sensitive (GTK_COMBO (cbo_scan), TRUE);
	cbo_scan_items = g_list_append (cbo_scan_items, ("100 milliseconds"));
	cbo_scan_items = g_list_append (cbo_scan_items, ("200 milliseconds"));
	cbo_scan_items = g_list_append (cbo_scan_items, ("300 milliseconds"));
	cbo_scan_items = g_list_append (cbo_scan_items, ("400 milliseconds"));
	cbo_scan_items = g_list_append (cbo_scan_items, ("500 milliseconds"));
	gtk_combo_set_popdown_strings (GTK_COMBO (cbo_scan), cbo_scan_items);
	g_list_free (cbo_scan_items);
	
	SetupModemTab.cbo_entry_scan = GTK_COMBO (cbo_scan)->entry;
	gtk_widget_show (SetupModemTab.cbo_entry_scan);
	gtk_entry_set_editable (GTK_ENTRY (SetupModemTab.cbo_entry_scan), FALSE);
	
	cbo_init = gtk_combo_new ();
	gtk_widget_show (cbo_init);
	gtk_table_attach (GTK_TABLE (table3), cbo_init, 1, 2, 3, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	cbo_init_items = g_list_append (cbo_init_items, "-iZ&FE&D2S7=120&C0");
	gtk_combo_set_popdown_strings (GTK_COMBO (cbo_init), cbo_init_items);
	g_list_free (cbo_init_items);
	SetupModemTab.cbo_entry_init = GTK_COMBO (cbo_init)->entry;
	gtk_widget_show (SetupModemTab.cbo_entry_init);
	cbo_init_after = gtk_combo_new ();
	gtk_widget_show (cbo_init_after);
	gtk_table_attach (GTK_TABLE (table3), cbo_init_after, 1, 2, 5, 6,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	cbo_init_after_items = g_list_append (cbo_init_after_items, ("-kZ"));
	cbo_init_after_items = g_list_append (cbo_init_after_items, ("-kZ&F+FCLASS=0"));
	gtk_combo_set_popdown_strings (GTK_COMBO (cbo_init_after), cbo_init_after_items);
	g_list_free (cbo_init_after_items);
	SetupModemTab.cbo_entry_init_after = GTK_COMBO (cbo_init_after)->entry;
	gtk_widget_show (SetupModemTab.cbo_entry_init_after);
	cbo_lock = gtk_combo_new ();
	gtk_widget_show (cbo_lock);
	gtk_table_attach (GTK_TABLE (table3), cbo_lock, 1, 2, 7, 8,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_combo_set_value_in_list (GTK_COMBO (cbo_lock), TRUE, TRUE);
	cbo_lock_items = g_list_append (cbo_lock_items, "none");
	cbo_lock_items = g_list_append (cbo_lock_items, "modern");
	cbo_lock_items = g_list_append (cbo_lock_items, "older style");
	cbo_lock_items = g_list_append (cbo_lock_items, "binary");
	/*cbo_lock_items = g_list_append (cbo_lock_items, _("NeXT"));*/
	gtk_combo_set_popdown_strings (GTK_COMBO (cbo_lock), cbo_lock_items);
	g_list_free (cbo_lock_items);
	SetupModemTab.cbo_entry_lock = GTK_COMBO (cbo_lock)->entry;
	gtk_widget_show (SetupModemTab.cbo_entry_lock);
	gtk_entry_set_editable (GTK_ENTRY (SetupModemTab.cbo_entry_lock), FALSE);
	
	cbo_xon = gtk_combo_new ();
	gtk_widget_show (cbo_xon);
	gtk_table_attach (GTK_TABLE (table3), cbo_xon, 1, 2, 9, 10,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_combo_set_value_in_list (GTK_COMBO (cbo_xon), TRUE, TRUE);
	cbo_xon_items = g_list_append (cbo_xon_items, ("none"));
	cbo_xon_items = g_list_append (cbo_xon_items, ("AT&T (Dataport, Paradyne)"));
	cbo_xon_items = g_list_append (cbo_xon_items, ("Motarola (Power Modem, 3400 Pro, ...)"));
	cbo_xon_items = g_list_append (cbo_xon_items, ("QuickCom (Spirit II)"));
	cbo_xon_items = g_list_append (cbo_xon_items, ("USR (Courier, Sportster)"));
	cbo_xon_items = g_list_append (cbo_xon_items, ("Bit Reversal (Multi-Tech)"));
	gtk_combo_set_popdown_strings (GTK_COMBO (cbo_xon), cbo_xon_items);
	g_list_free (cbo_xon_items);
	SetupModemTab.cbo_entry_xon = GTK_COMBO (cbo_xon)->entry;
	gtk_widget_show (SetupModemTab.cbo_entry_xon);
	gtk_entry_set_editable (GTK_ENTRY (SetupModemTab.cbo_entry_xon), FALSE);
	lbl_modem = gtk_label_new (_("Modem"));
	gtk_widget_show (lbl_modem);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (note_prop), gtk_notebook_get_nth_page (GTK_NOTEBOOK (note_prop), 1), lbl_modem);
	table7 = gtk_table_new (10, 1, TRUE);
	gtk_widget_show (table7);
	gtk_container_add (GTK_CONTAINER (note_prop), table7);
	gtk_container_set_border_width (GTK_CONTAINER (table7), 10);
	gtk_table_set_col_spacings (GTK_TABLE (table7), 5);
	label26 = gtk_label_new (_("Page header text"));
	gtk_widget_show (label26);
	gtk_table_attach (GTK_TABLE (table7), label26, 0, 1, 3, 4,
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_widget_set_usize (label26, 84, -2);
	gtk_label_set_justify (GTK_LABEL (label26), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label26), 1.41561e-07, 0.5);
	SetupFaxTab.entry_header = gtk_entry_new ();
	gtk_widget_show (SetupFaxTab.entry_header);
	gtk_table_attach (GTK_TABLE (table7), SetupFaxTab.entry_header, 0, 1, 4, 5,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	label28 = gtk_label_new (_("Fax station identification number"));
	gtk_widget_show (label28);
	gtk_table_attach (GTK_TABLE (table7), label28,  0, 1, 5, 6,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_widget_set_usize (label28, 86, -2);
	gtk_label_set_justify (GTK_LABEL (label28), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label28), 7.45058e-09, 0.5);
	SetupFaxTab.entry_id_num = gtk_entry_new ();
	gtk_widget_show (SetupFaxTab.entry_id_num);
	gtk_table_attach (GTK_TABLE (table7), SetupFaxTab.entry_id_num, 0, 1, 6, 7,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	SetupFaxTab.lbl_cover = gtk_label_new (_("Tiffg3 cover page file"));
	gtk_widget_show (SetupFaxTab.lbl_cover);
	gtk_table_attach (GTK_TABLE (table7), SetupFaxTab.lbl_cover,0, 1, 1, 2,
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (SetupFaxTab.lbl_cover), 7.45058e-09, 0.5);
	SetupFaxTab.chk_cover = gtk_check_button_new_with_label (_("Use a cover page"));
	gtk_widget_show (SetupFaxTab.chk_cover);
	gtk_table_attach (GTK_TABLE (table7), SetupFaxTab.chk_cover, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	file_cover = gnome_file_entry_new ("cover", _("Select a tiffg3 cover page file"));
	gtk_widget_show (file_cover);
	gtk_table_attach (GTK_TABLE (table7), file_cover,  0, 1, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	gnome_entry_set_max_saved (GNOME_ENTRY (GNOME_FILE_ENTRY (file_cover)->gentry), 5);
	gnome_file_entry_set_modal (GNOME_FILE_ENTRY (file_cover), TRUE);
	SetupFaxTab.cbo_entry_cover = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_cover));
	gtk_widget_show (SetupFaxTab.cbo_entry_cover);
	fra_wrap = gtk_frame_new (_("Text File Wrapping"));
	gtk_widget_set_name (fra_wrap, "fra_wrap");
	gtk_widget_show (fra_wrap);
	gtk_table_attach (GTK_TABLE (table7), fra_wrap, 0, 1, 7, 10,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

	hbox_w = gtk_hbox_new (TRUE, 15);
	gtk_widget_show (hbox_w);
	gtk_container_add (GTK_CONTAINER (fra_wrap), hbox_w);
	gtk_container_set_border_width (GTK_CONTAINER (hbox_w), 4);

	SetupFaxTab.chk_wrap_text = gtk_check_button_new_with_label (_("Word wrap\ntext files"));
	gtk_widget_show (SetupFaxTab.chk_wrap_text);
	gtk_box_pack_start (GTK_BOX (hbox_w), SetupFaxTab.chk_wrap_text, FALSE, FALSE, 0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (SetupFaxTab.chk_wrap_text), TextWrapArgs.do_wrap);

	SetupFaxTab.vbox_line_len = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (SetupFaxTab.vbox_line_len);
	gtk_box_pack_start (GTK_BOX (hbox_w), SetupFaxTab.vbox_line_len, TRUE, TRUE, 0);

	lbl_line_length = gtk_label_new (_("Max line length"));
	gtk_widget_show (lbl_line_length);
	gtk_box_pack_start (GTK_BOX (SetupFaxTab.vbox_line_len), lbl_line_length, TRUE, TRUE, 0);
	gtk_misc_set_alignment (GTK_MISC (lbl_line_length), 7.45058e-09, 0.5);

	spin_line_len_adj = gtk_adjustment_new (TextWrapArgs.line_len, 30, 100, 1, 10, 10);
	SetupFaxTab.spin_line_len = gtk_spin_button_new (GTK_ADJUSTMENT (spin_line_len_adj), 1, 0);
	gtk_widget_show (SetupFaxTab.spin_line_len);
	gtk_box_pack_start (GTK_BOX (SetupFaxTab.vbox_line_len), SetupFaxTab.spin_line_len, TRUE, TRUE, 1);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (SetupFaxTab.spin_line_len), TRUE);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (SetupFaxTab.spin_line_len), TRUE);

	SetupFaxTab.vbox_tab_len = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (SetupFaxTab.vbox_tab_len);
	gtk_box_pack_start (GTK_BOX (hbox_w), SetupFaxTab.vbox_tab_len, TRUE, TRUE, 0);

	lbl_tab_expand = gtk_label_new (_("Tab Expansion"));
	gtk_widget_show (lbl_tab_expand);
	gtk_box_pack_start (GTK_BOX (SetupFaxTab.vbox_tab_len), lbl_tab_expand, TRUE, TRUE, 0);
	gtk_misc_set_alignment (GTK_MISC (lbl_tab_expand), 0, 0.5);

	spin_tab_len_adj = gtk_adjustment_new (TextWrapArgs.tab_len, 1, 16, 1, 10, 10);
	SetupFaxTab.spin_tab_len = gtk_spin_button_new (GTK_ADJUSTMENT (spin_tab_len_adj), 1, 0);
	gtk_widget_show (SetupFaxTab.spin_tab_len);
	gtk_box_pack_start (GTK_BOX (SetupFaxTab.vbox_tab_len), SetupFaxTab.spin_tab_len, TRUE, TRUE, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (SetupFaxTab.spin_tab_len), TRUE);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (SetupFaxTab.spin_tab_len), TRUE);

	lbl_fax = gtk_label_new (_("Fax"));
	gtk_widget_show (lbl_fax);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (note_prop), gtk_notebook_get_nth_page (GTK_NOTEBOOK (note_prop), 2), lbl_fax);
	/*printer page*/
	vbox7 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox7);
	gtk_container_add (GTK_CONTAINER (note_prop), vbox7);
	gtk_container_set_border_width (GTK_CONTAINER (vbox7), 10);
	table18 = gtk_table_new (5, 2, FALSE);
	gtk_widget_show (table18);
	gtk_box_pack_start (GTK_BOX (vbox7), table18, TRUE, TRUE, 0);
	hseparator3 = gtk_hseparator_new ();
	gtk_widget_show (hseparator3);
	gtk_table_attach (GTK_TABLE (table18), hseparator3, 0, 2, 4, 5,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	label39 = gtk_label_new (_("Printing\nOption"));
	gtk_widget_show (label39);
	gtk_table_attach (GTK_TABLE (table18), label39, 0, 1, 2, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label39), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label39), 7.45058e-09, 0.49);
	label36 = gtk_label_new (_("Print command"));
	gtk_widget_show (label36);
	gtk_table_attach (GTK_TABLE (table18), label36, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label36), 7.45058e-09, 0.5);
	hseparator5 = gtk_hseparator_new ();
	gtk_widget_show (hseparator5);
	gtk_table_attach (GTK_TABLE (table18), hseparator5, 0, 2, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	SetupPrintTab.entry_printer = gtk_entry_new ();
	gtk_widget_show (SetupPrintTab.entry_printer);
	gtk_table_attach (GTK_TABLE (table18), SetupPrintTab.entry_printer, 1, 2, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	SetupPrintTab.rad_ps = gtk_radio_button_new_with_label (lan_group, _("Convert to postscript with fax2ps"));
	lan_group = gtk_radio_button_group (GTK_RADIO_BUTTON (SetupPrintTab.rad_ps));
	gtk_widget_show (SetupPrintTab.rad_ps);
	gtk_table_attach (GTK_TABLE (table18), SetupPrintTab.rad_ps, 1, 2, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	rad_pcl = gtk_radio_button_new_with_label (lan_group, _("Let my print filter handle conversions"));
	lan_group = gtk_radio_button_group (GTK_RADIO_BUTTON (rad_pcl));
	gtk_widget_show (rad_pcl);
	gtk_table_attach (GTK_TABLE (table18), rad_pcl, 1, 2, 3, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	table19 = gtk_table_new (3, 5, FALSE);
	gtk_widget_show (table19);
	gtk_box_pack_start (GTK_BOX (vbox7), table19, TRUE, TRUE, 0);
	label41 = gtk_label_new (_("Print Resolution"));
	gtk_widget_show (label41);
	gtk_table_attach (GTK_TABLE (table19), label41, 3, 5, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label41), 7.45058e-09, 0.5);
	label40 = gtk_label_new (_("Print Area"));
	gtk_widget_show (label40);
	gtk_table_attach (GTK_TABLE (table19), label40, 0, 2, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label40), 7.45058e-09, 0.5);
	label35 = gtk_label_new (_("H x inches"));
	gtk_widget_show (label35);
	gtk_table_attach (GTK_TABLE (table19), label35, 0, 1, 2, 3,
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label35), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label35), 0, 0.5);
	label34 = gtk_label_new (_("W x inches"));
	gtk_widget_show (label34);
	gtk_table_attach (GTK_TABLE (table19), label34, 0, 1, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label34), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label34), 0, 0.5);
	label31 = gtk_label_new (_("X dots/inch"));
	gtk_widget_show (label31);
	gtk_table_attach (GTK_TABLE (table19), label31, 3, 4, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label31), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label31), 0, 0.5);
	label32 = gtk_label_new (_("Y lines/inch"));
	gtk_widget_show (label32);
	gtk_table_attach (GTK_TABLE (table19), label32, 3, 4, 2, 3,
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label32), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label32), 0, 0.5);
	spin_x_res_adj = gtk_adjustment_new (Print.x_res, 75, 600, 5, 1, 1);
	SetupPrintTab.spin_x_res = gtk_spin_button_new (GTK_ADJUSTMENT (spin_x_res_adj), 1, 0);
	gtk_widget_show (SetupPrintTab.spin_x_res);
	gtk_table_attach (GTK_TABLE (table19), SetupPrintTab.spin_x_res, 4, 5, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (SetupPrintTab.spin_x_res), TRUE);
	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (SetupPrintTab.spin_x_res), GTK_UPDATE_IF_VALID);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (SetupPrintTab.spin_x_res), TRUE);
	spin_y_res_adj = gtk_adjustment_new (Print.y_res, 75, 600, 5, 1, 1);
	SetupPrintTab.spin_y_res = gtk_spin_button_new (GTK_ADJUSTMENT (spin_y_res_adj), 1, 0);
	gtk_widget_show (SetupPrintTab.spin_y_res);
	gtk_table_attach (GTK_TABLE (table19), SetupPrintTab.spin_y_res, 4, 5, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (SetupPrintTab.spin_y_res), TRUE);
	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (SetupPrintTab.spin_y_res), GTK_UPDATE_IF_VALID);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (SetupPrintTab.spin_y_res), TRUE);
	spin_page_width_adj = gtk_adjustment_new (Print.page_width, 5, 8.5, 0.1, 1, 1);
	SetupPrintTab.spin_page_width = gtk_spin_button_new (GTK_ADJUSTMENT (spin_page_width_adj), 1, 1);
	gtk_widget_show (SetupPrintTab.spin_page_width);
	gtk_table_attach (GTK_TABLE (table19), SetupPrintTab.spin_page_width, 1, 2, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (SetupPrintTab.spin_page_width), TRUE);
	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (SetupPrintTab.spin_page_width), GTK_UPDATE_IF_VALID);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (SetupPrintTab.spin_page_width), TRUE);
	spin_page_height_adj = gtk_adjustment_new (Print.page_height, 7, 11.5, 0.1, 1, 1);
	SetupPrintTab.spin_page_height = gtk_spin_button_new (GTK_ADJUSTMENT (spin_page_height_adj), 1, 1);
	gtk_widget_show (SetupPrintTab.spin_page_height);
	gtk_table_attach (GTK_TABLE (table19), SetupPrintTab.spin_page_height, 1, 2, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (SetupPrintTab.spin_page_height), TRUE);
	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (SetupPrintTab.spin_page_height), GTK_UPDATE_IF_VALID);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (SetupPrintTab.spin_page_height), TRUE);
	vseparator1 = gtk_vseparator_new ();
	gtk_widget_show (vseparator1);
	gtk_table_attach (GTK_TABLE (table19), vseparator1, 2, 3, 0, 3,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 10, 0);
	label29 = gtk_label_new (_("Printing"));
	gtk_widget_show (label29);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (note_prop), gtk_notebook_get_nth_page (GTK_NOTEBOOK (note_prop), 3), label29);
	gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (note_prop), gtk_notebook_get_nth_page (GTK_NOTEBOOK (note_prop), 3),_("Printing"));
	
	/*Advanced tab*/
	table_advanced = gtk_table_new (10, 2, FALSE);
	gtk_widget_show (table_advanced);
	gtk_container_add (GTK_CONTAINER (note_prop), table_advanced);
	gtk_container_set_border_width (GTK_CONTAINER (table_advanced), 5);

	SetupAdvancedTab.chk_debug_win = gtk_check_button_new_with_label (_("Open debug window when sending, polling and receiving"));
	gtk_widget_show (SetupAdvancedTab.chk_debug_win);
	gtk_table_attach (GTK_TABLE (table_advanced), SetupAdvancedTab.chk_debug_win, 0, 2, 0, 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
			(GtkAttachOptions) (0), 0, 0);
	SetupAdvancedTab.chk_auto_save_cardfile = gtk_check_button_new_with_label (_("Automatically save address book on exit"));
	gtk_widget_show (SetupAdvancedTab.chk_auto_save_cardfile);
	gtk_table_attach (GTK_TABLE (table_advanced), SetupAdvancedTab.chk_auto_save_cardfile, 0, 2, 1, 2,
			(GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
			(GtkAttachOptions) (0), 0, 0);
	label_adv = gtk_label_new (_("Advanced"));
	gtk_widget_show (label_adv);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (note_prop), gtk_notebook_get_nth_page (GTK_NOTEBOOK (note_prop), 4), label_adv);
	gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (note_prop), gtk_notebook_get_nth_page (GTK_NOTEBOOK (note_prop), 4),_("Advanced"));
	
	gtk_signal_connect (GTK_OBJECT (dialog_prop), "apply",
  					GTK_SIGNAL_FUNC(on_btn_prop_ok_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (dialog_prop), "help",
					GTK_SIGNAL_FUNC (on_btn_prop_help_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (dialog_prop), "close",
                      	GTK_SIGNAL_FUNC (destroy_dialog_prop), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupFaxTab.chk_cover), "toggled",
					GTK_SIGNAL_FUNC (on_chk_cover_toggled), file_cover);
	gtk_signal_connect (GTK_OBJECT (SetupPrintTab.rad_ps), "toggled",
					GTK_SIGNAL_FUNC (on_rad_ps_toggled), table19);
	gtk_signal_connect (GTK_OBJECT (SetupFaxTab.chk_wrap_text), "toggled",
					GTK_SIGNAL_FUNC (on_chk_wrap_text_toggled), NULL);
                      
	/*Set values as they are now*/
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SetupGeneralTab.chk_view_all), load_all);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SetupGeneralTab.chk_sort), load_ordered);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SetupGeneralTab.chk_auto_print), Print.auto_print);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SetupGeneralTab.chk_number_warn), remove_number_warn);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SetupAdvancedTab.chk_debug_win), debug_window);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SetupAdvancedTab.chk_auto_save_cardfile), auto_save_cardfile);
	gtk_entry_set_text(GTK_ENTRY(SetupGeneralTab.entry_fax_dir), recvd_dir);
	gtk_entry_set_text(GTK_ENTRY(SetupGeneralTab.entry_bin_dir), bin_dir);
	switch (EfaxArgs.dev[11]){
		case '0':
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_dev), "/dev/ttyS0");
  			break;
  		case '1':
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_dev), "/dev/ttyS1");
  			break;
  		case '2':
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_dev), "/dev/ttyS2");
  			break;
  		case '3':
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_dev), "/dev/ttyS3");
  			break;
  		default:
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_dev), "/dev/modem");
  		 	break;
  	}
  	if (EfaxArgs.mclass[2] == '2'){
  		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_class), "CLASS 2");
  	}else if (EfaxArgs.mclass[2] == '1'){
  		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_class), "CLASS 1");
  	}else{
  		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_class), "CLASS 2.0");
  	}
  	if (strcmp(EfaxArgs.vol, "-iM0L0") == 0){
  		GTK_ADJUSTMENT(SetupModemTab.adj_vol)->value = 0;
  	}else if (strcmp(EfaxArgs.vol, "-iM2L0") == 0){
  		GTK_ADJUSTMENT(SetupModemTab.adj_vol)->value = 1;
  	}else{
  		GTK_ADJUSTMENT(SetupModemTab.adj_vol)->value = 2;
  	}
  	switch (EfaxArgs.caps[4]){
  		case '0':
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_bitrate), "2400 bps");
  			break;
  		case '1':
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_bitrate), "4800 bps");
  			break;
  		case '2':
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_bitrate), "7200 bps");
  			break;
  		case '3':
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_bitrate), "9600 bps");
  			break;
  		case '4':
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_bitrate), "12000 (v:17)");
  			break;
  		default:
  			gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_bitrate), "14400 (v:17)");
  		 	break;
  	}
  	if (EfaxArgs.prefix[2] == 'T'){
  		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (SetupModemTab.rad_tone), TRUE);
  	}else{
  		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rad_pulse), TRUE);
  	}
  	if (strcmp(EfaxArgs.cmddelay, "default") == 0){
  		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_scan), ("100 milliseconds"));
  	}else if (strcmp(EfaxArgs.cmddelay, "-oz") == 0){
  		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_scan), ("200 milliseconds"));
  	}else if (strcmp(EfaxArgs.cmddelay, "-ozz") == 0){
  		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_scan), ("300 milliseconds"));
  	}else if (strcmp(EfaxArgs.cmddelay, "-ozzz") == 0){
  		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_scan), ("400 milliseconds"));
  	}else{
  		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_scan), ("500 milliseconds"));
  	}
  	gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_init), EfaxArgs.init);
  	gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_init_after), EfaxArgs.reset);
  	gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_lock), EfaxArgs.locktype);
  	if (strcmp(EfaxArgs.xonxoff, "none") == 0){
  		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_xon), ("none"));
	}else if (strcmp(EfaxArgs.xonxoff, "-j&H2") == 0){
		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_xon), ("USR (Courier, Sportster)"));
	}else if (strcmp(EfaxArgs.xonxoff, "-j*F1") == 0){
		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_xon), ("QuickCom (Spirit II)"));
	}else if (strcmp(EfaxArgs.xonxoff, "-j\\Q1") == 0){
		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_xon), ("Motarola (Power Modem, 3400 Pro, ...)"));
	}else if (strcmp(EfaxArgs.xonxoff, "-j\\Q4") == 0){
		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_xon), ("AT&T (Dataport, Paradyne)"));
	}else{
		gtk_entry_set_text (GTK_ENTRY (SetupModemTab.cbo_entry_xon), ("Bit Reversal (Multi-Tech)"));
	}
  
  	temp = g_strdup(EfaxArgs.header);
  	temp2 = strchr(temp, 'h');
	if (temp2 == NULL){
		g_error("No h in entry header\nQuitting");
	}
	*temp2 = '\0';	/*now points to "h" */
	++temp2;
  	gtk_entry_set_text (GTK_ENTRY (SetupFaxTab.entry_header), temp2);
  	g_free(temp);
  	temp = NULL;
  	temp2 = NULL;
   	temp = g_strdup(EfaxArgs.ident);
    	temp2 = strchr(temp, 'l');
	if (temp2 == NULL){
		g_error("No l in entry Ident\nQuitting");
	}
	*temp2 = '\0';	/*now points to "h" */
	++temp2;
  	gtk_entry_set_text (GTK_ENTRY (SetupFaxTab.entry_id_num), temp2);
  	g_free(temp);
  	temp2 = NULL;
  	temp = NULL;
  	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SetupFaxTab.chk_cover), EfaxArgs.usecover);
  	if (EfaxArgs.usecover == 0){
  		gtk_widget_set_sensitive (file_cover, FALSE);
  		gtk_widget_set_sensitive (SetupFaxTab.lbl_cover, FALSE);
  	}
  	gtk_entry_set_text (GTK_ENTRY (SetupFaxTab.cbo_entry_cover), EfaxArgs.cover);
  	gtk_entry_set_text (GTK_ENTRY (SetupPrintTab.entry_printer), Print.command);
  	if(Print.convert){
  		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (SetupPrintTab.rad_ps), TRUE);
  	}else{
  		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rad_pcl), TRUE);
  		gtk_widget_set_sensitive (GTK_WIDGET(table19), FALSE);
  	}


    	/*if anything changes set apply button sensitive*/
  	gtk_signal_connect (GTK_OBJECT (SetupFaxTab.chk_cover), "toggled",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupModemTab.rad_tone), "toggled",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupGeneralTab.chk_number_warn), "toggled",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupGeneralTab.chk_sort), "toggled",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupGeneralTab.chk_view_all), "toggled",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupGeneralTab.chk_auto_print), "toggled",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupPrintTab.entry_printer), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupPrintTab.spin_page_height), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupPrintTab.spin_page_width), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupPrintTab.spin_x_res), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupPrintTab.spin_y_res), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupPrintTab.rad_ps), "toggled",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupModemTab.cbo_entry_scan), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupModemTab.cbo_entry_xon), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupModemTab.cbo_entry_class), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupFaxTab.cbo_entry_cover), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupFaxTab.entry_id_num), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupFaxTab.entry_header), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupModemTab.cbo_entry_init_after), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupModemTab.cbo_entry_init), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupModemTab.cbo_entry_bitrate), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupModemTab.cbo_entry_lock), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupModemTab.cbo_entry_dev), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupGeneralTab.entry_bin_dir), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupGeneralTab.entry_fax_dir), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupModemTab.adj_vol), "value_changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupFaxTab.spin_line_len), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupFaxTab.spin_tab_len), "changed",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupFaxTab.chk_wrap_text), "toggled",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupAdvancedTab.chk_debug_win), "toggled",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (SetupAdvancedTab.chk_auto_save_cardfile), "toggled",
					GTK_SIGNAL_FUNC (property_box_changed), NULL);

  	/*show the dialog*/
	gtk_widget_show(dialog_prop);
	return;
}



/*end*/





