/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <gnome.h>

#include "sig_err.h"
#include "appbar.h"


static const gchar *EFAX_ERROR_1 =	N_("ERROR Efax Complains\n\n"
						"The dialed number was busy or\n"
						"the modem device was in use.\n\n"
						"Try again later.");

static const gchar *EFAX_ERROR_2 =	N_("ERROR Efax Complains\n"
						"Unknown Error\n\n"
						"file not found or\n"
						"disk full or\n"
						"no dialtone or\n"
						"you don't have permission to use the modem.");

static const gchar *EFAX_ERROR_3 =	N_("ERROR Efax Complains\n\n"
						"Modem protocol error.\n"
						"The program did not receive the expected  response  from  the  modem.\n"
						"The modem may not have been properly initialized,\n"
						"the correct -o options were not used,\n"
						"or a bug report may be in order.");

static const gchar *EFAX_ERROR_4 =	N_("ERROR Efax Complains\n"
						"The modem is not responding.\n\n"
						"Check that the modem is turned on and\n"
						"you have selected the correct port\n");

static const gchar *EFAX_ERROR_7 = 	N_("ERROR Efax ?\n"
						"Could not execute EFAX\n\n"
						"Check that EFAX is installed and\n"
						"in your path");

static const gchar *EFIX_ERROR_1 =	N_("ERROR Efix Complains\n\n"
						"EFIX cannot create fax pages");

static const gchar *GS_ERROR_1 = 	N_("ERROR gs Complains\n\n"
						"GHOSTSCRIPT cannot create fax pages");

static const gchar *EFIX_ERROR_7 = N_("ERROR Efix Complains\n"
						"Could not execute EFIX\n\n"
						"Check that EFIX is installed and\n"
						"in your path");

static const gchar *GS_ERROR_7 = 	N_("ERROR gs Complains\n"
						"Could not execute GHOSTSCRIPT\n\n"
						"Check that GHOSTSCRIPT is installed and\n"
						"in your path");

static const gchar *GS_UNKNOWN_ERROR = N_("ERROR gs Complains\n"
							"GHOSTSCRIPT UNKNOWN ERROR\n\n"
							"while creating fax pages\n"
							"The file may not be what it claims to be\n");
							
static const gchar *EFIX_UNKNOWN_ERROR = N_("ERROR Efix Complains\n"
							"EFIX UNKNOWN ERROR\n\n"
							"while creating fax pages\n"
							"The file may not be what it claims to be\n");
						
						
void efax_error(int err)
{
	switch (err){
		case 1:
			gnome_error_dialog(_(EFAX_ERROR_1));
			static_status_msg(_("ERROR: Efax returned 1"));
			break;
		case 2:
			gnome_error_dialog(_(EFAX_ERROR_2));
			static_status_msg(_("ERROR: Efax returned 2"));
			break;
		case 3:
			gnome_error_dialog(_(EFAX_ERROR_3));
			static_status_msg(_("ERROR: Efax returned 3"));
			break;
		case 4:
			gnome_error_dialog(_(EFAX_ERROR_4));
			static_status_msg(_("ERROR: Efax returned 4"));
			break;
		case 7:
			gnome_error_dialog(_(EFAX_ERROR_7));
			static_status_msg(_("ERROR: Can't execute Efax"));
			break;
		default:
			break;
	}

}

void efix_error(int err)
{
	switch (err){
		case 1:
			gnome_error_dialog(_(EFIX_ERROR_1));
			break;
		case 7:
			gnome_error_dialog(_(EFIX_ERROR_7));
			break;
		default:
			gnome_error_dialog(_(EFIX_UNKNOWN_ERROR));
			break;
	}
	static_status_msg(_("ERROR: Efix Can't Create Fax Pages"));
}

void gs_error(int err)
{
	switch (err){
		case 1:
			gnome_error_dialog(_(GS_ERROR_1));
			break;
		case 7:
			gnome_error_dialog(_(GS_ERROR_7));
			break;
		default:
			gnome_error_dialog(_(GS_UNKNOWN_ERROR));
			break;
	}
	static_status_msg(_("ERROR: gs Can't Create Fax Pages"));
}


/*end*/





