/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gnome.h>
#include "gefax.h"
#include "splash.h"

GtkWidget *splash = NULL;
static gint remove_splash(gpointer splash);

void display_splash_window(void)
{
	gchar *pixmap_filename;
	GtkWidget *pixmap;
	GtkWidget *fixed;
	GdkPixmap *gdk_pixmap;
	GdkBitmap *mask;
	GtkStyle *style;
	GdkGC *gc;

	/*show splash screen*/
	splash = gtk_window_new (GTK_WINDOW_POPUP);
	gtk_widget_realize(splash);
	gtk_window_set_position (GTK_WINDOW (splash), GTK_WIN_POS_CENTER);
	gtk_window_set_policy (GTK_WINDOW (splash), FALSE, FALSE, FALSE);

	pixmap_filename = gnome_pixmap_file (GEFAX_ICON_SPLASH);
	if (!pixmap_filename) {
		g_warning (_("Couldn't find pixmap file: %s"), GEFAX_ICON_SPLASH);
		g_free (pixmap_filename);
		gtk_widget_destroy(splash);
		return;
	}

	
	style = gtk_widget_get_default_style();
	gc = style->black_gc;
	gdk_pixmap = gdk_pixmap_create_from_xpm( splash->window, &mask, &style->bg[GTK_STATE_NORMAL], pixmap_filename );
	g_free (pixmap_filename);
	pixmap = gtk_pixmap_new( gdk_pixmap, mask );
	gtk_widget_show( pixmap );

	/* To display the pixmap, we use a fixed widget to place the pixmap */
	fixed = gtk_fixed_new();
	/*gtk_widget_set_usize( fixed, 200, 200 );*/
	gtk_fixed_put( GTK_FIXED(fixed), pixmap, 0, 0 );
	gtk_container_add( GTK_CONTAINER(splash), fixed );
	gtk_widget_show( fixed );

	/* This masks out everything except for the image itself */
	gtk_widget_shape_combine_mask( splash, mask, 0, 0 );
    
	gtk_widget_show (splash);
}

void destroy_spash(gint delay)
{
	gint splash_timer;
	gdk_window_raise(splash->window);
	splash_timer = gtk_timeout_add(delay, remove_splash,splash);
}

gint remove_splash(gpointer splash)
{
	gtk_widget_destroy (GTK_WIDGET(splash));
	splash = NULL;
	return FALSE;
}
