/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>

#include <gnome.h>

#include "support.h"

GList
*gefax_support_get_tiff_filenames_from_dir		(const gchar		*directory_path)
{
	GList *List = NULL;
	gint fcount;
	gint files;
	gchar *fname;
	gchar *ftype;
	struct dirent **filelist;
	
	files = scandir (directory_path, &filelist, 0, alphasort); /*count num of files*/
	if (files > 2) {
		for (fcount = 0; fcount < files; ++fcount) {
			fname = g_strdup_printf ("%s/%s", directory_path, filelist[fcount]->d_name);
			if (g_file_test (fname, G_FILE_TEST_ISFILE)) {
				ftype = g_strdup (gnome_mime_type_or_default_of_file (fname, "UNKNOWN"));
				if (strcmp (ftype, "image/tiff") == 0) {
					List = g_list_append (List, fname);
				} else {
					g_free (fname);
				}
				g_free (ftype);
			}
		}	
	}
	return List;
}

void
gefax_support_free_tiff_filenames			(GList			*FileList)
{
	gint list_len, i;
	GList *List = FileList;
	
	g_return_if_fail (List != NULL);
	
	list_len = g_list_length (List);
	for (i = 0; i < list_len; i++) {
		gchar *data = List->data;
		if (data) {
			g_free (data);
		}
		List = g_list_next (List);
	}
	g_list_free (FileList);
}

/* This is an internally used function to create pixmaps. */
static GtkWidget* create_dummy_pixmap(GtkWidget *widget, gboolean gnome_pixmap);

/* This is a dummy pixmap we use when a pixmap can't be found. */
static char *dummy_pixmap_xpm[] = {
/* columns rows colors chars-per-pixel */
"1 1 1 1",
"  c None",
/* pixels */
" ",
" "
};

void e_shortcut_bar_set_canvas_style (GtkWidget *w)
{
        GtkRcStyle *rc_style;

        rc_style = gtk_rc_style_new ();

        rc_style->color_flags[GTK_STATE_NORMAL] = GTK_RC_FG | GTK_RC_BG ;
        rc_style->color_flags[GTK_STATE_INSENSITIVE] = GTK_RC_FG | GTK_RC_BG | GTK_RC_BASE;
        rc_style->color_flags[GTK_STATE_ACTIVE] = GTK_RC_FG | GTK_RC_BG;
        rc_style->color_flags[GTK_STATE_PRELIGHT] = GTK_RC_FG | GTK_RC_BG;
        rc_style->color_flags[GTK_STATE_SELECTED] = GTK_RC_FG | GTK_RC_BG;
        
        rc_style->fg[GTK_STATE_NORMAL].red   = 65535;
        rc_style->fg[GTK_STATE_NORMAL].green = 65535;
        rc_style->fg[GTK_STATE_NORMAL].blue  = 65535;

        rc_style->bg[GTK_STATE_NORMAL].red   = 32512;
        rc_style->bg[GTK_STATE_NORMAL].green = 32512;
        rc_style->bg[GTK_STATE_NORMAL].blue  = 32512;
        
        rc_style->fg[GTK_STATE_INSENSITIVE].red   = 65535;
        rc_style->fg[GTK_STATE_INSENSITIVE].green = 65535;
        rc_style->fg[GTK_STATE_INSENSITIVE].blue  = 65535;

        rc_style->bg[GTK_STATE_INSENSITIVE].red   = 32512;
        rc_style->bg[GTK_STATE_INSENSITIVE].green = 32512;
        rc_style->bg[GTK_STATE_INSENSITIVE].blue  = 32512;
        
        rc_style->base[GTK_STATE_INSENSITIVE].red   = 220 * 255;
        rc_style->base[GTK_STATE_INSENSITIVE].green = 220 * 255;
        rc_style->base[GTK_STATE_INSENSITIVE].blue  = 220 * 255;
       
	rc_style->fg[GTK_STATE_ACTIVE].red   = 65535;
        rc_style->fg[GTK_STATE_ACTIVE].green = 65535;
        rc_style->fg[GTK_STATE_ACTIVE].blue  = 65535;

        rc_style->bg[GTK_STATE_ACTIVE].red   = 32512;
        rc_style->bg[GTK_STATE_ACTIVE].green = 32512;
        rc_style->bg[GTK_STATE_ACTIVE].blue  = 32512;

	rc_style->fg[GTK_STATE_PRELIGHT].red   = 65535;
        rc_style->fg[GTK_STATE_PRELIGHT].green = 65535;
        rc_style->fg[GTK_STATE_PRELIGHT].blue  = 65535;

        rc_style->bg[GTK_STATE_PRELIGHT].red   = 32512;
        rc_style->bg[GTK_STATE_PRELIGHT].green = 32512;
        rc_style->bg[GTK_STATE_PRELIGHT].blue  = 32512;
        
        rc_style->fg[GTK_STATE_SELECTED].red   = 65535;
        rc_style->fg[GTK_STATE_SELECTED].green = 65535;
        rc_style->fg[GTK_STATE_SELECTED].blue  = 65535;

        rc_style->bg[GTK_STATE_SELECTED].red   = 32512;
        rc_style->bg[GTK_STATE_SELECTED].green = 32512;
        rc_style->bg[GTK_STATE_SELECTED].blue  = 32512;
        gtk_widget_modify_style (GTK_WIDGET (w), rc_style);
        gtk_rc_style_unref (rc_style);
}

/* This is an internally used function to create pixmaps. */
static GtkWidget* create_dummy_pixmap(GtkWidget *widget, gboolean gnome_pixmap)
{
	GdkColormap *colormap;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
	GtkWidget *pixmap;

	if (gnome_pixmap)
	{
		return gnome_pixmap_new_from_xpm_d (dummy_pixmap_xpm);
	}

	colormap = gtk_widget_get_colormap (widget);
	gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask,
                                                     NULL, dummy_pixmap_xpm);
	if (gdkpixmap == NULL)
		g_error ("Couldn't create replacement pixmap.");
	pixmap = gtk_pixmap_new (gdkpixmap, mask);
	gdk_pixmap_unref (gdkpixmap);
	gdk_bitmap_unref (mask);
	return pixmap;
}

/* This is an internally used function to create pixmaps. */
GtkWidget* create_pixmap(GtkWidget *widget, const gchar *filename, gboolean gnome_pixmap)
{
	GtkWidget *pixmap;
	GdkColormap *colormap;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
	gchar *pathname;

	if (!filename || !filename[0])
		return create_dummy_pixmap (widget, gnome_pixmap);

	pathname = gnome_pixmap_file (filename);
	if (!pathname){
		g_warning (_("Couldn't find pixmap file: %s"), filename);
		return create_dummy_pixmap (widget, gnome_pixmap);
	}

	if (gnome_pixmap){
		pixmap = gnome_pixmap_new_from_file (pathname);
		g_free (pathname);
		return pixmap;
	}

	colormap = gtk_widget_get_colormap (widget);
	gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask,
                                                   NULL, pathname);
	if (gdkpixmap == NULL){
		g_warning (_("Couldn't create pixmap from file: %s"), pathname);
		g_free (pathname);
		return create_dummy_pixmap (widget, gnome_pixmap);
	}
	g_free (pathname);

	pixmap = gtk_pixmap_new (gdkpixmap, mask);
	gdk_pixmap_unref (gdkpixmap);
	gdk_bitmap_unref (mask);
	return pixmap;
}

/* This is an internally used function to create imlib images. */
GdkImlibImage* create_image(const gchar *filename)
{
	GdkImlibImage *image;
	gchar *pathname;

	pathname = gnome_pixmap_file (filename);
	if (!pathname){
		g_warning (_("Couldn't find pixmap file: %s"), filename);
		return NULL;
	}

	image = gdk_imlib_load_image (pathname);
	g_free (pathname);
	return image;
}


/*end*/





