
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gnome.h>

#include "gefax.h"
#include "term.h"
#include "support.h"

static GtkWidget *text = NULL;
static GtkWidget *gefax_term = NULL;

static void on_btn_term_close_clicked(GtkButton *b, gpointer data);
static void on_btn_save_as_clicked(GtkButton *b, gpointer data);
static void on_btn_clear_clicked(GtkButton *b, gpointer data);
static void on_btn_help_clicked(GtkButton *b, gpointer data);
static void on_fs_ok_button_clicked(GtkButton *b, GtkFileSelection *fs);

void destroy_gefax_terminal(void)
{
	if(gefax_term) on_btn_term_close_clicked(NULL, NULL);
}

void gefax_terminal_add_text(gchar *string)
{
	GdkFont *fixed;
	
	if(text == NULL) return;
	
	fixed = gdk_fontset_load (GEFAX_ADDRESS_VIEW_FIXED_FONT);
	gtk_text_insert (GTK_TEXT (text), fixed, NULL, NULL, string, -1);
	gtk_text_insert (GTK_TEXT (text), fixed, NULL, NULL, "\n", 1);
	
	return;
}

void on_fs_ok_button_clicked(GtkButton *b, GtkFileSelection *fs)
{
	gchar *fname;
	gchar *msg;
	gchar *data;
	guint tlen;
	FILE *outfile;
	
	fname = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
	if (g_file_test(fname, G_FILE_TEST_ISFILE)){
		msg = g_strdup_printf(_("%s already exists!"), fname);
		gnome_error_dialog(msg);
		g_free(msg);
			
	}else{
		tlen = gtk_text_get_length(GTK_TEXT(text));
		data = gtk_editable_get_chars(GTK_EDITABLE(text), 0, tlen);
		if ((outfile = fopen(fname, "w")) == NULL){
			gnome_error_dialog(_("Could not open file for output\n\n"
						"You may not have permission to write\n"
						"files in the selected directory"));
		}else{
			fputs(data,outfile);
			gtk_object_destroy(GTK_OBJECT (fs));
			g_free(data);
			fclose(outfile);
		}
	}
		
}

void on_btn_save_as_clicked(GtkButton *b, gpointer data)
{
	GtkWidget *fs_term;

	fs_term = gtk_file_selection_new (_("gefax - Select output file name"));
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs_term)->ok_button), "clicked",
				GTK_SIGNAL_FUNC (on_fs_ok_button_clicked), fs_term);
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs_term)->cancel_button), "clicked",
    				(GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (fs_term));

	gtk_widget_show(fs_term);
}

void on_btn_clear_clicked(GtkButton *b, gpointer data)
{
	gtk_text_set_point(GTK_TEXT(text), 0);
	gtk_text_forward_delete(GTK_TEXT(text), gtk_text_get_length(GTK_TEXT(text)));
}

void on_btn_help_clicked(GtkButton *b, gpointer data)
{
	
}

void on_btn_term_close_clicked(GtkButton *b, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(gefax_term));
	gefax_term = NULL;
	text = NULL;
}

void create_gefax_terminal (void)
{
	GtkWidget *dock_term;
	GtkWidget *tool_term;
	GtkWidget *tmp_toolbar_icon;
	GtkWidget *btn_save_as;
	GtkWidget *btn_clear;
	GtkWidget *btn_help;
	GtkWidget *btn_close;
	GtkWidget *sw_term;
	GtkWidget *toolbar_dock_term;
	GtkAccelGroup *accel_group;

	if (gefax_term){
		gdk_window_raise(gefax_term->window);
		return;
	}
	accel_group = gtk_accel_group_new ();
	gefax_term = gnome_app_new ("Gefax_term", _("gefax - Debug output"));
	GTK_WINDOW (gefax_term)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_default_size (GTK_WINDOW (gefax_term), 480, 350);
	gtk_window_set_policy (GTK_WINDOW (gefax_term), TRUE, TRUE, TRUE);
	gtk_window_set_transient_for(GTK_WINDOW (gefax_term),GTK_WINDOW (gefax));
	gnome_app_enable_layout_config (GNOME_APP (gefax_term), FALSE);
	
	dock_term = GNOME_APP (gefax_term)->dock;
	gtk_widget_show (dock_term);
	
	toolbar_dock_term = gnome_dock_item_new("toolbar_dock_term",GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL | GNOME_DOCK_ITEM_BEH_LOCKED | GNOME_DOCK_ITEM_BEH_EXCLUSIVE);
	gtk_widget_show (toolbar_dock_term);
	gnome_dock_add_item (GNOME_DOCK (dock_term), GNOME_DOCK_ITEM(toolbar_dock_term), GNOME_DOCK_TOP, 1, 0, 0,TRUE);
	
	tool_term = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	gtk_widget_show (tool_term);
	gtk_container_set_border_width (GTK_CONTAINER (tool_term), 1);
	gtk_container_add (GTK_CONTAINER (toolbar_dock_term), tool_term);
	gnome_dock_item_set_shadow_type(GNOME_DOCK_ITEM(toolbar_dock_term), GTK_SHADOW_NONE);
	gtk_toolbar_set_space_size (GTK_TOOLBAR (tool_term), 16);
	gtk_toolbar_set_space_style (GTK_TOOLBAR (tool_term), GTK_TOOLBAR_SPACE_LINE);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (tool_term), GTK_RELIEF_NONE);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax_term, GNOME_STOCK_PIXMAP_SAVE_AS);
	btn_save_as = gtk_toolbar_append_element (GTK_TOOLBAR (tool_term), GTK_TOOLBAR_CHILD_BUTTON, NULL,
				_("Save As"), _("Save File"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (btn_save_as);
	
	gtk_toolbar_append_space (GTK_TOOLBAR (tool_term));

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_CLEAR);
	btn_clear = gtk_toolbar_append_element (GTK_TOOLBAR (tool_term), GTK_TOOLBAR_CHILD_BUTTON, NULL,
				_("Clear"), _("Clear output"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (btn_clear);

	gtk_toolbar_append_space (GTK_TOOLBAR (tool_term));

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_HELP);
	btn_help = gtk_toolbar_append_element (GTK_TOOLBAR (tool_term), GTK_TOOLBAR_CHILD_BUTTON, NULL,
				_("Help"), _("Help with debug output"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (btn_help);
  
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax_term, GNOME_STOCK_PIXMAP_CLOSE);
	btn_close = gtk_toolbar_append_element (GTK_TOOLBAR (tool_term), GTK_TOOLBAR_CHILD_BUTTON, NULL,
				_("Close"), _("Close gefax terminal"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (btn_close);
	gtk_widget_add_accelerator (btn_save_as, "clicked", accel_group, GDK_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

	sw_term = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (sw_term);
	gtk_widget_show (sw_term);
	gnome_app_set_contents (GNOME_APP (gefax_term), sw_term);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw_term), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
	
	text = gtk_text_new (NULL, NULL);
	gtk_widget_show (text);
	gtk_container_add (GTK_CONTAINER (sw_term), text);
	gtk_text_set_editable (GTK_TEXT (text), TRUE);
	
	gtk_signal_connect (GTK_OBJECT (btn_save_as), "clicked",
			GTK_SIGNAL_FUNC (on_btn_save_as_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_clear), "clicked",
			GTK_SIGNAL_FUNC (on_btn_clear_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_help), "clicked",
			GTK_SIGNAL_FUNC (on_btn_help_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_close), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_term_close_clicked),
                      NULL);
	gtk_signal_connect (GTK_OBJECT (gefax_term), "delete_event",
				GTK_SIGNAL_FUNC (on_btn_term_close_clicked), NULL);

	gtk_window_add_accel_group (GTK_WINDOW (gefax), accel_group);

	gtk_widget_show(gefax_term);
}

