/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gnome.h>
#include <gal/shortcut-bar/e-shortcut-bar.h>
#include <gal/e-paned/e-hpaned.h>
#include <gal/shortcut-bar/e-vscrolled-bar.h>

#include "gefax.h"
#include "control.h"
#include "toolbars.h"
#include "support.h"
#include "exec_cmd.h"
#include "browser.h"
#include "hist.h"

extern GtkWidget *clist_fax;
extern GtkWidget *clist_nums;

typedef struct _toolbar_m{
	GtkWidget *toolbar;
	GtkWidget *btn_open;
	GtkWidget *btn_view;
	GtkWidget *btn_select_all;
	GtkWidget *btn_clear_all;
	GtkWidget *btn_send_fax;
	GtkWidget *btn_poll_fax;
	GtkWidget *btn_receive_fax;
	GtkWidget *btn_add_num;
	GtkWidget *btn_move_to_trash;
	GtkWidget *btn_get_from_trash;
	GtkWidget *btn_delete_trash;
	GtkWidget *btn_options;
	GtkWidget *btn_help;
	GtkWidget *btn_exit;
	GtkWidget *btn_cancel_fax;	/*inserted on the fly*/
 } toolbar_m;
toolbar_m *MainToolbar = NULL;

static GtkWidget *tmp_toolbar_icon;
static void gefax_shortcut_bar_item_selected (GtkWidget *widget, GdkEvent *event, gint item_num, gpointer nb);

void gefax_shortcut_bar_item_selected (GtkWidget *widget, GdkEvent *event, gint item_num, gpointer nb)
{
	if (item_num == 0) {
		change_to_browser(NULL,NULL);
	} else if (item_num == 1) {
		change_to_history(NULL,NULL);
	} else if (item_num == 2) {
		change_to_contacts(NULL,NULL);
	}
}


void create_ShortcutsToolbar(GtkWidget *app_e_paned)
{
	GtkWidget *frame;
	GtkWidget *lbl_shortcuts;
	GtkWidget *toolbar;
	GtkWidget *box;


	box = gtk_vbox_new (FALSE,0);
	gtk_widget_show (box);	
	e_paned_add1 (E_PANED (app_e_paned), box);
	
	toolbar = e_icon_bar_new();
	gtk_widget_show (toolbar);
/*	gtk_widget_set_usize (toolbar, 80, -1);*/
	
	/*v_sw = e_vscrolled_bar_new (NULL);
	gtk_widget_show (v_sw);
	gtk_container_add (GTK_CONTAINER (v_sw), toolbar);*/
	gtk_box_pack_end (GTK_BOX (box), toolbar, TRUE, TRUE, 0);
	e_shortcut_bar_set_canvas_style (toolbar);

	gtk_signal_connect (GTK_OBJECT (toolbar), "item_selected",
			    GTK_SIGNAL_FUNC (gefax_shortcut_bar_item_selected), NULL);

	e_icon_bar_add_item (E_ICON_BAR (toolbar),
			     (gdk_pixbuf_new_from_file (GEFAX_ICON_SHORTCUTS_FAX)), 
			     "Fax", -1);
	e_icon_bar_add_item (E_ICON_BAR (toolbar),
			     (gdk_pixbuf_new_from_file (GEFAX_ICON_SHORTCUTS_HISTORY)), 
			     "History", -1);
	e_icon_bar_add_item (E_ICON_BAR (toolbar),
			     (gdk_pixbuf_new_from_file (GEFAX_ICON_SHORTCUTS_CONTACTS)), 
			     "Contacts", -1);
	lbl_shortcuts = gtk_label_new (_("Shortcuts"));
	gtk_widget_show (lbl_shortcuts);
	
	frame = gtk_frame_new (NULL);
	gtk_widget_show (frame);
	gtk_box_pack_start (GTK_BOX (box), frame, FALSE, TRUE, 0);
	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
	gtk_container_add (GTK_CONTAINER (frame), lbl_shortcuts);
}

void create_MainToolbar(void)
{
	GtkAccelGroup *accel_group;
	
	if(MainToolbar) return;
	MainToolbar = g_malloc(sizeof(toolbar_m));
	accel_group = gtk_accel_group_new ();
	MainToolbar->toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);

	
	gtk_container_add (GTK_CONTAINER (toolbar_dock), MainToolbar->toolbar);
	gtk_widget_show (MainToolbar->toolbar);
	gtk_container_set_border_width (GTK_CONTAINER (MainToolbar->toolbar), 2);
	gtk_toolbar_set_space_style (GTK_TOOLBAR (MainToolbar->toolbar), GTK_TOOLBAR_SPACE_LINE);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (MainToolbar->toolbar), GTK_RELIEF_NONE);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_OPEN);
	MainToolbar->btn_open = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Open"),
                                _("Open Directory"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_open);

	/*Let the browser show while doing the rest. If you put this at the top the toolbar will hide then show */
	while (gtk_events_pending()) gtk_main_iteration();

	gtk_toolbar_append_space (GTK_TOOLBAR (MainToolbar->toolbar));

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_MAIL_SND);
	MainToolbar->btn_send_fax = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Send",
                                _("Send Fax"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_send_fax);
  
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_MAIL_RCV);
	MainToolbar->btn_receive_fax = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Receive",
                                _("Receive Fax"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_receive_fax);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_MAIL_FWD);
	MainToolbar->btn_poll_fax = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Poll",
                                _("Poll Fax"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_poll_fax);

	gtk_toolbar_append_space (GTK_TOOLBAR (MainToolbar->toolbar));
  

	tmp_toolbar_icon = create_pixmap (gefax, GEFAX_ICON_BROWSER_TOOL_SELECT, TRUE);
	MainToolbar->btn_select_all = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Select",
                                _("Select First 9 Files"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_select_all);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_REFRESH);
	MainToolbar->btn_clear_all = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Reload",
                                _("Clear And Reload File List"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_clear_all);
  
	tmp_toolbar_icon = create_pixmap (gefax, GEFAX_ICON_BROWSER_TOOL_NUMBERS, TRUE);
	MainToolbar->btn_add_num = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Number",
                                _("Add Fax Numbers"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_add_num);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_SEARCH);
	MainToolbar->btn_view = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Preview",
                                _("Preview fax"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_view);
  
	gtk_toolbar_append_space (GTK_TOOLBAR (MainToolbar->toolbar));

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_TRASH);
	MainToolbar->btn_move_to_trash = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Trash",
                                _("Move Selected File To Trash"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_move_to_trash);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_UNDELETE);
	MainToolbar->btn_get_from_trash = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Get",
                                _("Retrieve A File From Trash"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_get_from_trash);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_TRASH_FULL);
	MainToolbar->btn_delete_trash = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Clear",
                                _("Remove All Files From Trash"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_delete_trash);

	gtk_toolbar_append_space (GTK_TOOLBAR (MainToolbar->toolbar));

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_PREFERENCES);
	MainToolbar->btn_options = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Setup",
                                _("Setup Program Options"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_options);

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_HELP);
	MainToolbar->btn_help = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Help",
                                _("Open Help"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_help);
	gtk_widget_add_accelerator (MainToolbar->btn_help, "clicked", accel_group,
                              GDK_F1, 0,
                              GTK_ACCEL_VISIBLE);
	gtk_toolbar_append_space (GTK_TOOLBAR (MainToolbar->toolbar));

	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_EXIT);
	MainToolbar->btn_exit = gtk_toolbar_append_element (GTK_TOOLBAR (MainToolbar->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Exit",
                                _("Exit Application"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (MainToolbar->btn_exit);

	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_open), "clicked",
					GTK_SIGNAL_FUNC (on_btn_open_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_send_fax), "clicked",
					GTK_SIGNAL_FUNC (on_btn_send_fax_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_receive_fax), "clicked",
					GTK_SIGNAL_FUNC (on_btn_receive_fax_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_poll_fax), "clicked",
					GTK_SIGNAL_FUNC (on_btn_poll_fax_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_view), "clicked",
					GTK_SIGNAL_FUNC (on_btn_view_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_select_all), "clicked",
					GTK_SIGNAL_FUNC (on_btn_select_all_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_clear_all), "clicked",
					GTK_SIGNAL_FUNC (on_btn_clear_all_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_add_num), "clicked",
					GTK_SIGNAL_FUNC (on_btn_set_nums_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_move_to_trash), "clicked",
					GTK_SIGNAL_FUNC (on_btn_move_to_trash_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_get_from_trash), "clicked",
					GTK_SIGNAL_FUNC (on_btn_get_from_trash_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_delete_trash), "clicked",
					GTK_SIGNAL_FUNC (on_btn_delete_trash_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_options), "clicked",
					GTK_SIGNAL_FUNC (on_btn_options_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_help), "clicked",
					GTK_SIGNAL_FUNC (on_btn_help_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (MainToolbar->btn_exit), "clicked",
					GTK_SIGNAL_FUNC (on_btn_exit_clicked), NULL);
	gtk_window_add_accel_group (GTK_WINDOW (gefax), accel_group);
	
	MainToolbar->btn_cancel_fax = NULL;
	return;
}
  
void remove_MainToolbar(void)
{	
	gtk_widget_destroy(MainToolbar->toolbar);
	MainToolbar->toolbar = NULL;
	g_free(MainToolbar);
	MainToolbar = NULL;
}

void adjust_toolbar(gint action)
{
	GtkWidget *tmp_toolbar_icon;
	
	if(MainToolbar){
		switch (action){
			case SEND_A:	/*sending*/
				gtk_widget_hide(MainToolbar->btn_send_fax);
				gtk_widget_set_sensitive (MainToolbar->btn_open, FALSE);
 				tmp_toolbar_icon = gnome_stock_pixmap_widget (MainToolbar->toolbar, GNOME_STOCK_PIXMAP_CLOSE);
				MainToolbar->btn_cancel_fax = gtk_toolbar_insert_item (GTK_TOOLBAR (MainToolbar->toolbar),
    	                      						NULL, _("Cancel Send Fax"), "", tmp_toolbar_icon,
									GTK_SIGNAL_FUNC (kill_fax_operation), NULL, 2);
				gtk_widget_show (MainToolbar->btn_cancel_fax);
				break;
			case RECEIVE_A:	/*receiving*/
				gtk_widget_hide(MainToolbar->btn_receive_fax);
				tmp_toolbar_icon = gnome_stock_pixmap_widget (MainToolbar->toolbar, GNOME_STOCK_PIXMAP_CLOSE);
				MainToolbar->btn_cancel_fax = gtk_toolbar_insert_item (GTK_TOOLBAR (MainToolbar->toolbar),
    	 	                    					NULL, _("Cancel Receive Fax"), "", tmp_toolbar_icon,
									GTK_SIGNAL_FUNC (kill_fax_operation), NULL, 3);
				gtk_widget_show (MainToolbar->btn_cancel_fax);
				break;
			case POLL_A:	/*polling*/
				gtk_widget_hide(MainToolbar->btn_poll_fax);
				tmp_toolbar_icon = gnome_stock_pixmap_widget (MainToolbar->toolbar, GNOME_STOCK_PIXMAP_CLOSE);
				MainToolbar->btn_cancel_fax = gtk_toolbar_insert_item (GTK_TOOLBAR (MainToolbar->toolbar),
									NULL, _("Cancel Poll Fax"), "", tmp_toolbar_icon,
									GTK_SIGNAL_FUNC (kill_fax_operation), NULL, 4);
				break;
			default:
				break;
		}

	}
	
}

void restore_toolbar(void)
{
	if(MainToolbar != NULL){
		if(MainToolbar->btn_cancel_fax != NULL){
			gtk_widget_destroy(MainToolbar->btn_cancel_fax);
			MainToolbar->btn_cancel_fax = NULL;
		}
 		gtk_widget_show (MainToolbar->btn_receive_fax);
		gtk_widget_set_sensitive (MainToolbar->btn_receive_fax, TRUE);
		gtk_widget_show (MainToolbar->btn_send_fax);
		gtk_widget_set_sensitive (MainToolbar->btn_send_fax, TRUE);
		gtk_widget_show (MainToolbar->btn_poll_fax);
		gtk_widget_set_sensitive (MainToolbar->btn_poll_fax, TRUE);
		gtk_widget_set_sensitive (MainToolbar->btn_open, TRUE);
	}
		
	return;
}


void disable_kill_button(void)
{
	if ((MainToolbar) && (MainToolbar->btn_cancel_fax)){
		gtk_widget_set_sensitive (MainToolbar->btn_cancel_fax, FALSE);
	}
}


/*end*/





