/*  Copyright Daryl Gray 2000 email daryl.g@visto.com
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc.,  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <gnome.h>
#include <gefax/gefax.h>

#include "viewer.h"
#include "gefax.h"
#include "support.h"
#include "appbar.h"
#include "print.h"
#include "control.h"

typedef struct _Viewer_Config		Viewer_Config;
typedef struct _Viewer_Toolbar		Viewer_Toolbar;


struct _Viewer_Config {
	gdouble		scale;
	gboolean	high_quality;
	GtkWidget	*pop_menu_quality;
};

static GtkWidget *fax_viewer;
static gchar *viewer_directory = NULL;

/*
 *right click manu
*/
static GtkWidget *pop_menu;
/*
 *toolbar
*/
struct _Viewer_Toolbar {
	GtkWidget *toolbar;
	GtkWidget *print;
	GtkWidget *first;
	GtkWidget *previous;
	GtkWidget *next;
	GtkWidget *last;
	GtkWidget *decrease;
	GtkWidget *increase;
	GtkWidget *full;
	GtkWidget *fit;
	GtkWidget *help;
	GtkWidget *close;
};
static Viewer_Toolbar *ViewerToolbar;
static Viewer_Config ViewerConfig;

static GnomeHelpMenuEntry view_help_ref =
{
	"gefax",
	"view.html#top"
};

/*
 *Create and destroy widgets
*/
static GtkWidget*
create_viewer_pop_menu					(void);
static void
insert_viwer_menu					(void);
static void
remove_viewer_menu					(void);
static void
create_toolbar_viewer					(void);
static void
destroy_toolbar_viewer					(void);
/*
 *Signal handlers for toolbar & munus
*/
static void
on_print_clicked					(GtkWidget		*w,
							gpointer		data);
static void
on_first_clicked					(GtkWidget		*w,
							gpointer		data);

static void
on_previous_clicked					(GtkWidget		*w,
							gpointer		data);

static void
on_next_clicked						(GtkWidget		*w,
							gpointer		data);

static void
on_last_clicked						(GtkWidget		*w,
							gpointer		data);

static void
on_high_quality_menu_toggled				(GtkMenuItem		*tb,
							gpointer		data);
static void
on_increase_clicked					(GtkWidget		*w,
							gpointer		data);

static void
on_decrease_clicked					(GtkWidget		*w,
							gpointer		data);

static void
on_full_clicked						(GtkWidget		*w,
							gpointer		data);

static void
on_fit_clicked						(GtkWidget		*w,
							gpointer		data);

static void
on_help_clicked						(GtkWidget		*w,
							gpointer		data);

/*
 *Fax viewer signals
*/

static void
on_page_changed						(LibGefaxFaxViewer	*fv,
							gint			page,
							gpointer		data);

static void
on_right_click						(GtkWidget		*w,
							guint			button,
							guint32			activate_time);

static GnomeUIInfo viewer_view[] =
{
	{
		GNOME_APP_UI_ITEM, N_("Zoom In"), N_("Increase view area"),
		on_increase_clicked, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, GEFAX_ICON_VIEWER_MENU_INCREASE, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("Zoom Out"), N_("Decrease view area"),
		on_decrease_clicked, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, GEFAX_ICON_VIEWER_MENU_DECREASE, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("Full Size"), N_("View page pixel for pixel"),
		on_full_clicked, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, GEFAX_ICON_VIEWER_MENU_FULL, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("Default Size"), N_("View page at default size"),
		on_fit_clicked, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, GEFAX_ICON_VIEWER_MENU_DEFAULT, 0, 0, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo viewer_action[] =
{
	{
		GNOME_APP_UI_ITEM, N_("Next Page"), N_("View next page"),
		on_next_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_FORWARD, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("Previous Page"), N_("View previous page"),
		on_previous_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BACK, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("First Page"), N_("View first page"),
		on_first_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_FIRST, 0, 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("Last Page"), N_("View last page"),
		on_last_clicked, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_LAST, 0, 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	{
		GNOME_APP_UI_TOGGLEITEM, N_("_High Quality"), N_("View fax page/s in high quality (slower)"),
		on_high_quality_menu_toggled, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN, 0, 0, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo viewer_pop_menu_data[] =
{
	GNOMEUIINFO_SUBTREE (N_("_View"), viewer_action),
	GNOMEUIINFO_SUBTREE (N_("_Zoom"), viewer_view),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_PRINT_ITEM (on_print_clicked, NULL),
	GNOMEUIINFO_SEPARATOR,
	{
		GNOME_APP_UI_ITEM, N_("_Close Viewer"), N_("Close image viewer"),
		change_to_browser, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 0, 0, NULL
	},
	GNOMEUIINFO_END
};


static GnomeUIInfo close_viewer[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Close Viewer"), N_("Close image viewer"),
		change_to_browser, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 0, 0, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo viewer_menu[] =
{
	GNOMEUIINFO_SUBTREE (N_("_View"), viewer_action),
	GNOMEUIINFO_SUBTREE (N_("_Zoom"), viewer_view),
	GNOMEUIINFO_END
};


static void
insert_viwer_menu					(void)
{
	gnome_app_insert_menus (GNOME_APP (gefax), "File/", close_viewer);
	gnome_app_insert_menus (GNOME_APP (gefax), "File", viewer_menu);

	/*menu hints*/
	gnome_app_install_menu_hints (GNOME_APP (gefax), close_viewer);
	gnome_app_install_menu_hints (GNOME_APP (gefax), viewer_menu);

}
static void
remove_viewer_menu					(void)
{
	gnome_app_remove_menus (GNOME_APP (gefax), "File/Close Viewer", 1);
	gnome_app_remove_menus (GNOME_APP (gefax), "View", 2);

}

static GtkWidget
*create_viewer_pop_menu					(void)
{
	GtkWidget *viewer_pop_menu;

	viewer_pop_menu = gtk_menu_new ();
	gtk_object_set_data (GTK_OBJECT (viewer_pop_menu), "viewer_pop_menu", viewer_pop_menu);
	gnome_app_fill_menu (GTK_MENU_SHELL (viewer_pop_menu), viewer_pop_menu_data, NULL, FALSE, 0);
	gnome_app_install_menu_hints (GNOME_APP (gefax), viewer_pop_menu_data);
	ViewerConfig.pop_menu_quality = viewer_action[5].widget;
	return viewer_pop_menu;
}

static void
create_toolbar_viewer					(void)
{
	
	GtkWidget *tmp_toolbar_icon;
	GtkAccelGroup *accel_group;
	
	accel_group = gtk_accel_group_new ();
	
	ViewerToolbar = g_malloc (sizeof (Viewer_Toolbar));
	ViewerToolbar->toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	gtk_widget_show (ViewerToolbar->toolbar);
  
	gtk_container_add (GTK_CONTAINER (toolbar_dock),ViewerToolbar->toolbar);
	gtk_container_set_border_width (GTK_CONTAINER (ViewerToolbar->toolbar), 2);
	gtk_toolbar_set_space_style (GTK_TOOLBAR (ViewerToolbar->toolbar), GTK_TOOLBAR_SPACE_LINE);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (ViewerToolbar->toolbar), GTK_RELIEF_NONE);
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_PRINT);
	ViewerToolbar->print = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, "", _("Print Fax"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->print);

	while (gtk_events_pending()) gtk_main_iteration();
	  
	gtk_toolbar_append_space (GTK_TOOLBAR (ViewerToolbar->toolbar));
	  
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_FIRST);
	ViewerToolbar->first = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, "", _("First Page"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->first);
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_BACK);
	ViewerToolbar->previous = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, _(""), _("Previous Page"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->previous);
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_FORWARD);
	ViewerToolbar->next = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, _(""), _("Next_page"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->next);
	
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_LAST);
	ViewerToolbar->last = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, _(""), _("Last Page"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->last);
	
	gtk_toolbar_append_space (GTK_TOOLBAR (ViewerToolbar->toolbar));
	  
	tmp_toolbar_icon = create_pixmap (gefax, GEFAX_ICON_VIEWER_TOOL_INCREASE, TRUE);
	ViewerToolbar->increase = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, _(""), _("Zoom In"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->increase);  
	tmp_toolbar_icon = create_pixmap (gefax, GEFAX_ICON_VIEWER_TOOL_DECREASE, TRUE);
	ViewerToolbar->decrease = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, _(""), _("Zoom Out"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->decrease);
	
	tmp_toolbar_icon = create_pixmap (gefax, GEFAX_ICON_VIEWER_TOOL_FULL, TRUE);
	ViewerToolbar->full = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, _(""), _("Full Size"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->full);
	
	tmp_toolbar_icon = create_pixmap (gefax, GEFAX_ICON_VIEWER_TOOL_DEFAULT, TRUE);
	ViewerToolbar->fit = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, _(""), _("Default Size"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->fit);
	  
	gtk_toolbar_append_space (GTK_TOOLBAR (ViewerToolbar->toolbar));
	  
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_HELP);
	ViewerToolbar->help = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, _(""), _("Help"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->help);
	gtk_widget_add_accelerator (ViewerToolbar->help, "clicked", accel_group, GDK_F1, 0, GTK_ACCEL_VISIBLE);
	
	gtk_toolbar_append_space (GTK_TOOLBAR (ViewerToolbar->toolbar));
	  
	tmp_toolbar_icon = gnome_stock_pixmap_widget (gefax, GNOME_STOCK_PIXMAP_CLOSE);
 	ViewerToolbar->close = gtk_toolbar_append_element (GTK_TOOLBAR (ViewerToolbar->toolbar),
 			GTK_TOOLBAR_CHILD_BUTTON, NULL, _(""), _("Close"), NULL, tmp_toolbar_icon, NULL, NULL);
	gtk_widget_show (ViewerToolbar->close);
	  
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->help), "clicked",
				GTK_SIGNAL_FUNC (on_help_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->print), "clicked",
				GTK_SIGNAL_FUNC (on_print_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->first), "clicked",
				GTK_SIGNAL_FUNC (on_first_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->previous), "clicked",
				GTK_SIGNAL_FUNC (on_previous_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->next), "clicked",
				GTK_SIGNAL_FUNC (on_next_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->last), "clicked",
				GTK_SIGNAL_FUNC (on_last_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->help), "clicked",
				GTK_SIGNAL_FUNC (on_help_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->close), "clicked",
				GTK_SIGNAL_FUNC (change_to_browser), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->decrease), "clicked",
				GTK_SIGNAL_FUNC (on_decrease_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->increase), "clicked",
				GTK_SIGNAL_FUNC (on_increase_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->full), "clicked",
				GTK_SIGNAL_FUNC (on_full_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (ViewerToolbar->fit), "clicked",
				GTK_SIGNAL_FUNC (on_fit_clicked), NULL);
	gtk_window_add_accel_group (GTK_WINDOW (gefax), accel_group);
}

static void
destroy_toolbar_viewer					(void)
{
	gtk_widget_destroy (ViewerToolbar->toolbar);
	ViewerToolbar->toolbar = NULL;
	g_free (ViewerToolbar);
	ViewerToolbar = NULL;
}

void
remove_image_viewer					(void)
{
	
	/*destroy image viewer*/
	gtk_widget_destroy (fax_viewer);
	fax_viewer = NULL;

	/*remove viewer toolbar*/
	remove_viewer_menu ();
	reset_cursor ();
	gtk_widget_destroy (pop_menu);
	pop_menu = NULL;
	destroy_toolbar_viewer ();
	if (viewer_directory) {
		g_free (viewer_directory);
	}

	
	return;
}

void
viewer_load_directory					(const gchar		*dir)
{
	g_return_if_fail (dir != NULL);

	viewer_directory = g_strdup (dir);
	lib_gefax_fax_viewer_set_scale (LIBGEFAX_FAX_VIEWER (fax_viewer), ViewerConfig.scale);
	lib_gefax_fax_viewer_load_directory (LIBGEFAX_FAX_VIEWER (fax_viewer), dir);
	reset_progress ();
	reset_cursor ();
}

void
create_image_viewer					(GList			*FileList)
{	

	ActiveComponent = VIEWER_ON;
	
	create_toolbar_viewer ();

	/*create viewer in main window  */
	fax_viewer = lib_gefax_fax_viewer_new ();
	gefax_app_set_contents (fax_viewer);
	gtk_widget_show (fax_viewer);
	if (ViewerConfig.high_quality) {
		lib_gefax_fax_viewer_set_interp_type (LIBGEFAX_FAX_VIEWER (fax_viewer), GDK_INTERP_BILINEAR);
	} else {
		lib_gefax_fax_viewer_set_interp_type (LIBGEFAX_FAX_VIEWER (fax_viewer), GDK_INTERP_NEAREST);
	}

	gtk_signal_connect (GTK_OBJECT (fax_viewer), "page_changed",
				GTK_SIGNAL_FUNC (on_page_changed), NULL);
	
	/*pop up menu*/
	pop_menu = create_viewer_pop_menu ();
	gtk_signal_connect (GTK_OBJECT (fax_viewer), "right_click",
 				GTK_SIGNAL_FUNC (on_right_click), NULL);
	insert_viwer_menu ();
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (viewer_action[5].widget), ViewerConfig.high_quality);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ViewerConfig.pop_menu_quality), ViewerConfig.high_quality);
	if (FileList) {
		/*This is a nasty hack untill printing is re - written!						****FIX***/
		gchar *tmp = g_dirname (FileList->data);
		viewer_directory = g_strdup_printf ("%s/", tmp);
		g_free (tmp);

		/*Load the file list & view first page without holding up*/
		while (gtk_events_pending ()) {
			gtk_main_iteration ();
		}
		lib_gefax_fax_viewer_set_scale (LIBGEFAX_FAX_VIEWER (fax_viewer), ViewerConfig.scale);
		lib_gefax_fax_viewer_load_page_list (LIBGEFAX_FAX_VIEWER (fax_viewer), FileList);
	} else {
		/*show default untill loaded*/
		lib_gefax_fax_viewer_show_png_file (LIBGEFAX_FAX_VIEWER (fax_viewer), GEFAX_ICON_VIEWER_OOPS);
	}
	return;
}

void
fax_viewer_load_yourself				(void)
{
	gnome_config_push_prefix ("Gefax-FaxViewer/");

	ViewerConfig.high_quality = gnome_config_get_bool ("UserConfig/HighQuality=TRUE");
	ViewerConfig.scale = gnome_config_get_float ("UserConfig/ImageScale=0.35");
	
	gnome_config_sync ();
	gnome_config_pop_prefix ();

}

void
fax_viewer_save_yourself				(void)
{

	gnome_config_push_prefix ("Gefax-FaxViewer/");

	gnome_config_set_bool ("UserConfig/HighQuality", ViewerConfig.high_quality);
	gnome_config_set_float ("UserConfig/ImageScale", ViewerConfig.scale);

	gnome_config_sync ();
	gnome_config_pop_prefix ();
}

static void
on_page_changed						(LibGefaxFaxViewer	*fv,
							gint			page,
							gpointer		data)
{
	gint total_pages;
	gchar *msg;
	
	total_pages = lib_gefax_fax_viewer_get_pages (fv);
	
	msg = g_strdup_printf (_("Preview: Page %d of %d"), page, total_pages);
	static_status_msg (msg);
	g_free (msg);
	
}

static void
on_right_click						(GtkWidget		*w,
							guint			button,
							guint32			activate_time)
{
	gtk_menu_popup (GTK_MENU (pop_menu), NULL, NULL, NULL, NULL, button, activate_time);
}

static void
on_print_clicked					(GtkWidget		*w,
							gpointer		data)
{
	if (viewer_directory) {
		print_fax_pages (viewer_directory, FALSE);
	}
}

static void
on_first_clicked					(GtkWidget		*w,
							gpointer		data)
{
	lib_gefax_fax_viewer_show_first_page (NULL, LIBGEFAX_FAX_VIEWER (fax_viewer));
}

static void
on_previous_clicked					(GtkWidget		*w,
							gpointer		data)
{
	lib_gefax_fax_viewer_show_previous_page (NULL, LIBGEFAX_FAX_VIEWER (fax_viewer));
}

static void
on_next_clicked						(GtkWidget		*w,
							gpointer		data)
{
	lib_gefax_fax_viewer_show_next_page (NULL, LIBGEFAX_FAX_VIEWER (fax_viewer));
}

static void
on_last_clicked						(GtkWidget		*w,
							gpointer		data)
{
	lib_gefax_fax_viewer_show_last_page (NULL, LIBGEFAX_FAX_VIEWER (fax_viewer));
}

static void
on_high_quality_menu_toggled				(GtkMenuItem		*tb,
							gpointer		data)
{
	if (GTK_CHECK_MENU_ITEM (tb)->active != ViewerConfig.high_quality) {
		if (GTK_CHECK_MENU_ITEM (tb)->active) {
			ViewerConfig.high_quality = TRUE;
			lib_gefax_fax_viewer_set_interp_type (LIBGEFAX_FAX_VIEWER (fax_viewer), GDK_INTERP_BILINEAR);
		} else {
			lib_gefax_fax_viewer_set_interp_type (LIBGEFAX_FAX_VIEWER (fax_viewer), GDK_INTERP_NEAREST);
			ViewerConfig.high_quality = FALSE;
		}
		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (viewer_action[5].widget), ViewerConfig.high_quality);
		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ViewerConfig.pop_menu_quality), ViewerConfig.high_quality);
	}
}

static void
on_increase_clicked					(GtkWidget		*w,
							gpointer		data)
{
	gdouble scale;
	
	scale = lib_gefax_fax_viewer_get_scale (LIBGEFAX_FAX_VIEWER (fax_viewer));
	scale += 0.05;
	if (scale < LIBGEFAX_FAX_VIEWER_SCALE_MAX) {
		lib_gefax_fax_viewer_set_scale (LIBGEFAX_FAX_VIEWER (fax_viewer), scale);
		ViewerConfig.scale = scale;
	}	
}

static void
on_decrease_clicked					(GtkWidget		*w,
							gpointer		data)
{
	gdouble scale;
	
	scale = lib_gefax_fax_viewer_get_scale (LIBGEFAX_FAX_VIEWER (fax_viewer));
	scale -= 0.05;
	if (scale > LIBGEFAX_FAX_VIEWER_SCALE_MIN) {
		lib_gefax_fax_viewer_set_scale (LIBGEFAX_FAX_VIEWER (fax_viewer), scale);
		ViewerConfig.scale = scale;
	}
}

static void
on_full_clicked						(GtkWidget		*w,
							gpointer		data)
{
	lib_gefax_fax_viewer_set_scale (LIBGEFAX_FAX_VIEWER (fax_viewer), 1.0);
	ViewerConfig.scale = 1.0;
}

static void
on_fit_clicked						(GtkWidget		*w,
							gpointer		data)
{
	lib_gefax_fax_viewer_set_scale (LIBGEFAX_FAX_VIEWER (fax_viewer), 0.35);
	ViewerConfig.scale = 0.35;
}

static void
on_help_clicked						(GtkWidget		*w,
							gpointer		data)
{
	gnome_help_display ("ignored", &view_help_ref);
}
